#include <linux/ctype.h>
#include "ext3.h"

unsigned int get_environment_level() {
	unsigned int level;
	printk(KERN_WARNING "STARTED get_environment_level\n");
	mutex_lock(&ext3_env_level->lock);
	level = ext3_env_level->level;
	mutex_unlock(&ext3_env_level->lock);
	printk(KERN_WARNING "FINISHED get_environment_level\n");
	return level;
}

//TODO: change to allow multidigit numbers and multidigit strings
//Remove simple way of thinking - do recursive searches to prevent namespace colisions from user input
int ext3_is_dummy_file_p(const char *name) {
	int i;
	printk(KERN_WARNING "_ENTERING IS DUMMY FILE P (%s)\n", name);
	for (i = 0; name[i] != '\0'; ++i) {
		if (name[i] == *(EXT3_SECURE_STRING) && name[i+1] != '\0' && isdigit(name[i+1]) && name[i+2] == '\0') {
			printk(KERN_WARNING "_IS NOT DUMMY FILE\n");
			return 0;
		}
	}
	printk(KERN_WARNING "_IS DUMMY FILE\n");
	return 1;
}

char* ext3_generate_filename_from_dummy_file(struct dentry *dentry, unsigned int level) {
	char *filename;
	printk(KERN_WARNING "STARTED ext3_generate_filename_from_dummy_file\n");
	do {
		 filename = (char *)__get_free_page(GFP_USER);
	} while (filename == NULL);
	memset((void *)filename, 0, PAGE_SIZE);
	sprintf(filename, "%s%s%d", dentry->d_name.name, EXT3_SECURE_STRING, level);
	printk(KERN_WARNING "FINISHED ext3_generate_filename_from_dummy_file\n");
	return filename;
}

char* ext3_generate_fullname_from_dummy_file(struct dentry *dentry, unsigned int level) {
	char *name;
	char *filename;
	char *buf;
	
	printk(KERN_WARNING "STARTED ext3_generate_fullname_from_dummy_file\n");
	do {
		filename = (char *)__get_free_page(GFP_USER);
	} while (filename == NULL);
	do {
		 buf = (char *)__get_free_page(GFP_USER);
	} while (buf == NULL);
	memset((void *)filename, 0, PAGE_SIZE);
	mutex_lock(&ext3_env_level->lock);
	if (ext3_env_level->mount_size) {
		strcat(filename, ext3_env_level->mount_point);
	}
	mutex_unlock(&ext3_env_level->lock);
	name = dentry_path_raw(dentry, buf, PAGE_SIZE);
	sprintf(filename, "%s%s%s%d", filename, name, EXT3_SECURE_STRING, level);
	free_page((unsigned long) buf);
	printk(KERN_WARNING "FINISHED ext3_generate_fullname_from_dummy_file\n");
	return filename;
}

struct file* ext3_open_file_ptr_from_dummy_file(struct file *filp, unsigned int level) {	//Don't forget to call filp_close
	struct file *ret;
	unsigned int flags;
	umode_t mode;
	char *filename;
	
	printk(KERN_WARNING "STARTED ext3_open_file_ptr_from_dummy_file\n");
	filename = ext3_generate_fullname_from_dummy_file(filp->f_dentry, level);	
	if (filename == 0) {
		return NULL;
	}
	flags = filp->f_flags;
	mode = filp->f_mapping->host->i_mode;	
		
	ret = filp_open(filename, flags, mode);
	//free_page((unsigned long) filename);
	printk(KERN_WARNING "FINISHED ext3_open_file_ptr_from_dummy_file\n");
	return ret;
}
