/*								/*
 *  linux/fs/ext3/dir.c						 *  linux/fs/ext3/dir.c
 *								 *
 * Copyright (C) 1992, 1993, 1994, 1995				 * Copyright (C) 1992, 1993, 1994, 1995
 * Remy Card (card@masi.ibp.fr)					 * Remy Card (card@masi.ibp.fr)
 * Laboratoire MASI - Institut Blaise Pascal			 * Laboratoire MASI - Institut Blaise Pascal
 * Universite Pierre et Marie Curie (Paris VI)			 * Universite Pierre et Marie Curie (Paris VI)
 *								 *
 *  from							 *  from
 *								 *
 *  linux/fs/minix/dir.c					 *  linux/fs/minix/dir.c
 *								 *
 *  Copyright (C) 1991, 1992  Linus Torvalds			 *  Copyright (C) 1991, 1992  Linus Torvalds
 *								 *
 *  ext3 directory handling functions				 *  ext3 directory handling functions
 *								 *
 *  Big-endian to little-endian byte-swapping/bitmaps by	 *  Big-endian to little-endian byte-swapping/bitmaps by
 *        David S. Miller (davem@caip.rutgers.edu), 1995	 *        David S. Miller (davem@caip.rutgers.edu), 1995
 *								 *
 * Hash Tree Directory indexing (c) 2001  Daniel Phillips	 * Hash Tree Directory indexing (c) 2001  Daniel Phillips
 *								 *
 */								 */

#include <linux/compat.h>					#include <linux/compat.h>
#include "ext3.h"						#include "ext3.h"

static unsigned char ext3_filetype_table[] = {			static unsigned char ext3_filetype_table[] = {
	DT_UNKNOWN, DT_REG, DT_DIR, DT_CHR, DT_BLK, DT_FIFO, 		DT_UNKNOWN, DT_REG, DT_DIR, DT_CHR, DT_BLK, DT_FIFO, 
};								};

static int ext3_dx_readdir(struct file * filp,			static int ext3_dx_readdir(struct file * filp,
			   void * dirent, filldir_t filldir);				   void * dirent, filldir_t filldir);

static unsigned char get_dtype(struct super_block *sb, int fi	static unsigned char get_dtype(struct super_block *sb, int fi
{								{
	if (!EXT3_HAS_INCOMPAT_FEATURE(sb, EXT3_FEATURE_INCOM		if (!EXT3_HAS_INCOMPAT_FEATURE(sb, EXT3_FEATURE_INCOM
	    (filetype >= EXT3_FT_MAX))					    (filetype >= EXT3_FT_MAX))
		return DT_UNKNOWN;						return DT_UNKNOWN;

	return (ext3_filetype_table[filetype]);				return (ext3_filetype_table[filetype]);
}								}

/**								/**
 * Check if the given dir-inode refers to an htree-indexed di	 * Check if the given dir-inode refers to an htree-indexed di
 * (or a directory which chould potentially get coverted to u	 * (or a directory which chould potentially get coverted to u
 * indexing).							 * indexing).
 *								 *
 * Return 1 if it is a dx dir, 0 if not				 * Return 1 if it is a dx dir, 0 if not
 */								 */
static int is_dx_dir(struct inode *inode)			static int is_dx_dir(struct inode *inode)
{								{
	struct super_block *sb = inode->i_sb;				struct super_block *sb = inode->i_sb;

	if (EXT3_HAS_COMPAT_FEATURE(inode->i_sb,			if (EXT3_HAS_COMPAT_FEATURE(inode->i_sb,
		     EXT3_FEATURE_COMPAT_DIR_INDEX) &&				     EXT3_FEATURE_COMPAT_DIR_INDEX) &&
	    ((EXT3_I(inode)->i_flags & EXT3_INDEX_FL) ||		    ((EXT3_I(inode)->i_flags & EXT3_INDEX_FL) ||
	     ((inode->i_size >> sb->s_blocksize_bits) == 1)))		     ((inode->i_size >> sb->s_blocksize_bits) == 1)))
		return 1;							return 1;

	return 0;							return 0;
}								}

int ext3_check_dir_entry (const char * function, struct inode	int ext3_check_dir_entry (const char * function, struct inode
			  struct ext3_dir_entry_2 * de,					  struct ext3_dir_entry_2 * de,
			  struct buffer_head * bh,					  struct buffer_head * bh,
			  unsigned long offset)						  unsigned long offset)
{								{
	const char * error_msg = NULL;					const char * error_msg = NULL;
	const int rlen = ext3_rec_len_from_disk(de->rec_len);		const int rlen = ext3_rec_len_from_disk(de->rec_len);

	if (unlikely(rlen < EXT3_DIR_REC_LEN(1)))			if (unlikely(rlen < EXT3_DIR_REC_LEN(1)))
		error_msg = "rec_len is smaller than minimal"			error_msg = "rec_len is smaller than minimal"
	else if (unlikely(rlen % 4 != 0))				else if (unlikely(rlen % 4 != 0))
		error_msg = "rec_len % 4 != 0";					error_msg = "rec_len % 4 != 0";
	else if (unlikely(rlen < EXT3_DIR_REC_LEN(de->name_le		else if (unlikely(rlen < EXT3_DIR_REC_LEN(de->name_le
		error_msg = "rec_len is too small for name_le			error_msg = "rec_len is too small for name_le
	else if (unlikely((((char *) de - bh->b_data) + rlen 		else if (unlikely((((char *) de - bh->b_data) + rlen 
		error_msg = "directory entry across blocks";			error_msg = "directory entry across blocks";
	else if (unlikely(le32_to_cpu(de->inode) >			else if (unlikely(le32_to_cpu(de->inode) >
			le32_to_cpu(EXT3_SB(dir->i_sb)->s_es-				le32_to_cpu(EXT3_SB(dir->i_sb)->s_es-
		error_msg = "inode out of bounds";				error_msg = "inode out of bounds";

	if (unlikely(error_msg != NULL))				if (unlikely(error_msg != NULL))
		ext3_error (dir->i_sb, function,				ext3_error (dir->i_sb, function,
			"bad entry in directory #%lu: %s - "				"bad entry in directory #%lu: %s - "
			"offset=%lu, inode=%lu, rec_len=%d, n				"offset=%lu, inode=%lu, rec_len=%d, n
			dir->i_ino, error_msg, offset,					dir->i_ino, error_msg, offset,
			(unsigned long) le32_to_cpu(de->inode				(unsigned long) le32_to_cpu(de->inode
			rlen, de->name_len);						rlen, de->name_len);

	return error_msg == NULL ? 1 : 0;				return error_msg == NULL ? 1 : 0;
}								}

static int ext3_readdir(struct file * filp,			static int ext3_readdir(struct file * filp,
			 void * dirent, filldir_t filldir)				 void * dirent, filldir_t filldir)
{								{
	int error = 0;							int error = 0;
	unsigned long offset;						unsigned long offset;
	int i, stored;							int i, stored;
	struct ext3_dir_entry_2 *de;					struct ext3_dir_entry_2 *de;
	int err;							int err;
	struct inode *inode = filp->f_path.dentry->d_inode;		struct inode *inode = filp->f_path.dentry->d_inode;
	struct super_block *sb = inode->i_sb;				struct super_block *sb = inode->i_sb;
	int ret = 0;							int ret = 0;
	int dir_has_error = 0;						int dir_has_error = 0;

	if (is_dx_dir(inode)) {						if (is_dx_dir(inode)) {
		err = ext3_dx_readdir(filp, dirent, filldir);			err = ext3_dx_readdir(filp, dirent, filldir);
		if (err != ERR_BAD_DX_DIR) {					if (err != ERR_BAD_DX_DIR) {
			ret = err;							ret = err;
			goto out;							goto out;
		}								}
		/*								/*
		 * We don't set the inode dirty flag since it			 * We don't set the inode dirty flag since it
		 * critical that it get flushed back to the d			 * critical that it get flushed back to the d
		 */								 */
		EXT3_I(filp->f_path.dentry->d_inode)->i_flags			EXT3_I(filp->f_path.dentry->d_inode)->i_flags
	}								}
	stored = 0;							stored = 0;
	offset = filp->f_pos & (sb->s_blocksize - 1);			offset = filp->f_pos & (sb->s_blocksize - 1);

	while (!error && !stored && filp->f_pos < inode->i_si		while (!error && !stored && filp->f_pos < inode->i_si
		unsigned long blk = filp->f_pos >> EXT3_BLOCK			unsigned long blk = filp->f_pos >> EXT3_BLOCK
		struct buffer_head map_bh;					struct buffer_head map_bh;
		struct buffer_head *bh = NULL;					struct buffer_head *bh = NULL;

		map_bh.b_state = 0;						map_bh.b_state = 0;
		err = ext3_get_blocks_handle(NULL, inode, blk			err = ext3_get_blocks_handle(NULL, inode, blk
		if (err > 0) {							if (err > 0) {
			pgoff_t index = map_bh.b_blocknr >>				pgoff_t index = map_bh.b_blocknr >>
					(PAGE_CACHE_SHIFT - i						(PAGE_CACHE_SHIFT - i
			if (!ra_has_index(&filp->f_ra, index)				if (!ra_has_index(&filp->f_ra, index)
				page_cache_sync_readahead(					page_cache_sync_readahead(
					sb->s_bdev->bd_inode-						sb->s_bdev->bd_inode-
					&filp->f_ra, filp,						&filp->f_ra, filp,
					index, 1);							index, 1);
			filp->f_ra.prev_pos = (loff_t)index <				filp->f_ra.prev_pos = (loff_t)index <
			bh = ext3_bread(NULL, inode, blk, 0, 				bh = ext3_bread(NULL, inode, blk, 0, 
		}								}

		/*								/*
		 * We ignore I/O errors on directories so use			 * We ignore I/O errors on directories so use
		 * of recovering data when there's a bad sect			 * of recovering data when there's a bad sect
		 */								 */
		if (!bh) {							if (!bh) {
			if (!dir_has_error) {						if (!dir_has_error) {
				ext3_error(sb, __func__, "dir					ext3_error(sb, __func__, "dir
					"contains a hole at o						"contains a hole at o
					inode->i_ino, filp->f						inode->i_ino, filp->f
				dir_has_error = 1;						dir_has_error = 1;
			}								}
			/* corrupt size?  Maybe no more block				/* corrupt size?  Maybe no more block
			if (filp->f_pos > inode->i_blocks << 				if (filp->f_pos > inode->i_blocks << 
				break;								break;
			filp->f_pos += sb->s_blocksize - offs				filp->f_pos += sb->s_blocksize - offs
			continue;							continue;
		}								}

revalidate:							revalidate:
		/* If the dir block has changed since the las			/* If the dir block has changed since the las
		 * readdir(2), then we might be pointing to a			 * readdir(2), then we might be pointing to a
		 * dirent right now.  Scan from the start of 			 * dirent right now.  Scan from the start of 
		 * to make sure. */						 * to make sure. */
		if (filp->f_version != inode->i_version) {			if (filp->f_version != inode->i_version) {
			for (i = 0; i < sb->s_blocksize && i 				for (i = 0; i < sb->s_blocksize && i 
				de = (struct ext3_dir_entry_2					de = (struct ext3_dir_entry_2
					(bh->b_data + i);						(bh->b_data + i);
				/* It's too expensive to do a					/* It's too expensive to do a
				 * dirent test each time roun					 * dirent test each time roun
				 * loop, but we do have to te					 * loop, but we do have to te
				 * least that it is non-zero.					 * least that it is non-zero.
				 * failure will be detected i					 * failure will be detected i
				 * dirent test below. */					 * dirent test below. */
				if (ext3_rec_len_from_disk(de					if (ext3_rec_len_from_disk(de
						EXT3_DIR_REC_							EXT3_DIR_REC_
					break;								break;
				i += ext3_rec_len_from_disk(d					i += ext3_rec_len_from_disk(d
			}								}
			offset = i;							offset = i;
			filp->f_pos = (filp->f_pos & ~(sb->s_				filp->f_pos = (filp->f_pos & ~(sb->s_
				| offset;							| offset;
			filp->f_version = inode->i_version;				filp->f_version = inode->i_version;
		}								}

		while (!error && filp->f_pos < inode->i_size			while (!error && filp->f_pos < inode->i_size
		       && offset < sb->s_blocksize) {				       && offset < sb->s_blocksize) {
			de = (struct ext3_dir_entry_2 *) (bh-				de = (struct ext3_dir_entry_2 *) (bh-
			if (!ext3_check_dir_entry ("ext3_read				if (!ext3_check_dir_entry ("ext3_read
						   bh, offset							   bh, offset
				/* On error, skip the f_pos t					/* On error, skip the f_pos t
                                   next block. */		                                   next block. */
				filp->f_pos = (filp->f_pos |					filp->f_pos = (filp->f_pos |
						(sb->s_blocks							(sb->s_blocks
				brelse (bh);							brelse (bh);
				ret = stored;							ret = stored;
				goto out;							goto out;
			}								}
			offset += ext3_rec_len_from_disk(de->				offset += ext3_rec_len_from_disk(de->
							      <
			if (le32_to_cpu(de->inode)) {					if (le32_to_cpu(de->inode)) {
				/* We might block in the next					/* We might block in the next
				 * if the data destination is					 * if the data destination is
				 * currently swapped out.  So					 * currently swapped out.  So
				 * version stamp to detect wh					 * version stamp to detect wh
				 * not the directory has been					 * not the directory has been
				 * during the copy operation.					 * during the copy operation.
				 */								 */
				u64 version = filp->f_version					u64 version = filp->f_version

				error = filldir(dirent, de->n					error = filldir(dirent, de->n
						de->name_len,							de->name_len,
						filp->f_pos,							filp->f_pos,
						le32_to_cpu(d							le32_to_cpu(d
						get_dtype(sb,							get_dtype(sb,
				if (error)							if (error)
					break;								break;
				if (version != filp->f_versio					if (version != filp->f_versio
					goto revalidate;						goto revalidate;
				stored ++;							stored ++;
			}								}
			filp->f_pos += ext3_rec_len_from_disk				filp->f_pos += ext3_rec_len_from_disk
		}								}
		offset = 0;							offset = 0;
		brelse (bh);							brelse (bh);
	}								}
out:								out:
	return ret;							return ret;
}								}

static inline int is_32bit_api(void)				static inline int is_32bit_api(void)
{								{
#ifdef CONFIG_COMPAT						#ifdef CONFIG_COMPAT
	return is_compat_task();					return is_compat_task();
#else								#else
	return (BITS_PER_LONG == 32);					return (BITS_PER_LONG == 32);
#endif								#endif
}								}

/*								/*
 * These functions convert from the major/minor hash to an f_	 * These functions convert from the major/minor hash to an f_
 * value for dx directories					 * value for dx directories
 *								 *
 * Upper layer (for example NFS) should specify FMODE_32BITHA	 * Upper layer (for example NFS) should specify FMODE_32BITHA
 * FMODE_64BITHASH explicitly. On the other hand, we allow ex	 * FMODE_64BITHASH explicitly. On the other hand, we allow ex
 * directly on both 32-bit and 64-bit nodes, under such case,	 * directly on both 32-bit and 64-bit nodes, under such case,
 * FMODE_32BITHASH nor FMODE_64BITHASH is specified.		 * FMODE_32BITHASH nor FMODE_64BITHASH is specified.
 */								 */
static inline loff_t hash2pos(struct file *filp, __u32 major,	static inline loff_t hash2pos(struct file *filp, __u32 major,
{								{
	if ((filp->f_mode & FMODE_32BITHASH) ||				if ((filp->f_mode & FMODE_32BITHASH) ||
	    (!(filp->f_mode & FMODE_64BITHASH) && is_32bit_ap		    (!(filp->f_mode & FMODE_64BITHASH) && is_32bit_ap
		return major >> 1;						return major >> 1;
	else								else
		return ((__u64)(major >> 1) << 32) | (__u64)m			return ((__u64)(major >> 1) << 32) | (__u64)m
}								}

static inline __u32 pos2maj_hash(struct file *filp, loff_t po	static inline __u32 pos2maj_hash(struct file *filp, loff_t po
{								{
	if ((filp->f_mode & FMODE_32BITHASH) ||				if ((filp->f_mode & FMODE_32BITHASH) ||
	    (!(filp->f_mode & FMODE_64BITHASH) && is_32bit_ap		    (!(filp->f_mode & FMODE_64BITHASH) && is_32bit_ap
		return (pos << 1) & 0xffffffff;					return (pos << 1) & 0xffffffff;
	else								else
		return ((pos >> 32) << 1) & 0xffffffff;				return ((pos >> 32) << 1) & 0xffffffff;
}								}

static inline __u32 pos2min_hash(struct file *filp, loff_t po	static inline __u32 pos2min_hash(struct file *filp, loff_t po
{								{
	if ((filp->f_mode & FMODE_32BITHASH) ||				if ((filp->f_mode & FMODE_32BITHASH) ||
	    (!(filp->f_mode & FMODE_64BITHASH) && is_32bit_ap		    (!(filp->f_mode & FMODE_64BITHASH) && is_32bit_ap
		return 0;							return 0;
	else								else
		return pos & 0xffffffff;					return pos & 0xffffffff;
}								}

/*								/*
 * Return 32- or 64-bit end-of-file for dx directories		 * Return 32- or 64-bit end-of-file for dx directories
 */								 */
static inline loff_t ext3_get_htree_eof(struct file *filp)	static inline loff_t ext3_get_htree_eof(struct file *filp)
{								{
	if ((filp->f_mode & FMODE_32BITHASH) ||				if ((filp->f_mode & FMODE_32BITHASH) ||
	    (!(filp->f_mode & FMODE_64BITHASH) && is_32bit_ap		    (!(filp->f_mode & FMODE_64BITHASH) && is_32bit_ap
		return EXT3_HTREE_EOF_32BIT;					return EXT3_HTREE_EOF_32BIT;
	else								else
		return EXT3_HTREE_EOF_64BIT;					return EXT3_HTREE_EOF_64BIT;
}								}


/*								/*
 * ext3_dir_llseek() calls generic_file_llseek[_size]() to ha	 * ext3_dir_llseek() calls generic_file_llseek[_size]() to ha
 * non-htree and htree directories, where the "offset" is in 	 * non-htree and htree directories, where the "offset" is in 
 * of the filename hash value instead of the byte offset.	 * of the filename hash value instead of the byte offset.
 *								 *
 * Because we may return a 64-bit hash that is well beyond s_	 * Because we may return a 64-bit hash that is well beyond s_
 * we need to pass the max hash as the maximum allowable offs	 * we need to pass the max hash as the maximum allowable offs
 * the htree directory case.					 * the htree directory case.
 *								 *
 * NOTE: offsets obtained *before* ext3_set_inode_flag(dir, E	 * NOTE: offsets obtained *before* ext3_set_inode_flag(dir, E
 *       will be invalid once the directory was converted int	 *       will be invalid once the directory was converted int
 */								 */
loff_t ext3_dir_llseek(struct file *file, loff_t offset, int 	loff_t ext3_dir_llseek(struct file *file, loff_t offset, int 
{								{
	struct inode *inode = file->f_mapping->host;			struct inode *inode = file->f_mapping->host;
	int dx_dir = is_dx_dir(inode);					int dx_dir = is_dx_dir(inode);
	loff_t htree_max = ext3_get_htree_eof(file);			loff_t htree_max = ext3_get_htree_eof(file);

	if (likely(dx_dir))						if (likely(dx_dir))
		return generic_file_llseek_size(file, offset,			return generic_file_llseek_size(file, offset,
					        htree_max, ht						        htree_max, ht
	else								else
		return generic_file_llseek(file, offset, when			return generic_file_llseek(file, offset, when
}								}

/*								/*
 * This structure holds the nodes of the red-black tree used 	 * This structure holds the nodes of the red-black tree used 
 * the directory entry in hash order.				 * the directory entry in hash order.
 */								 */
struct fname {							struct fname {
	__u32		hash;						__u32		hash;
	__u32		minor_hash;					__u32		minor_hash;
	struct rb_node	rb_hash;					struct rb_node	rb_hash;
	struct fname	*next;						struct fname	*next;
	__u32		inode;						__u32		inode;
	__u8		name_len;					__u8		name_len;
	__u8		file_type;					__u8		file_type;
	char		name[0];					char		name[0];
};								};

/*								/*
 * This functoin implements a non-recursive way of freeing al	 * This functoin implements a non-recursive way of freeing al
 * nodes in the red-black tree.					 * nodes in the red-black tree.
 */								 */
static void free_rb_tree_fname(struct rb_root *root)		static void free_rb_tree_fname(struct rb_root *root)
{								{
	struct rb_node	*n = root->rb_node;				struct rb_node	*n = root->rb_node;
	struct rb_node	*parent;					struct rb_node	*parent;
	struct fname	*fname;						struct fname	*fname;

	while (n) {							while (n) {
		/* Do the node's children first */				/* Do the node's children first */
		if (n->rb_left) {						if (n->rb_left) {
			n = n->rb_left;							n = n->rb_left;
			continue;							continue;
		}								}
		if (n->rb_right) {						if (n->rb_right) {
			n = n->rb_right;						n = n->rb_right;
			continue;							continue;
		}								}
		/*								/*
		 * The node has no children; free it, and the			 * The node has no children; free it, and the
		 * out parent's link to it.  Finally go to th			 * out parent's link to it.  Finally go to th
		 * beginning of the loop and try to free the 			 * beginning of the loop and try to free the 
		 * node.							 * node.
		 */								 */
		parent = rb_parent(n);						parent = rb_parent(n);
		fname = rb_entry(n, struct fname, rb_hash);			fname = rb_entry(n, struct fname, rb_hash);
		while (fname) {							while (fname) {
			struct fname * old = fname;					struct fname * old = fname;
			fname = fname->next;						fname = fname->next;
			kfree (old);							kfree (old);
		}								}
		if (!parent)							if (!parent)
			*root = RB_ROOT;						*root = RB_ROOT;
		else if (parent->rb_left == n)					else if (parent->rb_left == n)
			parent->rb_left = NULL;						parent->rb_left = NULL;
		else if (parent->rb_right == n)					else if (parent->rb_right == n)
			parent->rb_right = NULL;					parent->rb_right = NULL;
		n = parent;							n = parent;
	}								}
}								}


static struct dir_private_info *ext3_htree_create_dir_info(st	static struct dir_private_info *ext3_htree_create_dir_info(st
							   lo								   lo
{								{
	struct dir_private_info *p;					struct dir_private_info *p;

	p = kzalloc(sizeof(struct dir_private_info), GFP_KERN		p = kzalloc(sizeof(struct dir_private_info), GFP_KERN
	if (!p)								if (!p)
		return NULL;							return NULL;
	p->curr_hash = pos2maj_hash(filp, pos);				p->curr_hash = pos2maj_hash(filp, pos);
	p->curr_minor_hash = pos2min_hash(filp, pos);			p->curr_minor_hash = pos2min_hash(filp, pos);
	return p;							return p;
}								}

void ext3_htree_free_dir_info(struct dir_private_info *p)	void ext3_htree_free_dir_info(struct dir_private_info *p)
{								{
	free_rb_tree_fname(&p->root);					free_rb_tree_fname(&p->root);
	kfree(p);							kfree(p);
}								}

/*								/*
 * Given a directory entry, enter it into the fname rb tree.	 * Given a directory entry, enter it into the fname rb tree.
 */								 */
int ext3_htree_store_dirent(struct file *dir_file, __u32 hash	int ext3_htree_store_dirent(struct file *dir_file, __u32 hash
			     __u32 minor_hash,						     __u32 minor_hash,
			     struct ext3_dir_entry_2 *dirent)				     struct ext3_dir_entry_2 *dirent)
{								{
	struct rb_node **p, *parent = NULL;				struct rb_node **p, *parent = NULL;
	struct fname * fname, *new_fn;					struct fname * fname, *new_fn;
	struct dir_private_info *info;					struct dir_private_info *info;
	int len;							int len;

	info = (struct dir_private_info *) dir_file->private_		info = (struct dir_private_info *) dir_file->private_
	p = &info->root.rb_node;					p = &info->root.rb_node;

	/* Create and allocate the fname structure */			/* Create and allocate the fname structure */
	len = sizeof(struct fname) + dirent->name_len + 1;		len = sizeof(struct fname) + dirent->name_len + 1;
	new_fn = kzalloc(len, GFP_KERNEL);				new_fn = kzalloc(len, GFP_KERNEL);
	if (!new_fn)							if (!new_fn)
		return -ENOMEM;							return -ENOMEM;
	new_fn->hash = hash;						new_fn->hash = hash;
	new_fn->minor_hash = minor_hash;				new_fn->minor_hash = minor_hash;
	new_fn->inode = le32_to_cpu(dirent->inode);			new_fn->inode = le32_to_cpu(dirent->inode);
	new_fn->name_len = dirent->name_len;				new_fn->name_len = dirent->name_len;
	new_fn->file_type = dirent->file_type;				new_fn->file_type = dirent->file_type;
	memcpy(new_fn->name, dirent->name, dirent->name_len);		memcpy(new_fn->name, dirent->name, dirent->name_len);
	new_fn->name[dirent->name_len] = 0;				new_fn->name[dirent->name_len] = 0;

	while (*p) {							while (*p) {
		parent = *p;							parent = *p;
		fname = rb_entry(parent, struct fname, rb_has			fname = rb_entry(parent, struct fname, rb_has

		/*								/*
		 * If the hash and minor hash match up, then 			 * If the hash and minor hash match up, then 
		 * them on a linked list.  This rarely happen			 * them on a linked list.  This rarely happen
		 */								 */
		if ((new_fn->hash == fname->hash) &&				if ((new_fn->hash == fname->hash) &&
		    (new_fn->minor_hash == fname->minor_hash)			    (new_fn->minor_hash == fname->minor_hash)
			new_fn->next = fname->next;					new_fn->next = fname->next;
			fname->next = new_fn;						fname->next = new_fn;
			return 0;							return 0;
		}								}

		if (new_fn->hash < fname->hash)					if (new_fn->hash < fname->hash)
			p = &(*p)->rb_left;						p = &(*p)->rb_left;
		else if (new_fn->hash > fname->hash)				else if (new_fn->hash > fname->hash)
			p = &(*p)->rb_right;						p = &(*p)->rb_right;
		else if (new_fn->minor_hash < fname->minor_ha			else if (new_fn->minor_hash < fname->minor_ha
			p = &(*p)->rb_left;						p = &(*p)->rb_left;
		else /* if (new_fn->minor_hash > fname->minor			else /* if (new_fn->minor_hash > fname->minor
			p = &(*p)->rb_right;						p = &(*p)->rb_right;
	}								}

	rb_link_node(&new_fn->rb_hash, parent, p);			rb_link_node(&new_fn->rb_hash, parent, p);
	rb_insert_color(&new_fn->rb_hash, &info->root);			rb_insert_color(&new_fn->rb_hash, &info->root);
	return 0;							return 0;
}								}



/*								/*
 * This is a helper function for ext3_dx_readdir.  It calls f	 * This is a helper function for ext3_dx_readdir.  It calls f
 * for all entres on the fname linked list.  (Normally there 	 * for all entres on the fname linked list.  (Normally there 
 * one entry on the linked list, unless there are 62 bit hash	 * one entry on the linked list, unless there are 62 bit hash
 */								 */
static int call_filldir(struct file * filp, void * dirent,	static int call_filldir(struct file * filp, void * dirent,
			filldir_t filldir, struct fname *fnam				filldir_t filldir, struct fname *fnam
{								{
	struct dir_private_info *info = filp->private_data;		struct dir_private_info *info = filp->private_data;
	loff_t	curr_pos;						loff_t	curr_pos;
	struct inode *inode = filp->f_path.dentry->d_inode;		struct inode *inode = filp->f_path.dentry->d_inode;
	struct super_block * sb;					struct super_block * sb;
	int error;							int error;

	sb = inode->i_sb;						sb = inode->i_sb;

	if (!fname) {							if (!fname) {
		printk("call_filldir: called with null fname?			printk("call_filldir: called with null fname?
		return 0;							return 0;
	}								}
	curr_pos = hash2pos(filp, fname->hash, fname->minor_h		curr_pos = hash2pos(filp, fname->hash, fname->minor_h
	while (fname) {							while (fname) {
		error = filldir(dirent, fname->name,				error = filldir(dirent, fname->name,
				fname->name_len, curr_pos,					fname->name_len, curr_pos,
				fname->inode,							fname->inode,
				get_dtype(sb, fname->file_typ					get_dtype(sb, fname->file_typ
		if (error) {							if (error) {
			filp->f_pos = curr_pos;						filp->f_pos = curr_pos;
			info->extra_fname = fname;					info->extra_fname = fname;
			return error;							return error;
		}								}
		fname = fname->next;						fname = fname->next;
	}								}
	return 0;							return 0;
}								}

static int ext3_dx_readdir(struct file * filp,			static int ext3_dx_readdir(struct file * filp,
			 void * dirent, filldir_t filldir)				 void * dirent, filldir_t filldir)
{								{
	struct dir_private_info *info = filp->private_data;		struct dir_private_info *info = filp->private_data;
	struct inode *inode = filp->f_path.dentry->d_inode;		struct inode *inode = filp->f_path.dentry->d_inode;
	struct fname *fname;						struct fname *fname;
	int	ret;							int	ret;

	if (!info) {							if (!info) {
		info = ext3_htree_create_dir_info(filp, filp-			info = ext3_htree_create_dir_info(filp, filp-
		if (!info)							if (!info)
			return -ENOMEM;							return -ENOMEM;
		filp->private_data = info;					filp->private_data = info;
	}								}

	if (filp->f_pos == ext3_get_htree_eof(filp))			if (filp->f_pos == ext3_get_htree_eof(filp))
		return 0;	/* EOF */					return 0;	/* EOF */

	/* Some one has messed with f_pos; reset the world */		/* Some one has messed with f_pos; reset the world */
	if (info->last_pos != filp->f_pos) {				if (info->last_pos != filp->f_pos) {
		free_rb_tree_fname(&info->root);				free_rb_tree_fname(&info->root);
		info->curr_node = NULL;						info->curr_node = NULL;
		info->extra_fname = NULL;					info->extra_fname = NULL;
		info->curr_hash = pos2maj_hash(filp, filp->f_			info->curr_hash = pos2maj_hash(filp, filp->f_
		info->curr_minor_hash = pos2min_hash(filp, fi			info->curr_minor_hash = pos2min_hash(filp, fi
	}								}

	/*								/*
	 * If there are any leftover names on the hash collis		 * If there are any leftover names on the hash collis
	 * chain, return them first.					 * chain, return them first.
	 */								 */
	if (info->extra_fname) {					if (info->extra_fname) {
		if (call_filldir(filp, dirent, filldir, info-			if (call_filldir(filp, dirent, filldir, info-
			goto finished;							goto finished;
		info->extra_fname = NULL;					info->extra_fname = NULL;
		goto next_node;							goto next_node;
	} else if (!info->curr_node)					} else if (!info->curr_node)
		info->curr_node = rb_first(&info->root);			info->curr_node = rb_first(&info->root);

	while (1) {							while (1) {
		/*								/*
		 * Fill the rbtree if we have no more entries			 * Fill the rbtree if we have no more entries
		 * or the inode has changed since we last rea			 * or the inode has changed since we last rea
		 * cached entries.						 * cached entries.
		 */								 */
		if ((!info->curr_node) ||					if ((!info->curr_node) ||
		    (filp->f_version != inode->i_version)) {			    (filp->f_version != inode->i_version)) {
			info->curr_node = NULL;						info->curr_node = NULL;
			free_rb_tree_fname(&info->root);				free_rb_tree_fname(&info->root);
			filp->f_version = inode->i_version;				filp->f_version = inode->i_version;
			ret = ext3_htree_fill_tree(filp, info				ret = ext3_htree_fill_tree(filp, info
						   info->curr							   info->curr
						   &info->nex							   &info->nex
			if (ret < 0)							if (ret < 0)
				return ret;							return ret;
			if (ret == 0) {							if (ret == 0) {
				filp->f_pos = ext3_get_htree_					filp->f_pos = ext3_get_htree_
				break;								break;
			}								}
			info->curr_node = rb_first(&info->roo				info->curr_node = rb_first(&info->roo
		}								}

		fname = rb_entry(info->curr_node, struct fnam			fname = rb_entry(info->curr_node, struct fnam
		if (ext3_is_dummy_file_p(fname->name)) {      |			info->curr_hash = fname->hash;
			info->curr_hash = fname->hash;	      |			info->curr_minor_hash = fname->minor_hash;
			info->curr_minor_hash = fname->minor_ |			if (call_filldir(filp, dirent, filldir, fname
			if (call_filldir(filp, dirent, filldi |				break;
				break;			      <
		}					      <
	next_node:							next_node:
		info->curr_node = rb_next(info->curr_node);			info->curr_node = rb_next(info->curr_node);
		if (info->curr_node) {						if (info->curr_node) {
			fname = rb_entry(info->curr_node, str				fname = rb_entry(info->curr_node, str
					 rb_hash);							 rb_hash);
			info->curr_hash = fname->hash;					info->curr_hash = fname->hash;
			info->curr_minor_hash = fname->minor_				info->curr_minor_hash = fname->minor_
		} else {							} else {
			if (info->next_hash == ~0) {					if (info->next_hash == ~0) {
				filp->f_pos = ext3_get_htree_					filp->f_pos = ext3_get_htree_
				break;								break;
			}								}
			info->curr_hash = info->next_hash;				info->curr_hash = info->next_hash;
			info->curr_minor_hash = 0;					info->curr_minor_hash = 0;
		}								}
	}								}
finished:							finished:
	info->last_pos = filp->f_pos;					info->last_pos = filp->f_pos;
	return 0;							return 0;
}								}

static int ext3_release_dir (struct inode * inode, struct fil	static int ext3_release_dir (struct inode * inode, struct fil
{								{
       if (filp->private_data)					       if (filp->private_data)
		ext3_htree_free_dir_info(filp->private_data);			ext3_htree_free_dir_info(filp->private_data);

	return 0;							return 0;
}								}

const struct file_operations ext3_dir_operations = {		const struct file_operations ext3_dir_operations = {
	.llseek		= ext3_dir_llseek,				.llseek		= ext3_dir_llseek,
	.read		= generic_read_dir,				.read		= generic_read_dir,
	.readdir	= ext3_readdir,					.readdir	= ext3_readdir,
	.unlocked_ioctl = ext3_ioctl,					.unlocked_ioctl = ext3_ioctl,
#ifdef CONFIG_COMPAT						#ifdef CONFIG_COMPAT
	.compat_ioctl	= ext3_compat_ioctl,				.compat_ioctl	= ext3_compat_ioctl,
#endif								#endif
	.fsync		= ext3_sync_file,				.fsync		= ext3_sync_file,
	.release	= ext3_release_dir,				.release	= ext3_release_dir,
};								};
