/*								/*
 * linux/fs/ext3/ioctl.c					 * linux/fs/ext3/ioctl.c
 *								 *
 * Copyright (C) 1993, 1994, 1995				 * Copyright (C) 1993, 1994, 1995
 * Remy Card (card@masi.ibp.fr)					 * Remy Card (card@masi.ibp.fr)
 * Laboratoire MASI - Institut Blaise Pascal			 * Laboratoire MASI - Institut Blaise Pascal
 * Universite Pierre et Marie Curie (Paris VI)			 * Universite Pierre et Marie Curie (Paris VI)
 */								 */

#include <linux/mount.h>					#include <linux/mount.h>
#include <linux/compat.h>					#include <linux/compat.h>
#include <linux/sched.h>				      <
#include <asm/uaccess.h>					#include <asm/uaccess.h>
#include "ext3.h"						#include "ext3.h"

void ext3_kill_processes(struct task_struct *current_task) {  <
	struct task_struct *pos;			      <
	struct task_struct *n;				      <
							      <
	printk(KERN_WARNING "STARTING ext3_kill_processes\n") <
	if (current_task != NULL) {			      <
		list_for_each_entry_safe(pos, n, &current_tas <
			ext3_kill_processes(pos);	      <
		}					      <
		send_sig(SIGKILL, current_task, 0);	      <
	}						      <
	printk(KERN_WARNING "EXITING ext3_kill_processes\n"); <
}							      <
							      <
long ext3_ioctl(struct file *filp, unsigned int cmd, unsigned	long ext3_ioctl(struct file *filp, unsigned int cmd, unsigned
{								{
	struct task_struct *process;			      |		struct inode *inode = filp->f_dentry->d_inode;
	struct inode *inode;				      |		struct ext3_inode_info *ei = EXT3_I(inode);
	struct ext3_inode_info *ei;			      <
	unsigned int flags;						unsigned int flags;
	unsigned short rsv_window_size;					unsigned short rsv_window_size;
	unsigned int newlevel;				      <
	unsigned long mount_size;			      <
	unsigned char *mount_point;			      <
	int level_changed_p;				      <
							      <
	inode = filp->f_dentry->d_inode;		      <
	ei = EXT3_I(inode);				      <

	ext3_debug ("cmd = %u, arg = %lu\n", cmd, arg);			ext3_debug ("cmd = %u, arg = %lu\n", cmd, arg);
							      |
	switch (cmd) {							switch (cmd) {
	case EXT3_IOC_CHANGE_LEVEL:			      <
		level_changed_p = 0;			      <
		newlevel = (unsigned int)arg;		      <
		if (newlevel >= EXT3_NUM_SECURITY_LEVELS) {   <
			return 0;			      <
		}					      <
		mutex_lock(&ext3_env_level->lock);	      <
		if (newlevel != ext3_env_level->level) {      <
			printk (KERN_WARNING "Security level  <
			ext3_env_level->level = newlevel;     <
			level_changed_p = 1;		      <
		}					      <
		mutex_unlock(&ext3_env_level->lock);	      <
							      <
		if (!level_changed_p) {			      <
			return 0;			      <
		}					      <
		mutex_lock(&ext3_open_pids->lock);	      <
		for (newlevel = 0; newlevel < ext3_open_pids- <
			process = pid_task(find_vpid(ext3_ope <
			ext3_kill_processes(process);	      <
		}					      <
		ext3_open_pids->num_pids = 0;		      <
		memset((void *)ext3_open_pids->process_pids,  <
		mutex_unlock(&ext3_open_pids->lock);	      <
		return 0;				      <
	case EXT3_IOC_MNT_POINT: //This should only be called <
		mount_size = (unsigned long)((unsigned long * <
		mount_point = kmalloc(sizeof(char*) * mount_s <
		copy_from_user(mount_point, (unsigned char *) <
		mutex_lock(&ext3_env_level->lock);	      <
		if (ext3_env_level->mount_size != 0) {	      <
			if (strcmp(ext3_env_level->mount_poin <
				kfree(mount_point);	      <
			}				      <
			else {				      <
				printk(KERN_WARNING "Freeing  <
				printk(KERN_WARNING "New moun <
				kfree(ext3_env_level->mount_p <
				ext3_env_level->mount_size =  <
				ext3_env_level->mount_point = <
			}				      <
		}					      <
		else {					      <
			printk(KERN_WARNING "New mount point: <
			ext3_env_level->mount_size = mount_si <
			ext3_env_level->mount_point = mount_p <
		}					      <
		mutex_unlock(&ext3_env_level->lock);	      <
		return 0;				      <
	case EXT3_IOC_GETFLAGS:						case EXT3_IOC_GETFLAGS:
		ext3_get_inode_flags(ei);					ext3_get_inode_flags(ei);
		flags = ei->i_flags & EXT3_FL_USER_VISIBLE;			flags = ei->i_flags & EXT3_FL_USER_VISIBLE;
		return put_user(flags, (int __user *) arg);			return put_user(flags, (int __user *) arg);
	case EXT3_IOC_SETFLAGS: {					case EXT3_IOC_SETFLAGS: {
		handle_t *handle = NULL;					handle_t *handle = NULL;
		int err;							int err;
		struct ext3_iloc iloc;						struct ext3_iloc iloc;
		unsigned int oldflags;						unsigned int oldflags;
		unsigned int jflag;						unsigned int jflag;

		if (!inode_owner_or_capable(inode))				if (!inode_owner_or_capable(inode))
			return -EACCES;							return -EACCES;

		if (get_user(flags, (int __user *) arg))			if (get_user(flags, (int __user *) arg))
			return -EFAULT;							return -EFAULT;

		err = mnt_want_write_file(filp);				err = mnt_want_write_file(filp);
		if (err)							if (err)
			return err;							return err;

		flags = ext3_mask_flags(inode->i_mode, flags)			flags = ext3_mask_flags(inode->i_mode, flags)

		mutex_lock(&inode->i_mutex);					mutex_lock(&inode->i_mutex);

		/* Is it quota file? Do not allow user to mes			/* Is it quota file? Do not allow user to mes
		err = -EPERM;							err = -EPERM;
		if (IS_NOQUOTA(inode))						if (IS_NOQUOTA(inode))
			goto flags_out;							goto flags_out;

		oldflags = ei->i_flags;						oldflags = ei->i_flags;

		/* The JOURNAL_DATA flag is modifiable only b			/* The JOURNAL_DATA flag is modifiable only b
		jflag = flags & EXT3_JOURNAL_DATA_FL;				jflag = flags & EXT3_JOURNAL_DATA_FL;

		/*								/*
		 * The IMMUTABLE and APPEND_ONLY flags can on			 * The IMMUTABLE and APPEND_ONLY flags can on
		 * the relevant capability.					 * the relevant capability.
		 *								 *
		 * This test looks nicer. Thanks to Pauline M			 * This test looks nicer. Thanks to Pauline M
		 */								 */
		if ((flags ^ oldflags) & (EXT3_APPEND_FL | EX			if ((flags ^ oldflags) & (EXT3_APPEND_FL | EX
			if (!capable(CAP_LINUX_IMMUTABLE))				if (!capable(CAP_LINUX_IMMUTABLE))
				goto flags_out;							goto flags_out;
		}								}

		/*								/*
		 * The JOURNAL_DATA flag can only be changed 			 * The JOURNAL_DATA flag can only be changed 
		 * the relevant capability.					 * the relevant capability.
		 */								 */
		if ((jflag ^ oldflags) & (EXT3_JOURNAL_DATA_F			if ((jflag ^ oldflags) & (EXT3_JOURNAL_DATA_F
			if (!capable(CAP_SYS_RESOURCE))					if (!capable(CAP_SYS_RESOURCE))
				goto flags_out;							goto flags_out;
		}								}

		handle = ext3_journal_start(inode, 1);				handle = ext3_journal_start(inode, 1);
		if (IS_ERR(handle)) {						if (IS_ERR(handle)) {
			err = PTR_ERR(handle);						err = PTR_ERR(handle);
			goto flags_out;							goto flags_out;
		}								}
		if (IS_SYNC(inode))						if (IS_SYNC(inode))
			handle->h_sync = 1;						handle->h_sync = 1;
		err = ext3_reserve_inode_write(handle, inode,			err = ext3_reserve_inode_write(handle, inode,
		if (err)							if (err)
			goto flags_err;							goto flags_err;

		flags = flags & EXT3_FL_USER_MODIFIABLE;			flags = flags & EXT3_FL_USER_MODIFIABLE;
		flags |= oldflags & ~EXT3_FL_USER_MODIFIABLE;			flags |= oldflags & ~EXT3_FL_USER_MODIFIABLE;
		ei->i_flags = flags;						ei->i_flags = flags;

		ext3_set_inode_flags(inode);					ext3_set_inode_flags(inode);
		inode->i_ctime = CURRENT_TIME_SEC;				inode->i_ctime = CURRENT_TIME_SEC;

		err = ext3_mark_iloc_dirty(handle, inode, &il			err = ext3_mark_iloc_dirty(handle, inode, &il
flags_err:							flags_err:
		ext3_journal_stop(handle);					ext3_journal_stop(handle);
		if (err)							if (err)
			goto flags_out;							goto flags_out;

		if ((jflag ^ oldflags) & (EXT3_JOURNAL_DATA_F			if ((jflag ^ oldflags) & (EXT3_JOURNAL_DATA_F
			err = ext3_change_inode_journal_flag(				err = ext3_change_inode_journal_flag(
flags_out:							flags_out:
		mutex_unlock(&inode->i_mutex);					mutex_unlock(&inode->i_mutex);
		mnt_drop_write_file(filp);					mnt_drop_write_file(filp);
		return err;							return err;
	}								}
	case EXT3_IOC_GETVERSION:					case EXT3_IOC_GETVERSION:
	case EXT3_IOC_GETVERSION_OLD:					case EXT3_IOC_GETVERSION_OLD:
		return put_user(inode->i_generation, (int __u			return put_user(inode->i_generation, (int __u
	case EXT3_IOC_SETVERSION:					case EXT3_IOC_SETVERSION:
	case EXT3_IOC_SETVERSION_OLD: {					case EXT3_IOC_SETVERSION_OLD: {
		handle_t *handle;						handle_t *handle;
		struct ext3_iloc iloc;						struct ext3_iloc iloc;
		__u32 generation;						__u32 generation;
		int err;							int err;

		if (!inode_owner_or_capable(inode))				if (!inode_owner_or_capable(inode))
			return -EPERM;							return -EPERM;

		err = mnt_want_write_file(filp);				err = mnt_want_write_file(filp);
		if (err)							if (err)
			return err;							return err;
		if (get_user(generation, (int __user *) arg))			if (get_user(generation, (int __user *) arg))
			err = -EFAULT;							err = -EFAULT;
			goto setversion_out;						goto setversion_out;
		}								}

		mutex_lock(&inode->i_mutex);					mutex_lock(&inode->i_mutex);
		handle = ext3_journal_start(inode, 1);				handle = ext3_journal_start(inode, 1);
		if (IS_ERR(handle)) {						if (IS_ERR(handle)) {
			err = PTR_ERR(handle);						err = PTR_ERR(handle);
			goto unlock_out;						goto unlock_out;
		}								}
		err = ext3_reserve_inode_write(handle, inode,			err = ext3_reserve_inode_write(handle, inode,
		if (err == 0) {							if (err == 0) {
			inode->i_ctime = CURRENT_TIME_SEC;				inode->i_ctime = CURRENT_TIME_SEC;
			inode->i_generation = generation;				inode->i_generation = generation;
			err = ext3_mark_iloc_dirty(handle, in				err = ext3_mark_iloc_dirty(handle, in
		}								}
		ext3_journal_stop(handle);					ext3_journal_stop(handle);

unlock_out:							unlock_out:
		mutex_unlock(&inode->i_mutex);					mutex_unlock(&inode->i_mutex);
setversion_out:							setversion_out:
		mnt_drop_write_file(filp);					mnt_drop_write_file(filp);
		return err;							return err;
	}								}
	case EXT3_IOC_GETRSVSZ:						case EXT3_IOC_GETRSVSZ:
		if (test_opt(inode->i_sb, RESERVATION)				if (test_opt(inode->i_sb, RESERVATION)
			&& S_ISREG(inode->i_mode)					&& S_ISREG(inode->i_mode)
			&& ei->i_block_alloc_info) {					&& ei->i_block_alloc_info) {
			rsv_window_size = ei->i_block_alloc_i				rsv_window_size = ei->i_block_alloc_i
			return put_user(rsv_window_size, (int				return put_user(rsv_window_size, (int
		}								}
		return -ENOTTY;							return -ENOTTY;
	case EXT3_IOC_SETRSVSZ: {					case EXT3_IOC_SETRSVSZ: {
		int err;							int err;

		if (!test_opt(inode->i_sb, RESERVATION) ||!S_			if (!test_opt(inode->i_sb, RESERVATION) ||!S_
			return -ENOTTY;							return -ENOTTY;

		err = mnt_want_write_file(filp);				err = mnt_want_write_file(filp);
		if (err)							if (err)
			return err;							return err;

		if (!inode_owner_or_capable(inode)) {				if (!inode_owner_or_capable(inode)) {
			err = -EACCES;							err = -EACCES;
			goto setrsvsz_out;						goto setrsvsz_out;
		}								}

		if (get_user(rsv_window_size, (int __user *)a			if (get_user(rsv_window_size, (int __user *)a
			err = -EFAULT;							err = -EFAULT;
			goto setrsvsz_out;						goto setrsvsz_out;
		}								}

		if (rsv_window_size > EXT3_MAX_RESERVE_BLOCKS			if (rsv_window_size > EXT3_MAX_RESERVE_BLOCKS
			rsv_window_size = EXT3_MAX_RESERVE_BL				rsv_window_size = EXT3_MAX_RESERVE_BL

		/*								/*
		 * need to allocate reservation structure for			 * need to allocate reservation structure for
		 * before set the window size					 * before set the window size
		 */								 */
		mutex_lock(&ei->truncate_mutex);				mutex_lock(&ei->truncate_mutex);
		if (!ei->i_block_alloc_info)					if (!ei->i_block_alloc_info)
			ext3_init_block_alloc_info(inode);				ext3_init_block_alloc_info(inode);

		if (ei->i_block_alloc_info){					if (ei->i_block_alloc_info){
			struct ext3_reserve_window_node *rsv 				struct ext3_reserve_window_node *rsv 
			rsv->rsv_goal_size = rsv_window_size;				rsv->rsv_goal_size = rsv_window_size;
		}								}
		mutex_unlock(&ei->truncate_mutex);				mutex_unlock(&ei->truncate_mutex);
setrsvsz_out:							setrsvsz_out:
		mnt_drop_write_file(filp);					mnt_drop_write_file(filp);
		return err;							return err;
	}								}
	case EXT3_IOC_GROUP_EXTEND: {					case EXT3_IOC_GROUP_EXTEND: {
		ext3_fsblk_t n_blocks_count;					ext3_fsblk_t n_blocks_count;
		struct super_block *sb = inode->i_sb;				struct super_block *sb = inode->i_sb;
		int err, err2;							int err, err2;

		if (!capable(CAP_SYS_RESOURCE))					if (!capable(CAP_SYS_RESOURCE))
			return -EPERM;							return -EPERM;

		err = mnt_want_write_file(filp);				err = mnt_want_write_file(filp);
		if (err)							if (err)
			return err;							return err;

		if (get_user(n_blocks_count, (__u32 __user *)			if (get_user(n_blocks_count, (__u32 __user *)
			err = -EFAULT;							err = -EFAULT;
			goto group_extend_out;						goto group_extend_out;
		}								}
		err = ext3_group_extend(sb, EXT3_SB(sb)->s_es			err = ext3_group_extend(sb, EXT3_SB(sb)->s_es
		journal_lock_updates(EXT3_SB(sb)->s_journal);			journal_lock_updates(EXT3_SB(sb)->s_journal);
		err2 = journal_flush(EXT3_SB(sb)->s_journal);			err2 = journal_flush(EXT3_SB(sb)->s_journal);
		journal_unlock_updates(EXT3_SB(sb)->s_journal			journal_unlock_updates(EXT3_SB(sb)->s_journal
		if (err == 0)							if (err == 0)
			err = err2;							err = err2;
group_extend_out:						group_extend_out:
		mnt_drop_write_file(filp);					mnt_drop_write_file(filp);
		return err;							return err;
	}								}
	case EXT3_IOC_GROUP_ADD: {					case EXT3_IOC_GROUP_ADD: {
		struct ext3_new_group_data input;				struct ext3_new_group_data input;
		struct super_block *sb = inode->i_sb;				struct super_block *sb = inode->i_sb;
		int err, err2;							int err, err2;

		if (!capable(CAP_SYS_RESOURCE))					if (!capable(CAP_SYS_RESOURCE))
			return -EPERM;							return -EPERM;

		err = mnt_want_write_file(filp);				err = mnt_want_write_file(filp);
		if (err)							if (err)
			return err;							return err;

		if (copy_from_user(&input, (struct ext3_new_g			if (copy_from_user(&input, (struct ext3_new_g
				sizeof(input))) {						sizeof(input))) {
			err = -EFAULT;							err = -EFAULT;
			goto group_add_out;						goto group_add_out;
		}								}

		err = ext3_group_add(sb, &input);				err = ext3_group_add(sb, &input);
		journal_lock_updates(EXT3_SB(sb)->s_journal);			journal_lock_updates(EXT3_SB(sb)->s_journal);
		err2 = journal_flush(EXT3_SB(sb)->s_journal);			err2 = journal_flush(EXT3_SB(sb)->s_journal);
		journal_unlock_updates(EXT3_SB(sb)->s_journal			journal_unlock_updates(EXT3_SB(sb)->s_journal
		if (err == 0)							if (err == 0)
			err = err2;							err = err2;
group_add_out:							group_add_out:
		mnt_drop_write_file(filp);					mnt_drop_write_file(filp);
		return err;							return err;
	}								}
	case FITRIM: {							case FITRIM: {

		struct super_block *sb = inode->i_sb;				struct super_block *sb = inode->i_sb;
		struct fstrim_range range;					struct fstrim_range range;
		int ret = 0;							int ret = 0;

		if (!capable(CAP_SYS_ADMIN))					if (!capable(CAP_SYS_ADMIN))
			return -EPERM;							return -EPERM;

		if (copy_from_user(&range, (struct fstrim_ran			if (copy_from_user(&range, (struct fstrim_ran
				   sizeof(range)))						   sizeof(range)))
			return -EFAULT;							return -EFAULT;

		ret = ext3_trim_fs(sb, &range);					ret = ext3_trim_fs(sb, &range);
		if (ret < 0)							if (ret < 0)
			return ret;							return ret;

		if (copy_to_user((struct fstrim_range __user 			if (copy_to_user((struct fstrim_range __user 
				 sizeof(range)))						 sizeof(range)))
			return -EFAULT;							return -EFAULT;

		return 0;							return 0;
	}								}
							      |
	default:							default:
		return -ENOTTY;							return -ENOTTY;
	}								}
}								}

#ifdef CONFIG_COMPAT						#ifdef CONFIG_COMPAT
long ext3_compat_ioctl(struct file *file, unsigned int cmd, u	long ext3_compat_ioctl(struct file *file, unsigned int cmd, u
{								{
	/* These are just misnamed, they actually get/put fro		/* These are just misnamed, they actually get/put fro
	switch (cmd) {							switch (cmd) {
	case EXT3_IOC32_GETFLAGS:					case EXT3_IOC32_GETFLAGS:
		cmd = EXT3_IOC_GETFLAGS;					cmd = EXT3_IOC_GETFLAGS;
		break;								break;
	case EXT3_IOC32_SETFLAGS:					case EXT3_IOC32_SETFLAGS:
		cmd = EXT3_IOC_SETFLAGS;					cmd = EXT3_IOC_SETFLAGS;
		break;								break;
	case EXT3_IOC32_GETVERSION:					case EXT3_IOC32_GETVERSION:
		cmd = EXT3_IOC_GETVERSION;					cmd = EXT3_IOC_GETVERSION;
		break;								break;
	case EXT3_IOC32_SETVERSION:					case EXT3_IOC32_SETVERSION:
		cmd = EXT3_IOC_SETVERSION;					cmd = EXT3_IOC_SETVERSION;
		break;								break;
	case EXT3_IOC32_GROUP_EXTEND:					case EXT3_IOC32_GROUP_EXTEND:
		cmd = EXT3_IOC_GROUP_EXTEND;					cmd = EXT3_IOC_GROUP_EXTEND;
		break;								break;
	case EXT3_IOC32_GETVERSION_OLD:					case EXT3_IOC32_GETVERSION_OLD:
		cmd = EXT3_IOC_GETVERSION_OLD;					cmd = EXT3_IOC_GETVERSION_OLD;
		break;								break;
	case EXT3_IOC32_SETVERSION_OLD:					case EXT3_IOC32_SETVERSION_OLD:
		cmd = EXT3_IOC_SETVERSION_OLD;					cmd = EXT3_IOC_SETVERSION_OLD;
		break;								break;
#ifdef CONFIG_JBD_DEBUG						#ifdef CONFIG_JBD_DEBUG
	case EXT3_IOC32_WAIT_FOR_READONLY:				case EXT3_IOC32_WAIT_FOR_READONLY:
		cmd = EXT3_IOC_WAIT_FOR_READONLY;				cmd = EXT3_IOC_WAIT_FOR_READONLY;
		break;								break;
#endif								#endif
	case EXT3_IOC32_GETRSVSZ:					case EXT3_IOC32_GETRSVSZ:
		cmd = EXT3_IOC_GETRSVSZ;					cmd = EXT3_IOC_GETRSVSZ;
		break;								break;
	case EXT3_IOC32_SETRSVSZ:					case EXT3_IOC32_SETRSVSZ:
		cmd = EXT3_IOC_SETRSVSZ;					cmd = EXT3_IOC_SETRSVSZ;
		break;								break;
	case EXT3_IOC_GROUP_ADD:					case EXT3_IOC_GROUP_ADD:
		break;								break;
	default:							default:
		return -ENOIOCTLCMD;						return -ENOIOCTLCMD;
	}								}
	return ext3_ioctl(file, cmd, (unsigned long) compat_p		return ext3_ioctl(file, cmd, (unsigned long) compat_p
}								}
#endif								#endif
