/*								/*
 *  linux/fs/ext3/inode.c					 *  linux/fs/ext3/inode.c
 *								 *
 * Copyright (C) 1992, 1993, 1994, 1995				 * Copyright (C) 1992, 1993, 1994, 1995
 * Remy Card (card@masi.ibp.fr)					 * Remy Card (card@masi.ibp.fr)
 * Laboratoire MASI - Institut Blaise Pascal			 * Laboratoire MASI - Institut Blaise Pascal
 * Universite Pierre et Marie Curie (Paris VI)			 * Universite Pierre et Marie Curie (Paris VI)
 *								 *
 *  from							 *  from
 *								 *
 *  linux/fs/minix/inode.c					 *  linux/fs/minix/inode.c
 *								 *
 *  Copyright (C) 1991, 1992  Linus Torvalds			 *  Copyright (C) 1991, 1992  Linus Torvalds
 *								 *
 *  Goal-directed block allocation by Stephen Tweedie		 *  Goal-directed block allocation by Stephen Tweedie
 *	(sct@redhat.com), 1993, 1998				 *	(sct@redhat.com), 1993, 1998
 *  Big-endian to little-endian byte-swapping/bitmaps by	 *  Big-endian to little-endian byte-swapping/bitmaps by
 *        David S. Miller (davem@caip.rutgers.edu), 1995	 *        David S. Miller (davem@caip.rutgers.edu), 1995
 *  64-bit file support on 64-bit platforms by Jakub Jelinek	 *  64-bit file support on 64-bit platforms by Jakub Jelinek
 *	(jj@sunsite.ms.mff.cuni.cz)				 *	(jj@sunsite.ms.mff.cuni.cz)
 *								 *
 *  Assorted race fixes, rewrite of ext3_get_block() by Al Vi	 *  Assorted race fixes, rewrite of ext3_get_block() by Al Vi
 */								 */

#include <linux/highuid.h>					#include <linux/highuid.h>
#include <linux/quotaops.h>					#include <linux/quotaops.h>
#include <linux/writeback.h>					#include <linux/writeback.h>
#include <linux/mpage.h>					#include <linux/mpage.h>
#include <linux/namei.h>					#include <linux/namei.h>
#include "ext3.h"						#include "ext3.h"
#include "xattr.h"						#include "xattr.h"
#include "acl.h"						#include "acl.h"

static int ext3_writepage_trans_blocks(struct inode *inode);	static int ext3_writepage_trans_blocks(struct inode *inode);
static int ext3_block_truncate_page(struct inode *inode, loff	static int ext3_block_truncate_page(struct inode *inode, loff

/*								/*
 * Test whether an inode is a fast symlink.			 * Test whether an inode is a fast symlink.
 */								 */
static int ext3_inode_is_fast_symlink(struct inode *inode)	static int ext3_inode_is_fast_symlink(struct inode *inode)
{								{
	int ea_blocks = EXT3_I(inode)->i_file_acl ?			int ea_blocks = EXT3_I(inode)->i_file_acl ?
		(inode->i_sb->s_blocksize >> 9) : 0;				(inode->i_sb->s_blocksize >> 9) : 0;

	return (S_ISLNK(inode->i_mode) && inode->i_blocks - e		return (S_ISLNK(inode->i_mode) && inode->i_blocks - e
}								}

/*								/*
 * The ext3 forget function must perform a revoke if we are f	 * The ext3 forget function must perform a revoke if we are f
 * which has been journaled.  Metadata (eg. indirect blocks) 	 * which has been journaled.  Metadata (eg. indirect blocks) 
 * revoked in all cases.					 * revoked in all cases.
 *								 *
 * "bh" may be NULL: a metadata block may have been freed fro	 * "bh" may be NULL: a metadata block may have been freed fro
 * but there may still be a record of it in the journal, and 	 * but there may still be a record of it in the journal, and 
 * still needs to be revoked.					 * still needs to be revoked.
 */								 */
int ext3_forget(handle_t *handle, int is_metadata, struct ino	int ext3_forget(handle_t *handle, int is_metadata, struct ino
			struct buffer_head *bh, ext3_fsblk_t 				struct buffer_head *bh, ext3_fsblk_t 
{								{
	int err;							int err;

	might_sleep();							might_sleep();

	trace_ext3_forget(inode, is_metadata, blocknr);			trace_ext3_forget(inode, is_metadata, blocknr);
	BUFFER_TRACE(bh, "enter");					BUFFER_TRACE(bh, "enter");

	jbd_debug(4, "forgetting bh %p: is_metadata = %d, mod		jbd_debug(4, "forgetting bh %p: is_metadata = %d, mod
		  "data mode %lx\n",						  "data mode %lx\n",
		  bh, is_metadata, inode->i_mode,				  bh, is_metadata, inode->i_mode,
		  test_opt(inode->i_sb, DATA_FLAGS));				  test_opt(inode->i_sb, DATA_FLAGS));

	/* Never use the revoke function if we are doing full		/* Never use the revoke function if we are doing full
	 * journaling: there is no need to, and a V1 superblo		 * journaling: there is no need to, and a V1 superblo
	 * support it.  Otherwise, only skip the revoke on un		 * support it.  Otherwise, only skip the revoke on un
	 * data blocks. */						 * data blocks. */

	if (test_opt(inode->i_sb, DATA_FLAGS) == EXT3_MOUNT_J		if (test_opt(inode->i_sb, DATA_FLAGS) == EXT3_MOUNT_J
	    (!is_metadata && !ext3_should_journal_data(inode)		    (!is_metadata && !ext3_should_journal_data(inode)
		if (bh) {							if (bh) {
			BUFFER_TRACE(bh, "call journal_forget				BUFFER_TRACE(bh, "call journal_forget
			return ext3_journal_forget(handle, bh				return ext3_journal_forget(handle, bh
		}								}
		return 0;							return 0;
	}								}

	/*								/*
	 * data!=journal && (is_metadata || should_journal_da		 * data!=journal && (is_metadata || should_journal_da
	 */								 */
	BUFFER_TRACE(bh, "call ext3_journal_revoke");			BUFFER_TRACE(bh, "call ext3_journal_revoke");
	err = ext3_journal_revoke(handle, blocknr, bh);			err = ext3_journal_revoke(handle, blocknr, bh);
	if (err)							if (err)
		ext3_abort(inode->i_sb, __func__,				ext3_abort(inode->i_sb, __func__,
			   "error %d when attempting revoke",				   "error %d when attempting revoke",
	BUFFER_TRACE(bh, "exit");					BUFFER_TRACE(bh, "exit");
	return err;							return err;
}								}

/*								/*
 * Work out how many blocks we need to proceed with the next 	 * Work out how many blocks we need to proceed with the next 
 * truncate transaction.					 * truncate transaction.
 */								 */
static unsigned long blocks_for_truncate(struct inode *inode)	static unsigned long blocks_for_truncate(struct inode *inode)
{								{
	unsigned long needed;						unsigned long needed;

	needed = inode->i_blocks >> (inode->i_sb->s_blocksize		needed = inode->i_blocks >> (inode->i_sb->s_blocksize

	/* Give ourselves just enough room to cope with inode		/* Give ourselves just enough room to cope with inode
	 * i_blocks is corrupt: we've seen disk corruptions i		 * i_blocks is corrupt: we've seen disk corruptions i
	 * which resulted in random data in an inode which lo		 * which resulted in random data in an inode which lo
	 * like a regular file for ext3 to try to delete it. 		 * like a regular file for ext3 to try to delete it. 
	 * will go a bit crazy if that happens, but at least 		 * will go a bit crazy if that happens, but at least 
	 * try not to panic the whole kernel. */			 * try not to panic the whole kernel. */
	if (needed < 2)							if (needed < 2)
		needed = 2;							needed = 2;

	/* But we need to bound the transaction so we don't o		/* But we need to bound the transaction so we don't o
	 * journal. */							 * journal. */
	if (needed > EXT3_MAX_TRANS_DATA)				if (needed > EXT3_MAX_TRANS_DATA)
		needed = EXT3_MAX_TRANS_DATA;					needed = EXT3_MAX_TRANS_DATA;

	return EXT3_DATA_TRANS_BLOCKS(inode->i_sb) + needed;		return EXT3_DATA_TRANS_BLOCKS(inode->i_sb) + needed;
}								}

/*								/*
 * Truncate transactions can be complex and absolutely huge. 	 * Truncate transactions can be complex and absolutely huge. 
 * be able to restart the transaction at a conventient checkp	 * be able to restart the transaction at a conventient checkp
 * sure we don't overflow the journal.				 * sure we don't overflow the journal.
 *								 *
 * start_transaction gets us a new handle for a truncate tran	 * start_transaction gets us a new handle for a truncate tran
 * and extend_transaction tries to extend the existing one a 	 * and extend_transaction tries to extend the existing one a 
 * extend fails, we need to propagate the failure up and rest	 * extend fails, we need to propagate the failure up and rest
 * transaction in the top-level truncate loop. --sct		 * transaction in the top-level truncate loop. --sct
 */								 */
static handle_t *start_transaction(struct inode *inode)		static handle_t *start_transaction(struct inode *inode)
{								{
	handle_t *result;						handle_t *result;

	result = ext3_journal_start(inode, blocks_for_truncat		result = ext3_journal_start(inode, blocks_for_truncat
	if (!IS_ERR(result))						if (!IS_ERR(result))
		return result;							return result;

	ext3_std_error(inode->i_sb, PTR_ERR(result));			ext3_std_error(inode->i_sb, PTR_ERR(result));
	return result;							return result;
}								}

/*								/*
 * Try to extend this transaction for the purposes of truncat	 * Try to extend this transaction for the purposes of truncat
 *								 *
 * Returns 0 if we managed to create more room.  If we can't 	 * Returns 0 if we managed to create more room.  If we can't 
 * room, and the transaction must be restarted we return 1.	 * room, and the transaction must be restarted we return 1.
 */								 */
static int try_to_extend_transaction(handle_t *handle, struct	static int try_to_extend_transaction(handle_t *handle, struct
{								{
	if (handle->h_buffer_credits > EXT3_RESERVE_TRANS_BLO		if (handle->h_buffer_credits > EXT3_RESERVE_TRANS_BLO
		return 0;							return 0;
	if (!ext3_journal_extend(handle, blocks_for_truncate(		if (!ext3_journal_extend(handle, blocks_for_truncate(
		return 0;							return 0;
	return 1;							return 1;
}								}

/*								/*
 * Restart the transaction associated with *handle.  This doe	 * Restart the transaction associated with *handle.  This doe
 * so before we call here everything must be consistently dir	 * so before we call here everything must be consistently dir
 * this transaction.						 * this transaction.
 */								 */
static int truncate_restart_transaction(handle_t *handle, str	static int truncate_restart_transaction(handle_t *handle, str
{								{
	int ret;							int ret;

	jbd_debug(2, "restarting handle %p\n", handle);			jbd_debug(2, "restarting handle %p\n", handle);
	/*								/*
	 * Drop truncate_mutex to avoid deadlock with ext3_ge		 * Drop truncate_mutex to avoid deadlock with ext3_ge
	 * At this moment, get_block can be called only for b		 * At this moment, get_block can be called only for b
	 * i_size since page cache has been already dropped a		 * i_size since page cache has been already dropped a
	 * blocked by i_mutex. So we can safely drop the trun		 * blocked by i_mutex. So we can safely drop the trun
	 */								 */
	mutex_unlock(&EXT3_I(inode)->truncate_mutex);			mutex_unlock(&EXT3_I(inode)->truncate_mutex);
	ret = ext3_journal_restart(handle, blocks_for_truncat		ret = ext3_journal_restart(handle, blocks_for_truncat
	mutex_lock(&EXT3_I(inode)->truncate_mutex);			mutex_lock(&EXT3_I(inode)->truncate_mutex);
	return ret;							return ret;
}								}

/*								/*
 * Called at inode eviction from icache				 * Called at inode eviction from icache
 */								 */
void ext3_evict_inode (struct inode *inode)			void ext3_evict_inode (struct inode *inode)
{								{
	struct ext3_inode_info *ei = EXT3_I(inode);			struct ext3_inode_info *ei = EXT3_I(inode);
	struct ext3_block_alloc_info *rsv;				struct ext3_block_alloc_info *rsv;
	handle_t *handle;						handle_t *handle;
	int want_delete = 0;						int want_delete = 0;

	trace_ext3_evict_inode(inode);					trace_ext3_evict_inode(inode);
	if (!inode->i_nlink && !is_bad_inode(inode)) {			if (!inode->i_nlink && !is_bad_inode(inode)) {
		dquot_initialize(inode);					dquot_initialize(inode);
		want_delete = 1;						want_delete = 1;
	}								}

	/*								/*
	 * When journalling data dirty buffers are tracked on		 * When journalling data dirty buffers are tracked on
	 * So although mm thinks everything is clean and read		 * So although mm thinks everything is clean and read
	 * inode might still have some pages to write in the 		 * inode might still have some pages to write in the 
	 * transaction or waiting to be checkpointed. Thus ca		 * transaction or waiting to be checkpointed. Thus ca
	 * journal_invalidatepage() (via truncate_inode_pages		 * journal_invalidatepage() (via truncate_inode_pages
	 * these buffers can cause data loss. Also even if we		 * these buffers can cause data loss. Also even if we
	 * these buffers, we would have no way to find them a		 * these buffers, we would have no way to find them a
	 * is reaped and thus user could see stale data if he		 * is reaped and thus user could see stale data if he
	 * them before the transaction is checkpointed. So be		 * them before the transaction is checkpointed. So be
	 * force everything to disk here... We use ei->i_data		 * force everything to disk here... We use ei->i_data
	 * store the newest transaction containing inode's da		 * store the newest transaction containing inode's da
	 *								 *
	 * Note that directories do not have this problem bec		 * Note that directories do not have this problem bec
	 * use page cache.						 * use page cache.
	 *								 *
	 * The s_journal check handles the case when ext3_get		 * The s_journal check handles the case when ext3_get
	 * and puts the journal inode.					 * and puts the journal inode.
	 */								 */
	if (inode->i_nlink && ext3_should_journal_data(inode)		if (inode->i_nlink && ext3_should_journal_data(inode)
	    EXT3_SB(inode->i_sb)->s_journal &&				    EXT3_SB(inode->i_sb)->s_journal &&
	    (S_ISLNK(inode->i_mode) || S_ISREG(inode->i_mode)		    (S_ISLNK(inode->i_mode) || S_ISREG(inode->i_mode)
		tid_t commit_tid = atomic_read(&ei->i_datasyn			tid_t commit_tid = atomic_read(&ei->i_datasyn
		journal_t *journal = EXT3_SB(inode->i_sb)->s_			journal_t *journal = EXT3_SB(inode->i_sb)->s_

		log_start_commit(journal, commit_tid);				log_start_commit(journal, commit_tid);
		log_wait_commit(journal, commit_tid);				log_wait_commit(journal, commit_tid);
		filemap_write_and_wait(&inode->i_data);				filemap_write_and_wait(&inode->i_data);
	}								}
	truncate_inode_pages(&inode->i_data, 0);			truncate_inode_pages(&inode->i_data, 0);

	ext3_discard_reservation(inode);				ext3_discard_reservation(inode);
	rsv = ei->i_block_alloc_info;					rsv = ei->i_block_alloc_info;
	ei->i_block_alloc_info = NULL;					ei->i_block_alloc_info = NULL;
	if (unlikely(rsv))						if (unlikely(rsv))
		kfree(rsv);							kfree(rsv);

	if (!want_delete)						if (!want_delete)
		goto no_delete;							goto no_delete;

	handle = start_transaction(inode);				handle = start_transaction(inode);
	if (IS_ERR(handle)) {						if (IS_ERR(handle)) {
		/*								/*
		 * If we're going to skip the normal cleanup,			 * If we're going to skip the normal cleanup,
		 * make sure that the in-core orphan linked l			 * make sure that the in-core orphan linked l
		 * cleaned up.							 * cleaned up.
		 */								 */
		ext3_orphan_del(NULL, inode);					ext3_orphan_del(NULL, inode);
		goto no_delete;							goto no_delete;
	}								}

	if (IS_SYNC(inode))						if (IS_SYNC(inode))
		handle->h_sync = 1;						handle->h_sync = 1;
	inode->i_size = 0;						inode->i_size = 0;
	if (inode->i_blocks)						if (inode->i_blocks)
		ext3_truncate(inode);						ext3_truncate(inode);
	/*								/*
	 * Kill off the orphan record created when the inode 		 * Kill off the orphan record created when the inode 
	 * link.  Note that ext3_orphan_del() has to be able 		 * link.  Note that ext3_orphan_del() has to be able 
	 * deletion of a non-existent orphan - ext3_truncate(		 * deletion of a non-existent orphan - ext3_truncate(
	 * have removed the record.					 * have removed the record.
	 */								 */
	ext3_orphan_del(handle, inode);					ext3_orphan_del(handle, inode);
	ei->i_dtime = get_seconds();					ei->i_dtime = get_seconds();

	/*								/*
	 * One subtle ordering requirement: if anything has g		 * One subtle ordering requirement: if anything has g
	 * (transaction abort, IO errors, whatever), then we 		 * (transaction abort, IO errors, whatever), then we 
	 * do these next steps (the fs will already have been		 * do these next steps (the fs will already have been
	 * having errors), but we can't free the inode if the		 * having errors), but we can't free the inode if the
	 * fails.							 * fails.
	 */								 */
	if (ext3_mark_inode_dirty(handle, inode)) {			if (ext3_mark_inode_dirty(handle, inode)) {
		/* If that failed, just dquot_drop() and be d			/* If that failed, just dquot_drop() and be d
		dquot_drop(inode);						dquot_drop(inode);
		clear_inode(inode);						clear_inode(inode);
	} else {							} else {
		ext3_xattr_delete_inode(handle, inode);				ext3_xattr_delete_inode(handle, inode);
		dquot_free_inode(inode);					dquot_free_inode(inode);
		dquot_drop(inode);						dquot_drop(inode);
		clear_inode(inode);						clear_inode(inode);
		ext3_free_inode(handle, inode);					ext3_free_inode(handle, inode);
	}								}
	ext3_journal_stop(handle);					ext3_journal_stop(handle);
	return;								return;
no_delete:							no_delete:
	clear_inode(inode);						clear_inode(inode);
	dquot_drop(inode);						dquot_drop(inode);
}								}

typedef struct {						typedef struct {
	__le32	*p;							__le32	*p;
	__le32	key;							__le32	key;
	struct buffer_head *bh;						struct buffer_head *bh;
} Indirect;							} Indirect;

static inline void add_chain(Indirect *p, struct buffer_head 	static inline void add_chain(Indirect *p, struct buffer_head 
{								{
	p->key = *(p->p = v);						p->key = *(p->p = v);
	p->bh = bh;							p->bh = bh;
}								}

static int verify_chain(Indirect *from, Indirect *to)		static int verify_chain(Indirect *from, Indirect *to)
{								{
	while (from <= to && from->key == *from->p)			while (from <= to && from->key == *from->p)
		from++;								from++;
	return (from > to);						return (from > to);
}								}

/**								/**
 *	ext3_block_to_path - parse the block number into arra	 *	ext3_block_to_path - parse the block number into arra
 *	@inode: inode in question (we are only interested in 	 *	@inode: inode in question (we are only interested in 
 *	@i_block: block number to be parsed			 *	@i_block: block number to be parsed
 *	@offsets: array to store the offsets in			 *	@offsets: array to store the offsets in
 *      @boundary: set this non-zero if the referred-to block	 *      @boundary: set this non-zero if the referred-to block
 *             followed (on disk) by an indirect block.		 *             followed (on disk) by an indirect block.
 *								 *
 *	To store the locations of file's data ext3 uses a dat	 *	To store the locations of file's data ext3 uses a dat
 *	for UNIX filesystems - tree of pointers anchored in t	 *	for UNIX filesystems - tree of pointers anchored in t
 *	data blocks at leaves and indirect blocks in intermed	 *	data blocks at leaves and indirect blocks in intermed
 *	This function translates the block number into path i	 *	This function translates the block number into path i
 *	return value is the path length and @offsets[n] is th	 *	return value is the path length and @offsets[n] is th
 *	pointer to (n+1)th node in the nth one. If @block is 	 *	pointer to (n+1)th node in the nth one. If @block is 
 *	(negative or too large) warning is printed and zero r	 *	(negative or too large) warning is printed and zero r
 *								 *
 *	Note: function doesn't find node addresses, so no IO 	 *	Note: function doesn't find node addresses, so no IO 
 *	we need to know is the capacity of indirect blocks (t	 *	we need to know is the capacity of indirect blocks (t
 *	inode->i_sb).						 *	inode->i_sb).
 */								 */

/*								/*
 * Portability note: the last comparison (check that we fit i	 * Portability note: the last comparison (check that we fit i
 * indirect block) is spelled differently, because otherwise 	 * indirect block) is spelled differently, because otherwise 
 * architecture with 32-bit longs and 8Kb pages we might get 	 * architecture with 32-bit longs and 8Kb pages we might get 
 * if our filesystem had 8Kb blocks. We might use long long, 	 * if our filesystem had 8Kb blocks. We might use long long, 
 * kill us on x86. Oh, well, at least the sign propagation do	 * kill us on x86. Oh, well, at least the sign propagation do
 * i_block would have to be negative in the very beginning, s	 * i_block would have to be negative in the very beginning, s
 * get there at all.						 * get there at all.
 */								 */

static int ext3_block_to_path(struct inode *inode,		static int ext3_block_to_path(struct inode *inode,
			long i_block, int offsets[4], int *bo				long i_block, int offsets[4], int *bo
{								{
	int ptrs = EXT3_ADDR_PER_BLOCK(inode->i_sb);			int ptrs = EXT3_ADDR_PER_BLOCK(inode->i_sb);
	int ptrs_bits = EXT3_ADDR_PER_BLOCK_BITS(inode->i_sb)		int ptrs_bits = EXT3_ADDR_PER_BLOCK_BITS(inode->i_sb)
	const long direct_blocks = EXT3_NDIR_BLOCKS,			const long direct_blocks = EXT3_NDIR_BLOCKS,
		indirect_blocks = ptrs,						indirect_blocks = ptrs,
		double_blocks = (1 << (ptrs_bits * 2));				double_blocks = (1 << (ptrs_bits * 2));
	int n = 0;							int n = 0;
	int final = 0;							int final = 0;

	if (i_block < 0) {						if (i_block < 0) {
		ext3_warning (inode->i_sb, "ext3_block_to_pat			ext3_warning (inode->i_sb, "ext3_block_to_pat
	} else if (i_block < direct_blocks) {				} else if (i_block < direct_blocks) {
		offsets[n++] = i_block;						offsets[n++] = i_block;
		final = direct_blocks;						final = direct_blocks;
	} else if ( (i_block -= direct_blocks) < indirect_blo		} else if ( (i_block -= direct_blocks) < indirect_blo
		offsets[n++] = EXT3_IND_BLOCK;					offsets[n++] = EXT3_IND_BLOCK;
		offsets[n++] = i_block;						offsets[n++] = i_block;
		final = ptrs;							final = ptrs;
	} else if ((i_block -= indirect_blocks) < double_bloc		} else if ((i_block -= indirect_blocks) < double_bloc
		offsets[n++] = EXT3_DIND_BLOCK;					offsets[n++] = EXT3_DIND_BLOCK;
		offsets[n++] = i_block >> ptrs_bits;				offsets[n++] = i_block >> ptrs_bits;
		offsets[n++] = i_block & (ptrs - 1);				offsets[n++] = i_block & (ptrs - 1);
		final = ptrs;							final = ptrs;
	} else if (((i_block -= double_blocks) >> (ptrs_bits 		} else if (((i_block -= double_blocks) >> (ptrs_bits 
		offsets[n++] = EXT3_TIND_BLOCK;					offsets[n++] = EXT3_TIND_BLOCK;
		offsets[n++] = i_block >> (ptrs_bits * 2);			offsets[n++] = i_block >> (ptrs_bits * 2);
		offsets[n++] = (i_block >> ptrs_bits) & (ptrs			offsets[n++] = (i_block >> ptrs_bits) & (ptrs
		offsets[n++] = i_block & (ptrs - 1);				offsets[n++] = i_block & (ptrs - 1);
		final = ptrs;							final = ptrs;
	} else {							} else {
		ext3_warning(inode->i_sb, "ext3_block_to_path			ext3_warning(inode->i_sb, "ext3_block_to_path
	}								}
	if (boundary)							if (boundary)
		*boundary = final - 1 - (i_block & (ptrs - 1)			*boundary = final - 1 - (i_block & (ptrs - 1)
	return n;							return n;
}								}

/**								/**
 *	ext3_get_branch - read the chain of indirect blocks l	 *	ext3_get_branch - read the chain of indirect blocks l
 *	@inode: inode in question				 *	@inode: inode in question
 *	@depth: depth of the chain (1 - direct pointer, etc.)	 *	@depth: depth of the chain (1 - direct pointer, etc.)
 *	@offsets: offsets of pointers in inode/indirect block	 *	@offsets: offsets of pointers in inode/indirect block
 *	@chain: place to store the result			 *	@chain: place to store the result
 *	@err: here we store the error value			 *	@err: here we store the error value
 *								 *
 *	Function fills the array of triples <key, p, bh> and 	 *	Function fills the array of triples <key, p, bh> and 
 *	if everything went OK or the pointer to the last fill	 *	if everything went OK or the pointer to the last fill
 *	(incomplete one) otherwise. Upon the return chain[i].	 *	(incomplete one) otherwise. Upon the return chain[i].
 *	the number of (i+1)-th block in the chain (as it is s	 *	the number of (i+1)-th block in the chain (as it is s
 *	i.e. little-endian 32-bit), chain[i].p contains the a	 *	i.e. little-endian 32-bit), chain[i].p contains the a
 *	number (it points into struct inode for i==0 and into	 *	number (it points into struct inode for i==0 and into
 *	for i>0) and chain[i].bh points to the buffer_head of	 *	for i>0) and chain[i].bh points to the buffer_head of
 *	block for i>0 and NULL for i==0. In other words, it h	 *	block for i>0 and NULL for i==0. In other words, it h
 *	numbers of the chain, addresses they were taken from 	 *	numbers of the chain, addresses they were taken from 
 *	verify that chain did not change) and buffer_heads ho	 *	verify that chain did not change) and buffer_heads ho
 *	numbers.						 *	numbers.
 *								 *
 *	Function stops when it stumbles upon zero pointer (ab	 *	Function stops when it stumbles upon zero pointer (ab
 *		(pointer to last triple returned, *@err == 0)	 *		(pointer to last triple returned, *@err == 0)
 *	or when it gets an IO error reading an indirect block	 *	or when it gets an IO error reading an indirect block
 *		(ditto, *@err == -EIO)				 *		(ditto, *@err == -EIO)
 *	or when it notices that chain had been changed while 	 *	or when it notices that chain had been changed while 
 *		(ditto, *@err == -EAGAIN)			 *		(ditto, *@err == -EAGAIN)
 *	or when it reads all @depth-1 indirect blocks success	 *	or when it reads all @depth-1 indirect blocks success
 *	the whole chain, all way to the data (returns %NULL, 	 *	the whole chain, all way to the data (returns %NULL, 
 */								 */
static Indirect *ext3_get_branch(struct inode *inode, int dep	static Indirect *ext3_get_branch(struct inode *inode, int dep
				 Indirect chain[4], int *err)					 Indirect chain[4], int *err)
{								{
	struct super_block *sb = inode->i_sb;				struct super_block *sb = inode->i_sb;
	Indirect *p = chain;						Indirect *p = chain;
	struct buffer_head *bh;						struct buffer_head *bh;

	*err = 0;							*err = 0;
	/* i_data is not going away, no lock needed */			/* i_data is not going away, no lock needed */
	add_chain (chain, NULL, EXT3_I(inode)->i_data + *offs		add_chain (chain, NULL, EXT3_I(inode)->i_data + *offs
	if (!p->key)							if (!p->key)
		goto no_block;							goto no_block;
	while (--depth) {						while (--depth) {
		bh = sb_bread(sb, le32_to_cpu(p->key));				bh = sb_bread(sb, le32_to_cpu(p->key));
		if (!bh)							if (!bh)
			goto failure;							goto failure;
		/* Reader: pointers */						/* Reader: pointers */
		if (!verify_chain(chain, p))					if (!verify_chain(chain, p))
			goto changed;							goto changed;
		add_chain(++p, bh, (__le32*)bh->b_data + *++o			add_chain(++p, bh, (__le32*)bh->b_data + *++o
		/* Reader: end */						/* Reader: end */
		if (!p->key)							if (!p->key)
			goto no_block;							goto no_block;
	}								}
	return NULL;							return NULL;

changed:							changed:
	brelse(bh);							brelse(bh);
	*err = -EAGAIN;							*err = -EAGAIN;
	goto no_block;							goto no_block;
failure:							failure:
	*err = -EIO;							*err = -EIO;
no_block:							no_block:
	return p;							return p;
}								}

/**								/**
 *	ext3_find_near - find a place for allocation with suf	 *	ext3_find_near - find a place for allocation with suf
 *	@inode: owner						 *	@inode: owner
 *	@ind: descriptor of indirect block.			 *	@ind: descriptor of indirect block.
 *								 *
 *	This function returns the preferred place for block a	 *	This function returns the preferred place for block a
 *	It is used when heuristic for sequential allocation f	 *	It is used when heuristic for sequential allocation f
 *	Rules are:						 *	Rules are:
 *	  + if there is a block to the left of our position -	 *	  + if there is a block to the left of our position -
 *	  + if pointer will live in indirect block - allocate	 *	  + if pointer will live in indirect block - allocate
 *	  + if pointer will live in inode - allocate in the s	 *	  + if pointer will live in inode - allocate in the s
 *	    cylinder group.					 *	    cylinder group.
 *								 *
 * In the latter case we colour the starting block by the cal	 * In the latter case we colour the starting block by the cal
 * prevent it from clashing with concurrent allocations for a	 * prevent it from clashing with concurrent allocations for a
 * in the same block group.   The PID is used here so that fu	 * in the same block group.   The PID is used here so that fu
 * files will be close-by on-disk.				 * files will be close-by on-disk.
 *								 *
 *	Caller must make sure that @ind is valid and will sta	 *	Caller must make sure that @ind is valid and will sta
 */								 */
static ext3_fsblk_t ext3_find_near(struct inode *inode, Indir	static ext3_fsblk_t ext3_find_near(struct inode *inode, Indir
{								{
	struct ext3_inode_info *ei = EXT3_I(inode);			struct ext3_inode_info *ei = EXT3_I(inode);
	__le32 *start = ind->bh ? (__le32*) ind->bh->b_data :		__le32 *start = ind->bh ? (__le32*) ind->bh->b_data :
	__le32 *p;							__le32 *p;
	ext3_fsblk_t bg_start;						ext3_fsblk_t bg_start;
	ext3_grpblk_t colour;						ext3_grpblk_t colour;

	/* Try to find previous block */				/* Try to find previous block */
	for (p = ind->p - 1; p >= start; p--) {				for (p = ind->p - 1; p >= start; p--) {
		if (*p)								if (*p)
			return le32_to_cpu(*p);						return le32_to_cpu(*p);
	}								}

	/* No such thing, so let's try location of indirect b		/* No such thing, so let's try location of indirect b
	if (ind->bh)							if (ind->bh)
		return ind->bh->b_blocknr;					return ind->bh->b_blocknr;

	/*								/*
	 * It is going to be referred to from the inode itsel		 * It is going to be referred to from the inode itsel
	 * into the same cylinder group then.				 * into the same cylinder group then.
	 */								 */
	bg_start = ext3_group_first_block_no(inode->i_sb, ei-		bg_start = ext3_group_first_block_no(inode->i_sb, ei-
	colour = (current->pid % 16) *					colour = (current->pid % 16) *
			(EXT3_BLOCKS_PER_GROUP(inode->i_sb) /				(EXT3_BLOCKS_PER_GROUP(inode->i_sb) /
	return bg_start + colour;					return bg_start + colour;
}								}

/**								/**
 *	ext3_find_goal - find a preferred place for allocatio	 *	ext3_find_goal - find a preferred place for allocatio
 *	@inode: owner						 *	@inode: owner
 *	@block:  block we want					 *	@block:  block we want
 *	@partial: pointer to the last triple within a chain	 *	@partial: pointer to the last triple within a chain
 *								 *
 *	Normally this function find the preferred place for b	 *	Normally this function find the preferred place for b
 *	returns it.						 *	returns it.
 */								 */

static ext3_fsblk_t ext3_find_goal(struct inode *inode, long 	static ext3_fsblk_t ext3_find_goal(struct inode *inode, long 
				   Indirect *partial)						   Indirect *partial)
{								{
	struct ext3_block_alloc_info *block_i;				struct ext3_block_alloc_info *block_i;

	block_i =  EXT3_I(inode)->i_block_alloc_info;			block_i =  EXT3_I(inode)->i_block_alloc_info;

	/*								/*
	 * try the heuristic for sequential allocation,			 * try the heuristic for sequential allocation,
	 * failing that at least try to get decent locality.		 * failing that at least try to get decent locality.
	 */								 */
	if (block_i && (block == block_i->last_alloc_logical_		if (block_i && (block == block_i->last_alloc_logical_
		&& (block_i->last_alloc_physical_block != 0))			&& (block_i->last_alloc_physical_block != 0))
		return block_i->last_alloc_physical_block + 1			return block_i->last_alloc_physical_block + 1
	}								}

	return ext3_find_near(inode, partial);				return ext3_find_near(inode, partial);
}								}

/**								/**
 *	ext3_blks_to_allocate - Look up the block map and cou	 *	ext3_blks_to_allocate - Look up the block map and cou
 *	of direct blocks need to be allocated for the given b	 *	of direct blocks need to be allocated for the given b
 *								 *
 *	@branch: chain of indirect blocks			 *	@branch: chain of indirect blocks
 *	@k: number of blocks need for indirect blocks		 *	@k: number of blocks need for indirect blocks
 *	@blks: number of data blocks to be mapped.		 *	@blks: number of data blocks to be mapped.
 *	@blocks_to_boundary:  the offset in the indirect bloc	 *	@blocks_to_boundary:  the offset in the indirect bloc
 *								 *
 *	return the total number of blocks to be allocate, inc	 *	return the total number of blocks to be allocate, inc
 *	direct and indirect blocks.				 *	direct and indirect blocks.
 */								 */
static int ext3_blks_to_allocate(Indirect *branch, int k, uns	static int ext3_blks_to_allocate(Indirect *branch, int k, uns
		int blocks_to_boundary)						int blocks_to_boundary)
{								{
	unsigned long count = 0;					unsigned long count = 0;

	/*								/*
	 * Simple case, [t,d]Indirect block(s) has not alloca		 * Simple case, [t,d]Indirect block(s) has not alloca
	 * then it's clear blocks on that path have not alloc		 * then it's clear blocks on that path have not alloc
	 */								 */
	if (k > 0) {							if (k > 0) {
		/* right now we don't handle cross boundary a			/* right now we don't handle cross boundary a
		if (blks < blocks_to_boundary + 1)				if (blks < blocks_to_boundary + 1)
			count += blks;							count += blks;
		else								else
			count += blocks_to_boundary + 1;				count += blocks_to_boundary + 1;
		return count;							return count;
	}								}

	count++;							count++;
	while (count < blks && count <= blocks_to_boundary &&		while (count < blks && count <= blocks_to_boundary &&
		le32_to_cpu(*(branch[0].p + count)) == 0) {			le32_to_cpu(*(branch[0].p + count)) == 0) {
		count++;							count++;
	}								}
	return count;							return count;
}								}

/**								/**
 *	ext3_alloc_blocks - multiple allocate blocks needed f	 *	ext3_alloc_blocks - multiple allocate blocks needed f
 *	@handle: handle for this transaction			 *	@handle: handle for this transaction
 *	@inode: owner						 *	@inode: owner
 *	@goal: preferred place for allocation			 *	@goal: preferred place for allocation
 *	@indirect_blks: the number of blocks need to allocate	 *	@indirect_blks: the number of blocks need to allocate
 *			blocks					 *			blocks
 *	@blks:	number of blocks need to allocated for direct	 *	@blks:	number of blocks need to allocated for direct
 *	@new_blocks: on return it will store the new block nu	 *	@new_blocks: on return it will store the new block nu
 *	the indirect blocks(if needed) and the first direct b	 *	the indirect blocks(if needed) and the first direct b
 *	@err: here we store the error value			 *	@err: here we store the error value
 *								 *
 *	return the number of direct blocks allocated		 *	return the number of direct blocks allocated
 */								 */
static int ext3_alloc_blocks(handle_t *handle, struct inode *	static int ext3_alloc_blocks(handle_t *handle, struct inode *
			ext3_fsblk_t goal, int indirect_blks,				ext3_fsblk_t goal, int indirect_blks,
			ext3_fsblk_t new_blocks[4], int *err)				ext3_fsblk_t new_blocks[4], int *err)
{								{
	int target, i;							int target, i;
	unsigned long count = 0;					unsigned long count = 0;
	int index = 0;							int index = 0;
	ext3_fsblk_t current_block = 0;					ext3_fsblk_t current_block = 0;
	int ret = 0;							int ret = 0;

	/*								/*
	 * Here we try to allocate the requested multiple blo		 * Here we try to allocate the requested multiple blo
	 * on a best-effort basis.					 * on a best-effort basis.
	 * To build a branch, we should allocate blocks for		 * To build a branch, we should allocate blocks for
	 * the indirect blocks(if not allocated yet), and at 		 * the indirect blocks(if not allocated yet), and at 
	 * the first direct block of this branch.  That's the		 * the first direct block of this branch.  That's the
	 * minimum number of blocks need to allocate(required		 * minimum number of blocks need to allocate(required
	 */								 */
	target = blks + indirect_blks;					target = blks + indirect_blks;

	while (1) {							while (1) {
		count = target;							count = target;
		/* allocating blocks for indirect blocks and 			/* allocating blocks for indirect blocks and 
		current_block = ext3_new_blocks(handle,inode,			current_block = ext3_new_blocks(handle,inode,
		if (*err)							if (*err)
			goto failed_out;						goto failed_out;

		target -= count;						target -= count;
		/* allocate blocks for indirect blocks */			/* allocate blocks for indirect blocks */
		while (index < indirect_blks && count) {			while (index < indirect_blks && count) {
			new_blocks[index++] = current_block++				new_blocks[index++] = current_block++
			count--;							count--;
		}								}

		if (count > 0)							if (count > 0)
			break;								break;
	}								}

	/* save the new block number for the first direct blo		/* save the new block number for the first direct blo
	new_blocks[index] = current_block;				new_blocks[index] = current_block;

	/* total number of blocks allocated for direct blocks		/* total number of blocks allocated for direct blocks
	ret = count;							ret = count;
	*err = 0;							*err = 0;
	return ret;							return ret;
failed_out:							failed_out:
	for (i = 0; i <index; i++)					for (i = 0; i <index; i++)
		ext3_free_blocks(handle, inode, new_blocks[i]			ext3_free_blocks(handle, inode, new_blocks[i]
	return ret;							return ret;
}								}

/**								/**
 *	ext3_alloc_branch - allocate and set up a chain of bl	 *	ext3_alloc_branch - allocate and set up a chain of bl
 *	@handle: handle for this transaction			 *	@handle: handle for this transaction
 *	@inode: owner						 *	@inode: owner
 *	@indirect_blks: number of allocated indirect blocks	 *	@indirect_blks: number of allocated indirect blocks
 *	@blks: number of allocated direct blocks		 *	@blks: number of allocated direct blocks
 *	@goal: preferred place for allocation			 *	@goal: preferred place for allocation
 *	@offsets: offsets (in the blocks) to store the pointe	 *	@offsets: offsets (in the blocks) to store the pointe
 *	@branch: place to store the chain in.			 *	@branch: place to store the chain in.
 *								 *
 *	This function allocates blocks, zeroes out all but th	 *	This function allocates blocks, zeroes out all but th
 *	links them into chain and (if we are synchronous) wri	 *	links them into chain and (if we are synchronous) wri
 *	In other words, it prepares a branch that can be spli	 *	In other words, it prepares a branch that can be spli
 *	inode. It stores the information about that chain in 	 *	inode. It stores the information about that chain in 
 *	the same format as ext3_get_branch() would do. We are	 *	the same format as ext3_get_branch() would do. We are
 *	we had read the existing part of chain and partial po	 *	we had read the existing part of chain and partial po
 *	triple of that (one with zero ->key). Upon the exit w	 *	triple of that (one with zero ->key). Upon the exit w
 *	picture as after the successful ext3_get_block(), exc	 *	picture as after the successful ext3_get_block(), exc
 *	place chain is disconnected - *branch->p is still zer	 *	place chain is disconnected - *branch->p is still zer
 *	set the last link), but branch->key contains the numb	 *	set the last link), but branch->key contains the numb
 *	be placed into *branch->p to fill that gap.		 *	be placed into *branch->p to fill that gap.
 *								 *
 *	If allocation fails we free all blocks we've allocate	 *	If allocation fails we free all blocks we've allocate
 *	their buffer_heads) and return the error value the fr	 *	their buffer_heads) and return the error value the fr
 *	ext3_alloc_block() (normally -ENOSPC). Otherwise we s	 *	ext3_alloc_block() (normally -ENOSPC). Otherwise we s
 *	as described above and return 0.			 *	as described above and return 0.
 */								 */
static int ext3_alloc_branch(handle_t *handle, struct inode *	static int ext3_alloc_branch(handle_t *handle, struct inode *
			int indirect_blks, int *blks, ext3_fs				int indirect_blks, int *blks, ext3_fs
			int *offsets, Indirect *branch)					int *offsets, Indirect *branch)
{								{
	int blocksize = inode->i_sb->s_blocksize;			int blocksize = inode->i_sb->s_blocksize;
	int i, n = 0;							int i, n = 0;
	int err = 0;							int err = 0;
	struct buffer_head *bh;						struct buffer_head *bh;
	int num;							int num;
	ext3_fsblk_t new_blocks[4];					ext3_fsblk_t new_blocks[4];
	ext3_fsblk_t current_block;					ext3_fsblk_t current_block;

	num = ext3_alloc_blocks(handle, inode, goal, indirect		num = ext3_alloc_blocks(handle, inode, goal, indirect
				*blks, new_blocks, &err);					*blks, new_blocks, &err);
	if (err)							if (err)
		return err;							return err;

	branch[0].key = cpu_to_le32(new_blocks[0]);			branch[0].key = cpu_to_le32(new_blocks[0]);
	/*								/*
	 * metadata blocks and data blocks are allocated.		 * metadata blocks and data blocks are allocated.
	 */								 */
	for (n = 1; n <= indirect_blks;  n++) {				for (n = 1; n <= indirect_blks;  n++) {
		/*								/*
		 * Get buffer_head for parent block, zero it 			 * Get buffer_head for parent block, zero it 
		 * and set the pointer to new one, then send			 * and set the pointer to new one, then send
		 * parent to disk.						 * parent to disk.
		 */								 */
		bh = sb_getblk(inode->i_sb, new_blocks[n-1]);			bh = sb_getblk(inode->i_sb, new_blocks[n-1]);
		branch[n].bh = bh;						branch[n].bh = bh;
		lock_buffer(bh);						lock_buffer(bh);
		BUFFER_TRACE(bh, "call get_create_access");			BUFFER_TRACE(bh, "call get_create_access");
		err = ext3_journal_get_create_access(handle, 			err = ext3_journal_get_create_access(handle, 
		if (err) {							if (err) {
			unlock_buffer(bh);						unlock_buffer(bh);
			brelse(bh);							brelse(bh);
			goto failed;							goto failed;
		}								}

		memset(bh->b_data, 0, blocksize);				memset(bh->b_data, 0, blocksize);
		branch[n].p = (__le32 *) bh->b_data + offsets			branch[n].p = (__le32 *) bh->b_data + offsets
		branch[n].key = cpu_to_le32(new_blocks[n]);			branch[n].key = cpu_to_le32(new_blocks[n]);
		*branch[n].p = branch[n].key;					*branch[n].p = branch[n].key;
		if ( n == indirect_blks) {					if ( n == indirect_blks) {
			current_block = new_blocks[n];					current_block = new_blocks[n];
			/*								/*
			 * End of chain, update the last new 				 * End of chain, update the last new 
			 * the chain to point to the new allo				 * the chain to point to the new allo
			 * data blocks numbers						 * data blocks numbers
			 */								 */
			for (i=1; i < num; i++)						for (i=1; i < num; i++)
				*(branch[n].p + i) = cpu_to_l					*(branch[n].p + i) = cpu_to_l
		}								}
		BUFFER_TRACE(bh, "marking uptodate");				BUFFER_TRACE(bh, "marking uptodate");
		set_buffer_uptodate(bh);					set_buffer_uptodate(bh);
		unlock_buffer(bh);						unlock_buffer(bh);

		BUFFER_TRACE(bh, "call ext3_journal_dirty_met			BUFFER_TRACE(bh, "call ext3_journal_dirty_met
		err = ext3_journal_dirty_metadata(handle, bh)			err = ext3_journal_dirty_metadata(handle, bh)
		if (err)							if (err)
			goto failed;							goto failed;
	}								}
	*blks = num;							*blks = num;
	return err;							return err;
failed:								failed:
	/* Allocation failed, free what we already allocated 		/* Allocation failed, free what we already allocated 
	for (i = 1; i <= n ; i++) {					for (i = 1; i <= n ; i++) {
		BUFFER_TRACE(branch[i].bh, "call journal_forg			BUFFER_TRACE(branch[i].bh, "call journal_forg
		ext3_journal_forget(handle, branch[i].bh);			ext3_journal_forget(handle, branch[i].bh);
	}								}
	for (i = 0; i <indirect_blks; i++)				for (i = 0; i <indirect_blks; i++)
		ext3_free_blocks(handle, inode, new_blocks[i]			ext3_free_blocks(handle, inode, new_blocks[i]

	ext3_free_blocks(handle, inode, new_blocks[i], num);		ext3_free_blocks(handle, inode, new_blocks[i], num);

	return err;							return err;
}								}

/**								/**
 * ext3_splice_branch - splice the allocated branch onto inod	 * ext3_splice_branch - splice the allocated branch onto inod
 * @handle: handle for this transaction				 * @handle: handle for this transaction
 * @inode: owner						 * @inode: owner
 * @block: (logical) number of block we are adding		 * @block: (logical) number of block we are adding
 * @where: location of missing link				 * @where: location of missing link
 * @num:   number of indirect blocks we are adding		 * @num:   number of indirect blocks we are adding
 * @blks:  number of direct blocks we are adding		 * @blks:  number of direct blocks we are adding
 *								 *
 * This function fills the missing link and does all housekee	 * This function fills the missing link and does all housekee
 * inode (->i_blocks, etc.). In case of success we end up wit	 * inode (->i_blocks, etc.). In case of success we end up wit
 * chain to new block and return 0.				 * chain to new block and return 0.
 */								 */
static int ext3_splice_branch(handle_t *handle, struct inode 	static int ext3_splice_branch(handle_t *handle, struct inode 
			long block, Indirect *where, int num,				long block, Indirect *where, int num,
{								{
	int i;								int i;
	int err = 0;							int err = 0;
	struct ext3_block_alloc_info *block_i;				struct ext3_block_alloc_info *block_i;
	ext3_fsblk_t current_block;					ext3_fsblk_t current_block;
	struct ext3_inode_info *ei = EXT3_I(inode);			struct ext3_inode_info *ei = EXT3_I(inode);
	struct timespec now;						struct timespec now;

	block_i = ei->i_block_alloc_info;				block_i = ei->i_block_alloc_info;
	/*								/*
	 * If we're splicing into a [td]indirect block (as op		 * If we're splicing into a [td]indirect block (as op
	 * inode) then we need to get write access to the [td		 * inode) then we need to get write access to the [td
	 * before the splice.						 * before the splice.
	 */								 */
	if (where->bh) {						if (where->bh) {
		BUFFER_TRACE(where->bh, "get_write_access");			BUFFER_TRACE(where->bh, "get_write_access");
		err = ext3_journal_get_write_access(handle, w			err = ext3_journal_get_write_access(handle, w
		if (err)							if (err)
			goto err_out;							goto err_out;
	}								}
	/* That's it */							/* That's it */

	*where->p = where->key;						*where->p = where->key;

	/*								/*
	 * Update the host buffer_head or inode to point to m		 * Update the host buffer_head or inode to point to m
	 * direct blocks blocks						 * direct blocks blocks
	 */								 */
	if (num == 0 && blks > 1) {					if (num == 0 && blks > 1) {
		current_block = le32_to_cpu(where->key) + 1;			current_block = le32_to_cpu(where->key) + 1;
		for (i = 1; i < blks; i++)					for (i = 1; i < blks; i++)
			*(where->p + i ) = cpu_to_le32(curren				*(where->p + i ) = cpu_to_le32(curren
	}								}

	/*								/*
	 * update the most recently allocated logical & physi		 * update the most recently allocated logical & physi
	 * in i_block_alloc_info, to assist find the proper g		 * in i_block_alloc_info, to assist find the proper g
	 * allocation							 * allocation
	 */								 */
	if (block_i) {							if (block_i) {
		block_i->last_alloc_logical_block = block + b			block_i->last_alloc_logical_block = block + b
		block_i->last_alloc_physical_block =				block_i->last_alloc_physical_block =
				le32_to_cpu(where[num].key) +					le32_to_cpu(where[num].key) +
	}								}

	/* We are done with atomic stuff, now do the rest of 		/* We are done with atomic stuff, now do the rest of 
	now = CURRENT_TIME_SEC;						now = CURRENT_TIME_SEC;
	if (!timespec_equal(&inode->i_ctime, &now) || !where-		if (!timespec_equal(&inode->i_ctime, &now) || !where-
		inode->i_ctime = now;						inode->i_ctime = now;
		ext3_mark_inode_dirty(handle, inode);				ext3_mark_inode_dirty(handle, inode);
	}								}
	/* ext3_mark_inode_dirty already updated i_sync_tid *		/* ext3_mark_inode_dirty already updated i_sync_tid *
	atomic_set(&ei->i_datasync_tid, handle->h_transaction		atomic_set(&ei->i_datasync_tid, handle->h_transaction

	/* had we spliced it onto indirect block? */			/* had we spliced it onto indirect block? */
	if (where->bh) {						if (where->bh) {
		/*								/*
		 * If we spliced it onto an indirect block, w			 * If we spliced it onto an indirect block, w
		 * altered the inode.  Note however that if i			 * altered the inode.  Note however that if i
		 * onto an indirect block at the very end of 			 * onto an indirect block at the very end of 
		 * file is growing) then we *will* alter the 			 * file is growing) then we *will* alter the 
		 * the new i_size.  But that is not done here			 * the new i_size.  But that is not done here
		 * generic_commit_write->__mark_inode_dirty->			 * generic_commit_write->__mark_inode_dirty->
		 */								 */
		jbd_debug(5, "splicing indirect only\n");			jbd_debug(5, "splicing indirect only\n");
		BUFFER_TRACE(where->bh, "call ext3_journal_di			BUFFER_TRACE(where->bh, "call ext3_journal_di
		err = ext3_journal_dirty_metadata(handle, whe			err = ext3_journal_dirty_metadata(handle, whe
		if (err)							if (err)
			goto err_out;							goto err_out;
	} else {							} else {
		/*								/*
		 * OK, we spliced it into the inode itself on			 * OK, we spliced it into the inode itself on
		 * Inode was dirtied above.					 * Inode was dirtied above.
		 */								 */
		jbd_debug(5, "splicing direct\n");				jbd_debug(5, "splicing direct\n");
	}								}
	return err;							return err;

err_out:							err_out:
	for (i = 1; i <= num; i++) {					for (i = 1; i <= num; i++) {
		BUFFER_TRACE(where[i].bh, "call journal_forge			BUFFER_TRACE(where[i].bh, "call journal_forge
		ext3_journal_forget(handle, where[i].bh);			ext3_journal_forget(handle, where[i].bh);
		ext3_free_blocks(handle,inode,le32_to_cpu(whe			ext3_free_blocks(handle,inode,le32_to_cpu(whe
	}								}
	ext3_free_blocks(handle, inode, le32_to_cpu(where[num		ext3_free_blocks(handle, inode, le32_to_cpu(where[num

	return err;							return err;
}								}

/*								/*
 * Allocation strategy is simple: if we have to allocate some	 * Allocation strategy is simple: if we have to allocate some
 * have to go the whole way to leaf. So let's do it before at	 * have to go the whole way to leaf. So let's do it before at
 * to tree, set linkage between the newborn blocks, write the	 * to tree, set linkage between the newborn blocks, write the
 * required, recheck the path, free and repeat if check fails	 * required, recheck the path, free and repeat if check fails
 * set the last missing link (that will protect us from any t	 * set the last missing link (that will protect us from any t
 * removals - all blocks on the path are immune now) and poss	 * removals - all blocks on the path are immune now) and poss
 * write on the parent block.					 * write on the parent block.
 * That has a nice additional property: no special recovery f	 * That has a nice additional property: no special recovery f
 * allocations is needed - we simply release blocks and do no	 * allocations is needed - we simply release blocks and do no
 * reachable from inode.					 * reachable from inode.
 *								 *
 * `handle' can be NULL if create == 0.				 * `handle' can be NULL if create == 0.
 *								 *
 * The BKL may not be held on entry here.  Be sure to take it	 * The BKL may not be held on entry here.  Be sure to take it
 * return > 0, # of blocks mapped or allocated.			 * return > 0, # of blocks mapped or allocated.
 * return = 0, if plain lookup failed.				 * return = 0, if plain lookup failed.
 * return < 0, error case.					 * return < 0, error case.
 */								 */
int ext3_get_blocks_handle(handle_t *handle, struct inode *in	int ext3_get_blocks_handle(handle_t *handle, struct inode *in
		sector_t iblock, unsigned long maxblocks,			sector_t iblock, unsigned long maxblocks,
		struct buffer_head *bh_result,					struct buffer_head *bh_result,
		int create)							int create)
{								{
	int err = -EIO;							int err = -EIO;
	int offsets[4];							int offsets[4];
	Indirect chain[4];						Indirect chain[4];
	Indirect *partial;						Indirect *partial;
	ext3_fsblk_t goal;						ext3_fsblk_t goal;
	int indirect_blks;						int indirect_blks;
	int blocks_to_boundary = 0;					int blocks_to_boundary = 0;
	int depth;							int depth;
	struct ext3_inode_info *ei = EXT3_I(inode);			struct ext3_inode_info *ei = EXT3_I(inode);
	int count = 0;							int count = 0;
	ext3_fsblk_t first_block = 0;					ext3_fsblk_t first_block = 0;


	trace_ext3_get_blocks_enter(inode, iblock, maxblocks,		trace_ext3_get_blocks_enter(inode, iblock, maxblocks,
	J_ASSERT(handle != NULL || create == 0);			J_ASSERT(handle != NULL || create == 0);
	depth = ext3_block_to_path(inode,iblock,offsets,&bloc		depth = ext3_block_to_path(inode,iblock,offsets,&bloc

	if (depth == 0)							if (depth == 0)
		goto out;							goto out;

	partial = ext3_get_branch(inode, depth, offsets, chai		partial = ext3_get_branch(inode, depth, offsets, chai

	/* Simplest case - block found, no allocation needed 		/* Simplest case - block found, no allocation needed 
	if (!partial) {							if (!partial) {
		first_block = le32_to_cpu(chain[depth - 1].ke			first_block = le32_to_cpu(chain[depth - 1].ke
		clear_buffer_new(bh_result);					clear_buffer_new(bh_result);
		count++;							count++;
		/*map more blocks*/						/*map more blocks*/
		while (count < maxblocks && count <= blocks_t			while (count < maxblocks && count <= blocks_t
			ext3_fsblk_t blk;						ext3_fsblk_t blk;

			if (!verify_chain(chain, chain + dept				if (!verify_chain(chain, chain + dept
				/*								/*
				 * Indirect block might be re					 * Indirect block might be re
				 * truncate while we were rea					 * truncate while we were rea
				 * Handling of that case: for					 * Handling of that case: for
				 * got now. Flag the err as E					 * got now. Flag the err as E
				 * will reread.							 * will reread.
				 */								 */
				err = -EAGAIN;							err = -EAGAIN;
				count = 0;							count = 0;
				break;								break;
			}								}
			blk = le32_to_cpu(*(chain[depth-1].p 				blk = le32_to_cpu(*(chain[depth-1].p 

			if (blk == first_block + count)					if (blk == first_block + count)
				count++;							count++;
			else								else
				break;								break;
		}								}
		if (err != -EAGAIN)						if (err != -EAGAIN)
			goto got_it;							goto got_it;
	}								}

	/* Next simple case - plain lookup or failed read of 		/* Next simple case - plain lookup or failed read of 
	if (!create || err == -EIO)					if (!create || err == -EIO)
		goto cleanup;							goto cleanup;

	/*								/*
	 * Block out ext3_truncate while we alter the tree		 * Block out ext3_truncate while we alter the tree
	 */								 */
	mutex_lock(&ei->truncate_mutex);				mutex_lock(&ei->truncate_mutex);

	/*								/*
	 * If the indirect block is missing while we are read		 * If the indirect block is missing while we are read
	 * the chain(ext3_get_branch() returns -EAGAIN err), 		 * the chain(ext3_get_branch() returns -EAGAIN err), 
	 * if the chain has been changed after we grab the se		 * if the chain has been changed after we grab the se
	 * (either because another process truncated this bra		 * (either because another process truncated this bra
	 * another get_block allocated this branch) re-grab t		 * another get_block allocated this branch) re-grab t
	 * the request block has been allocated or not.			 * the request block has been allocated or not.
	 *								 *
	 * Since we already block the truncate/other get_bloc		 * Since we already block the truncate/other get_bloc
	 * at this point, we will have the current copy of th		 * at this point, we will have the current copy of th
	 * splice the branch into the tree.				 * splice the branch into the tree.
	 */								 */
	if (err == -EAGAIN || !verify_chain(chain, partial)) 		if (err == -EAGAIN || !verify_chain(chain, partial)) 
		while (partial > chain) {					while (partial > chain) {
			brelse(partial->bh);						brelse(partial->bh);
			partial--;							partial--;
		}								}
		partial = ext3_get_branch(inode, depth, offse			partial = ext3_get_branch(inode, depth, offse
		if (!partial) {							if (!partial) {
			count++;							count++;
			mutex_unlock(&ei->truncate_mutex);				mutex_unlock(&ei->truncate_mutex);
			if (err)							if (err)
				goto cleanup;							goto cleanup;
			clear_buffer_new(bh_result);					clear_buffer_new(bh_result);
			goto got_it;							goto got_it;
		}								}
	}								}

	/*								/*
	 * Okay, we need to do block allocation.  Lazily init		 * Okay, we need to do block allocation.  Lazily init
	 * allocation info here if necessary				 * allocation info here if necessary
	*/								*/
	if (S_ISREG(inode->i_mode) && (!ei->i_block_alloc_inf		if (S_ISREG(inode->i_mode) && (!ei->i_block_alloc_inf
		ext3_init_block_alloc_info(inode);				ext3_init_block_alloc_info(inode);

	goal = ext3_find_goal(inode, iblock, partial);			goal = ext3_find_goal(inode, iblock, partial);

	/* the number of blocks need to allocate for [d,t]ind		/* the number of blocks need to allocate for [d,t]ind
	indirect_blks = (chain + depth) - partial - 1;			indirect_blks = (chain + depth) - partial - 1;

	/*								/*
	 * Next look up the indirect map to count the totoal 		 * Next look up the indirect map to count the totoal 
	 * direct blocks to allocate for this branch.			 * direct blocks to allocate for this branch.
	 */								 */
	count = ext3_blks_to_allocate(partial, indirect_blks,		count = ext3_blks_to_allocate(partial, indirect_blks,
					maxblocks, blocks_to_						maxblocks, blocks_to_
	err = ext3_alloc_branch(handle, inode, indirect_blks,		err = ext3_alloc_branch(handle, inode, indirect_blks,
				offsets + (partial - chain), 					offsets + (partial - chain), 

	/*								/*
	 * The ext3_splice_branch call will free and forget a		 * The ext3_splice_branch call will free and forget a
	 * on the new chain if there is a failure, but that r		 * on the new chain if there is a failure, but that r
	 * up transaction credits, especially for bitmaps whe		 * up transaction credits, especially for bitmaps whe
	 * credits cannot be returned.  Can we handle this so		 * credits cannot be returned.  Can we handle this so
	 * may need to return -EAGAIN upwards in the worst ca		 * may need to return -EAGAIN upwards in the worst ca
	 */								 */
	if (!err)							if (!err)
		err = ext3_splice_branch(handle, inode, ibloc			err = ext3_splice_branch(handle, inode, ibloc
					partial, indirect_blk						partial, indirect_blk
	mutex_unlock(&ei->truncate_mutex);				mutex_unlock(&ei->truncate_mutex);
	if (err)							if (err)
		goto cleanup;							goto cleanup;

	set_buffer_new(bh_result);					set_buffer_new(bh_result);
got_it:								got_it:
	map_bh(bh_result, inode->i_sb, le32_to_cpu(chain[dept		map_bh(bh_result, inode->i_sb, le32_to_cpu(chain[dept
	if (count > blocks_to_boundary)					if (count > blocks_to_boundary)
		set_buffer_boundary(bh_result);					set_buffer_boundary(bh_result);
	err = count;							err = count;
	/* Clean up and exit */						/* Clean up and exit */
	partial = chain + depth - 1;	/* the whole chain */		partial = chain + depth - 1;	/* the whole chain */
cleanup:							cleanup:
	while (partial > chain) {					while (partial > chain) {
		BUFFER_TRACE(partial->bh, "call brelse");			BUFFER_TRACE(partial->bh, "call brelse");
		brelse(partial->bh);						brelse(partial->bh);
		partial--;							partial--;
	}								}
	BUFFER_TRACE(bh_result, "returned");				BUFFER_TRACE(bh_result, "returned");
out:								out:
	trace_ext3_get_blocks_exit(inode, iblock,			trace_ext3_get_blocks_exit(inode, iblock,
				   depth ? le32_to_cpu(chain[					   depth ? le32_to_cpu(chain[
				   count, err);							   count, err);
	return err;							return err;
}								}

/* Maximum number of blocks we map for direct IO at once. */	/* Maximum number of blocks we map for direct IO at once. */
#define DIO_MAX_BLOCKS 4096					#define DIO_MAX_BLOCKS 4096
/*								/*
 * Number of credits we need for writing DIO_MAX_BLOCKS:	 * Number of credits we need for writing DIO_MAX_BLOCKS:
 * We need sb + group descriptor + bitmap + inode -> 4		 * We need sb + group descriptor + bitmap + inode -> 4
 * For B blocks with A block pointers per block we need:	 * For B blocks with A block pointers per block we need:
 * 1 (triple ind.) + (B/A/A + 2) (doubly ind.) + (B/A + 2) (i	 * 1 (triple ind.) + (B/A/A + 2) (doubly ind.) + (B/A + 2) (i
 * If we plug in 4096 for B and 256 for A (for 1KB block size	 * If we plug in 4096 for B and 256 for A (for 1KB block size
 */								 */
#define DIO_CREDITS 25						#define DIO_CREDITS 25

static int ext3_get_block(struct inode *inode, sector_t ibloc	static int ext3_get_block(struct inode *inode, sector_t ibloc
			struct buffer_head *bh_result, int cr				struct buffer_head *bh_result, int cr
{								{
	handle_t *handle = ext3_journal_current_handle();		handle_t *handle = ext3_journal_current_handle();
	int ret = 0, started = 0;					int ret = 0, started = 0;
	unsigned max_blocks = bh_result->b_size >> inode->i_b		unsigned max_blocks = bh_result->b_size >> inode->i_b

	if (create && !handle) {	/* Direct IO write...		if (create && !handle) {	/* Direct IO write...
		if (max_blocks > DIO_MAX_BLOCKS)				if (max_blocks > DIO_MAX_BLOCKS)
			max_blocks = DIO_MAX_BLOCKS;					max_blocks = DIO_MAX_BLOCKS;
		handle = ext3_journal_start(inode, DIO_CREDIT			handle = ext3_journal_start(inode, DIO_CREDIT
				EXT3_MAXQUOTAS_TRANS_BLOCKS(i					EXT3_MAXQUOTAS_TRANS_BLOCKS(i
		if (IS_ERR(handle)) {						if (IS_ERR(handle)) {
			ret = PTR_ERR(handle);						ret = PTR_ERR(handle);
			goto out;							goto out;
		}								}
		started = 1;							started = 1;
	}								}

	ret = ext3_get_blocks_handle(handle, inode, iblock,		ret = ext3_get_blocks_handle(handle, inode, iblock,
					max_blocks, bh_result						max_blocks, bh_result
	if (ret > 0) {							if (ret > 0) {
		bh_result->b_size = (ret << inode->i_blkbits)			bh_result->b_size = (ret << inode->i_blkbits)
		ret = 0;							ret = 0;
	}								}
	if (started)							if (started)
		ext3_journal_stop(handle);					ext3_journal_stop(handle);
out:								out:
	return ret;							return ret;
}								}

int ext3_fiemap(struct inode *inode, struct fiemap_extent_inf	int ext3_fiemap(struct inode *inode, struct fiemap_extent_inf
		u64 start, u64 len)						u64 start, u64 len)
{								{
	return generic_block_fiemap(inode, fieinfo, start, le		return generic_block_fiemap(inode, fieinfo, start, le
				    ext3_get_block);						    ext3_get_block);
}								}

/*								/*
 * `handle' can be NULL if create is zero			 * `handle' can be NULL if create is zero
 */								 */
struct buffer_head *ext3_getblk(handle_t *handle, struct inod	struct buffer_head *ext3_getblk(handle_t *handle, struct inod
				long block, int create, int *					long block, int create, int *
{								{
	struct buffer_head dummy;					struct buffer_head dummy;
	int fatal = 0, err;						int fatal = 0, err;

	J_ASSERT(handle != NULL || create == 0);			J_ASSERT(handle != NULL || create == 0);

	dummy.b_state = 0;						dummy.b_state = 0;
	dummy.b_blocknr = -1000;					dummy.b_blocknr = -1000;
	buffer_trace_init(&dummy.b_history);				buffer_trace_init(&dummy.b_history);
	err = ext3_get_blocks_handle(handle, inode, block, 1,		err = ext3_get_blocks_handle(handle, inode, block, 1,
					&dummy, create);						&dummy, create);
	/*								/*
	 * ext3_get_blocks_handle() returns number of blocks		 * ext3_get_blocks_handle() returns number of blocks
	 * mapped. 0 in case of a HOLE.					 * mapped. 0 in case of a HOLE.
	 */								 */
	if (err > 0) {							if (err > 0) {
		WARN_ON(err > 1);						WARN_ON(err > 1);
		err = 0;							err = 0;
	}								}
	*errp = err;							*errp = err;
	if (!err && buffer_mapped(&dummy)) {				if (!err && buffer_mapped(&dummy)) {
		struct buffer_head *bh;						struct buffer_head *bh;
		bh = sb_getblk(inode->i_sb, dummy.b_blocknr);			bh = sb_getblk(inode->i_sb, dummy.b_blocknr);
		if (!bh) {							if (!bh) {
			*errp = -EIO;							*errp = -EIO;
			goto err;							goto err;
		}								}
		if (buffer_new(&dummy)) {					if (buffer_new(&dummy)) {
			J_ASSERT(create != 0);						J_ASSERT(create != 0);
			J_ASSERT(handle != NULL);					J_ASSERT(handle != NULL);

			/*								/*
			 * Now that we do not always journal 				 * Now that we do not always journal 
			 * keep in mind whether this should a				 * keep in mind whether this should a
			 * new buffer as metadata.  For now, 				 * new buffer as metadata.  For now, 
			 * writes use ext3_get_block instead,				 * writes use ext3_get_block instead,
			 * problem.							 * problem.
			 */								 */
			lock_buffer(bh);						lock_buffer(bh);
			BUFFER_TRACE(bh, "call get_create_acc				BUFFER_TRACE(bh, "call get_create_acc
			fatal = ext3_journal_get_create_acces				fatal = ext3_journal_get_create_acces
			if (!fatal && !buffer_uptodate(bh)) {				if (!fatal && !buffer_uptodate(bh)) {
				memset(bh->b_data,0,inode->i_					memset(bh->b_data,0,inode->i_
				set_buffer_uptodate(bh);					set_buffer_uptodate(bh);
			}								}
			unlock_buffer(bh);						unlock_buffer(bh);
			BUFFER_TRACE(bh, "call ext3_journal_d				BUFFER_TRACE(bh, "call ext3_journal_d
			err = ext3_journal_dirty_metadata(han				err = ext3_journal_dirty_metadata(han
			if (!fatal)							if (!fatal)
				fatal = err;							fatal = err;
		} else {							} else {
			BUFFER_TRACE(bh, "not a new buffer");				BUFFER_TRACE(bh, "not a new buffer");
		}								}
		if (fatal) {							if (fatal) {
			*errp = fatal;							*errp = fatal;
			brelse(bh);							brelse(bh);
			bh = NULL;							bh = NULL;
		}								}
		return bh;							return bh;
	}								}
err:								err:
	return NULL;							return NULL;
}								}

struct buffer_head *ext3_bread(handle_t *handle, struct inode	struct buffer_head *ext3_bread(handle_t *handle, struct inode
			       int block, int create, int *er				       int block, int create, int *er
{								{
	struct buffer_head * bh;					struct buffer_head * bh;

	bh = ext3_getblk(handle, inode, block, create, err);		bh = ext3_getblk(handle, inode, block, create, err);
	if (!bh)							if (!bh)
		return bh;							return bh;
	if (bh_uptodate_or_lock(bh))					if (bh_uptodate_or_lock(bh))
		return bh;							return bh;
	get_bh(bh);							get_bh(bh);
	bh->b_end_io = end_buffer_read_sync;				bh->b_end_io = end_buffer_read_sync;
	submit_bh(READ | REQ_META | REQ_PRIO, bh);			submit_bh(READ | REQ_META | REQ_PRIO, bh);
	wait_on_buffer(bh);						wait_on_buffer(bh);
	if (buffer_uptodate(bh))					if (buffer_uptodate(bh))
		return bh;							return bh;
	put_bh(bh);							put_bh(bh);
	*err = -EIO;							*err = -EIO;
	return NULL;							return NULL;
}								}

static int walk_page_buffers(	handle_t *handle,		static int walk_page_buffers(	handle_t *handle,
				struct buffer_head *head,					struct buffer_head *head,
				unsigned from,							unsigned from,
				unsigned to,							unsigned to,
				int *partial,							int *partial,
				int (*fn)(	handle_t *han					int (*fn)(	handle_t *han
						struct buffer							struct buffer
{								{
	struct buffer_head *bh;						struct buffer_head *bh;
	unsigned block_start, block_end;				unsigned block_start, block_end;
	unsigned blocksize = head->b_size;				unsigned blocksize = head->b_size;
	int err, ret = 0;						int err, ret = 0;
	struct buffer_head *next;					struct buffer_head *next;

	for (	bh = head, block_start = 0;				for (	bh = head, block_start = 0;
		ret == 0 && (bh != head || !block_start);			ret == 0 && (bh != head || !block_start);
		block_start = block_end, bh = next)				block_start = block_end, bh = next)
	{								{
		next = bh->b_this_page;						next = bh->b_this_page;
		block_end = block_start + blocksize;				block_end = block_start + blocksize;
		if (block_end <= from || block_start >= to) {			if (block_end <= from || block_start >= to) {
			if (partial && !buffer_uptodate(bh))				if (partial && !buffer_uptodate(bh))
				*partial = 1;							*partial = 1;
			continue;							continue;
		}								}
		err = (*fn)(handle, bh);					err = (*fn)(handle, bh);
		if (!ret)							if (!ret)
			ret = err;							ret = err;
	}								}
	return ret;							return ret;
}								}

/*								/*
 * To preserve ordering, it is essential that the hole instan	 * To preserve ordering, it is essential that the hole instan
 * the data write be encapsulated in a single transaction.  W	 * the data write be encapsulated in a single transaction.  W
 * close off a transaction and start a new one between the ex	 * close off a transaction and start a new one between the ex
 * and the commit_write().  So doing the journal_start at the	 * and the commit_write().  So doing the journal_start at the
 * prepare_write() is the right place.				 * prepare_write() is the right place.
 *								 *
 * Also, this function can nest inside ext3_writepage() ->	 * Also, this function can nest inside ext3_writepage() ->
 * block_write_full_page(). In that case, we *know* that ext3	 * block_write_full_page(). In that case, we *know* that ext3
 * has generated enough buffer credits to do the whole page. 	 * has generated enough buffer credits to do the whole page. 
 * block on the journal in that case, which is good, because 	 * block on the journal in that case, which is good, because 
 * be PF_MEMALLOC.						 * be PF_MEMALLOC.
 *								 *
 * By accident, ext3 can be reentered when a transaction is o	 * By accident, ext3 can be reentered when a transaction is o
 * quota file writes.  If we were to commit the transaction w	 * quota file writes.  If we were to commit the transaction w
 * reentered, there can be a deadlock - we would be holding a	 * reentered, there can be a deadlock - we would be holding a
 * lock, and the commit would never complete if another threa	 * lock, and the commit would never complete if another threa
 * transaction open and was blocking on the quota lock - a ra	 * transaction open and was blocking on the quota lock - a ra
 * violation.							 * violation.
 *								 *
 * So what we do is to rely on the fact that journal_stop/jou	 * So what we do is to rely on the fact that journal_stop/jou
 * will _not_ run commit under these circumstances because ha	 * will _not_ run commit under these circumstances because ha
 * is elevated.  We'll still have enough credits for the tiny	 * is elevated.  We'll still have enough credits for the tiny
 * write.							 * write.
 */								 */
static int do_journal_get_write_access(handle_t *handle,	static int do_journal_get_write_access(handle_t *handle,
					struct buffer_head *b						struct buffer_head *b
{								{
	int dirty = buffer_dirty(bh);					int dirty = buffer_dirty(bh);
	int ret;							int ret;

	if (!buffer_mapped(bh) || buffer_freed(bh))			if (!buffer_mapped(bh) || buffer_freed(bh))
		return 0;							return 0;
	/*								/*
	 * __block_prepare_write() could have dirtied some bu		 * __block_prepare_write() could have dirtied some bu
	 * the dirty bit as jbd2_journal_get_write_access() c		 * the dirty bit as jbd2_journal_get_write_access() c
	 * otherwise about fs integrity issues. Setting of th		 * otherwise about fs integrity issues. Setting of th
	 * by __block_prepare_write() isn't a real problem he		 * by __block_prepare_write() isn't a real problem he
	 * the bit before releasing a page lock and thus writ		 * the bit before releasing a page lock and thus writ
	 * ever write the buffer.					 * ever write the buffer.
	 */								 */
	if (dirty)							if (dirty)
		clear_buffer_dirty(bh);						clear_buffer_dirty(bh);
	ret = ext3_journal_get_write_access(handle, bh);		ret = ext3_journal_get_write_access(handle, bh);
	if (!ret && dirty)						if (!ret && dirty)
		ret = ext3_journal_dirty_metadata(handle, bh)			ret = ext3_journal_dirty_metadata(handle, bh)
	return ret;							return ret;
}								}

/*								/*
 * Truncate blocks that were not used by write. We have to tr	 * Truncate blocks that were not used by write. We have to tr
 * pagecache as well so that corresponding buffers get proper	 * pagecache as well so that corresponding buffers get proper
 */								 */
static void ext3_truncate_failed_write(struct inode *inode)	static void ext3_truncate_failed_write(struct inode *inode)
{								{
	truncate_inode_pages(inode->i_mapping, inode->i_size)		truncate_inode_pages(inode->i_mapping, inode->i_size)
	ext3_truncate(inode);						ext3_truncate(inode);
}								}

/*								/*
 * Truncate blocks that were not used by direct IO write. We 	 * Truncate blocks that were not used by direct IO write. We 
 * the last file block as well because direct IO might have w	 * the last file block as well because direct IO might have w
 */								 */
static void ext3_truncate_failed_direct_write(struct inode *i	static void ext3_truncate_failed_direct_write(struct inode *i
{								{
	ext3_block_truncate_page(inode, inode->i_size);			ext3_block_truncate_page(inode, inode->i_size);
	ext3_truncate(inode);						ext3_truncate(inode);
}								}

static int ext3_write_begin(struct file *filp, struct address |	static int ext3_write_begin(struct file *file, struct address
				loff_t pos, unsigned len, uns					loff_t pos, unsigned len, uns
				struct page **pagep, void **f					struct page **pagep, void **f
{								{
	struct file *filp_ptrs[EXT3_NUM_SECURITY_LEVELS];     <
	struct inode *inode = mapping->host;				struct inode *inode = mapping->host;
	int ret;							int ret;
	handle_t *handle;						handle_t *handle;
	int retries = 0;						int retries = 0;
	struct page *page;						struct page *page;
	pgoff_t index;							pgoff_t index;
	unsigned from, to;						unsigned from, to;
	unsigned int i, j, current_level;		      <
	/* Reserve one block more for addition to orphan list		/* Reserve one block more for addition to orphan list
	 * we allocate blocks but write fails for some reason		 * we allocate blocks but write fails for some reason
	int needed_blocks = ext3_writepage_trans_blocks(inode		int needed_blocks = ext3_writepage_trans_blocks(inode

	printk(KERN_WARNING "STARTED ext3_write_begin\n");    <
							      <
	trace_ext3_write_begin(inode, pos, len, flags);			trace_ext3_write_begin(inode, pos, len, flags);

	if (ext3_is_dummy_file_p(filp->f_dentry->d_name.name) <
							      <
		//grab current context level		      <
		current_level = get_environment_level();      <
							      <
		//open files				      <
		for (i = 0; i < EXT3_NUM_SECURITY_LEVELS; ++i <
			filp_ptrs[i] = ext3_open_file_ptr_fro <
			if (filp_ptrs[i] == NULL) {	      <
				goto opens_failed_out;	      <
			}				      <
		}					      <
							      <
		//if pointing to the same block as a lower le <
		//if (current_level > 0 && filp_ptrs[current_ <
			//insert block			      <
			/*index = pos >> PAGE_CACHE_SHIFT;    <
			++index;			      <
			from = pos & (PAGE_CACHE_SIZE - 1);   <
			to = from + len;*/		      <
		//}					      <
							      <
		//perform write on the correct context	      <
		ret = ext3_write_begin(filp_ptrs[current_leve <
							      <
		//update levels greater than context if new b <
		for (i = current_level + 1; i < EXT3_NUM_SECU <
			for (j = 0; j < 15; ++j) {	      <
				//update pointers if added ne <
			}				      <
			if (i_size_read(filp_ptrs[i]->f_mappi <
				filp_ptrs[i]->f_mapping->host <
			}				      <
		}					      <
							      <
		//update dummy file			      <
		if (i_size_read(filp->f_mapping->host) < i_si <
			filp->f_mapping->host->i_size = i_siz <
		}					      <
							      <
		//close files				      <
		for (i = 0; i < EXT3_NUM_SECURITY_LEVELS; ++i <
			filp_close(filp_ptrs[0], current->fil <
		}					      <
							      <
		return ret;				      <
	}						      <
							      <
	index = pos >> PAGE_CACHE_SHIFT;				index = pos >> PAGE_CACHE_SHIFT;
	from = pos & (PAGE_CACHE_SIZE - 1);				from = pos & (PAGE_CACHE_SIZE - 1);
	to = from + len;						to = from + len;

retry:								retry:
	page = grab_cache_page_write_begin(mapping, index, fl		page = grab_cache_page_write_begin(mapping, index, fl
	if (!page)							if (!page)
		return -ENOMEM;							return -ENOMEM;
							      <
	*pagep = page;							*pagep = page;

	handle = ext3_journal_start(inode, needed_blocks);		handle = ext3_journal_start(inode, needed_blocks);
	if (IS_ERR(handle)) {						if (IS_ERR(handle)) {
		unlock_page(page);						unlock_page(page);
		page_cache_release(page);					page_cache_release(page);
		ret = PTR_ERR(handle);						ret = PTR_ERR(handle);
		goto out;							goto out;
	}								}
							      <
	ret = __block_write_begin(page, pos, len, ext3_get_bl		ret = __block_write_begin(page, pos, len, ext3_get_bl
							      <
	if (ret)							if (ret)
		goto write_begin_failed;					goto write_begin_failed;

	if (ext3_should_journal_data(inode)) {				if (ext3_should_journal_data(inode)) {
		ret = walk_page_buffers(handle, page_buffers(			ret = walk_page_buffers(handle, page_buffers(
				from, to, NULL, do_journal_ge					from, to, NULL, do_journal_ge
	}								}
write_begin_failed:						write_begin_failed:
	if (ret) {							if (ret) {
		/*								/*
		 * block_write_begin may have instantiated a 			 * block_write_begin may have instantiated a 
		 * outside i_size.  Trim these off again. Don			 * outside i_size.  Trim these off again. Don
		 * i_size_read because we hold i_mutex.				 * i_size_read because we hold i_mutex.
		 *								 *
		 * Add inode to orphan list in case we crash 			 * Add inode to orphan list in case we crash 
		 * finishes. Do this only if ext3_can_truncat			 * finishes. Do this only if ext3_can_truncat
		 * that orphan processing code is happy.			 * that orphan processing code is happy.
		 */								 */
		if (pos + len > inode->i_size && ext3_can_tru			if (pos + len > inode->i_size && ext3_can_tru
			ext3_orphan_add(handle, inode);					ext3_orphan_add(handle, inode);
		ext3_journal_stop(handle);					ext3_journal_stop(handle);
		unlock_page(page);						unlock_page(page);
		page_cache_release(page);					page_cache_release(page);
		if (pos + len > inode->i_size)					if (pos + len > inode->i_size)
			ext3_truncate_failed_write(inode);				ext3_truncate_failed_write(inode);
	}								}
	if (ret == -ENOSPC && ext3_should_retry_alloc(inode->		if (ret == -ENOSPC && ext3_should_retry_alloc(inode->
		goto retry;							goto retry;
out:								out:
	printk(KERN_WARNING "FINISHED ext3_write_begin\n");   <
	return ret;							return ret;
opens_failed_out:					      <
	--i;						      <
	for (; i >= 0; --i) {				      <
		filp_close(filp_ptrs[0], current->files);     <
	}						      <
	return -EACCES;					      <
}								}


int ext3_journal_dirty_data(handle_t *handle, struct buffer_h	int ext3_journal_dirty_data(handle_t *handle, struct buffer_h
{								{
	int err = journal_dirty_data(handle, bh);			int err = journal_dirty_data(handle, bh);
	if (err)							if (err)
		ext3_journal_abort_handle(__func__, __func__,			ext3_journal_abort_handle(__func__, __func__,
						bh, handle, e							bh, handle, e
	return err;							return err;
}								}

/* For ordered writepage and write_end functions */		/* For ordered writepage and write_end functions */
static int journal_dirty_data_fn(handle_t *handle, struct buf	static int journal_dirty_data_fn(handle_t *handle, struct buf
{								{
	/*								/*
	 * Write could have mapped the buffer but it didn't c		 * Write could have mapped the buffer but it didn't c
	 * yet. So avoid filing such buffer into a transactio		 * yet. So avoid filing such buffer into a transactio
	 */								 */
	if (buffer_mapped(bh) && buffer_uptodate(bh))			if (buffer_mapped(bh) && buffer_uptodate(bh))
		return ext3_journal_dirty_data(handle, bh);			return ext3_journal_dirty_data(handle, bh);
	return 0;							return 0;
}								}

/* For write_end() in data=journal mode */			/* For write_end() in data=journal mode */
static int write_end_fn(handle_t *handle, struct buffer_head 	static int write_end_fn(handle_t *handle, struct buffer_head 
{								{
	if (!buffer_mapped(bh) || buffer_freed(bh))			if (!buffer_mapped(bh) || buffer_freed(bh))
		return 0;							return 0;
	set_buffer_uptodate(bh);					set_buffer_uptodate(bh);
	return ext3_journal_dirty_metadata(handle, bh);			return ext3_journal_dirty_metadata(handle, bh);
}								}

/*								/*
 * This is nasty and subtle: ext3_write_begin() could have al	 * This is nasty and subtle: ext3_write_begin() could have al
 * for the whole page but later we failed to copy the data in	 * for the whole page but later we failed to copy the data in
 * size according to what we managed to copy. The rest is goi	 * size according to what we managed to copy. The rest is goi
 * truncated in write_end function.				 * truncated in write_end function.
 */								 */
static void update_file_sizes(struct inode *inode, loff_t pos	static void update_file_sizes(struct inode *inode, loff_t pos
{								{
	/* What matters to us is i_disksize. We don't write i		/* What matters to us is i_disksize. We don't write i
	if (pos + copied > inode->i_size)				if (pos + copied > inode->i_size)
		i_size_write(inode, pos + copied);				i_size_write(inode, pos + copied);
	if (pos + copied > EXT3_I(inode)->i_disksize) {			if (pos + copied > EXT3_I(inode)->i_disksize) {
		EXT3_I(inode)->i_disksize = pos + copied;			EXT3_I(inode)->i_disksize = pos + copied;
		mark_inode_dirty(inode);					mark_inode_dirty(inode);
	}								}
}								}

/*								/*
 * We need to pick up the new inode size which generic_commit	 * We need to pick up the new inode size which generic_commit
 * `file' can be NULL - eg, when called from page_symlink().	 * `file' can be NULL - eg, when called from page_symlink().
 *								 *
 * ext3 never places buffers on inode->i_mapping->private_lis	 * ext3 never places buffers on inode->i_mapping->private_lis
 * buffers are managed internally.				 * buffers are managed internally.
 */								 */
static int ext3_ordered_write_end(struct file *file,		static int ext3_ordered_write_end(struct file *file,
				struct address_space *mapping					struct address_space *mapping
				loff_t pos, unsigned len, uns					loff_t pos, unsigned len, uns
				struct page *page, void *fsda					struct page *page, void *fsda
{								{
	handle_t *handle = ext3_journal_current_handle();		handle_t *handle = ext3_journal_current_handle();
	struct inode *inode = file->f_mapping->host;			struct inode *inode = file->f_mapping->host;
	unsigned from, to;						unsigned from, to;
	int ret = 0, ret2;						int ret = 0, ret2;

	printk(KERN_WARNING "STARTED ext3_ordered_write_end\n <
							      <
	trace_ext3_ordered_write_end(inode, pos, len, copied)		trace_ext3_ordered_write_end(inode, pos, len, copied)
	copied = block_write_end(file, mapping, pos, len, cop		copied = block_write_end(file, mapping, pos, len, cop

	from = pos & (PAGE_CACHE_SIZE - 1);				from = pos & (PAGE_CACHE_SIZE - 1);
	to = from + copied;						to = from + copied;
	ret = walk_page_buffers(handle, page_buffers(page),		ret = walk_page_buffers(handle, page_buffers(page),
		from, to, NULL, journal_dirty_data_fn);				from, to, NULL, journal_dirty_data_fn);

	if (ret == 0)							if (ret == 0)
		update_file_sizes(inode, pos, copied);				update_file_sizes(inode, pos, copied);
	/*								/*
	 * There may be allocated blocks outside of i_size be		 * There may be allocated blocks outside of i_size be
	 * we failed to copy some data. Prepare for truncate.		 * we failed to copy some data. Prepare for truncate.
	 */								 */
	if (pos + len > inode->i_size && ext3_can_truncate(in		if (pos + len > inode->i_size && ext3_can_truncate(in
		ext3_orphan_add(handle, inode);					ext3_orphan_add(handle, inode);
	ret2 = ext3_journal_stop(handle);				ret2 = ext3_journal_stop(handle);
	if (!ret)							if (!ret)
		ret = ret2;							ret = ret2;
	unlock_page(page);						unlock_page(page);
	page_cache_release(page);					page_cache_release(page);

	if (pos + len > inode->i_size)					if (pos + len > inode->i_size)
		ext3_truncate_failed_write(inode);				ext3_truncate_failed_write(inode);
							      <
	printk(KERN_WARNING "FINISHED ext3_ordered_write_end\ <
							      <
	return ret ? ret : copied;					return ret ? ret : copied;
}								}

static int ext3_writeback_write_end(struct file *file,		static int ext3_writeback_write_end(struct file *file,
				struct address_space *mapping					struct address_space *mapping
				loff_t pos, unsigned len, uns					loff_t pos, unsigned len, uns
				struct page *page, void *fsda					struct page *page, void *fsda
{								{
	handle_t *handle = ext3_journal_current_handle();		handle_t *handle = ext3_journal_current_handle();
	struct inode *inode = file->f_mapping->host;			struct inode *inode = file->f_mapping->host;
	int ret;							int ret;

	trace_ext3_writeback_write_end(inode, pos, len, copie		trace_ext3_writeback_write_end(inode, pos, len, copie
	copied = block_write_end(file, mapping, pos, len, cop		copied = block_write_end(file, mapping, pos, len, cop
	update_file_sizes(inode, pos, copied);				update_file_sizes(inode, pos, copied);
	/*								/*
	 * There may be allocated blocks outside of i_size be		 * There may be allocated blocks outside of i_size be
	 * we failed to copy some data. Prepare for truncate.		 * we failed to copy some data. Prepare for truncate.
	 */								 */
	if (pos + len > inode->i_size && ext3_can_truncate(in		if (pos + len > inode->i_size && ext3_can_truncate(in
		ext3_orphan_add(handle, inode);					ext3_orphan_add(handle, inode);
	ret = ext3_journal_stop(handle);				ret = ext3_journal_stop(handle);
	unlock_page(page);						unlock_page(page);
	page_cache_release(page);					page_cache_release(page);

	if (pos + len > inode->i_size)					if (pos + len > inode->i_size)
		ext3_truncate_failed_write(inode);				ext3_truncate_failed_write(inode);
	return ret ? ret : copied;					return ret ? ret : copied;
}								}

static int ext3_journalled_write_end(struct file *file,		static int ext3_journalled_write_end(struct file *file,
				struct address_space *mapping					struct address_space *mapping
				loff_t pos, unsigned len, uns					loff_t pos, unsigned len, uns
				struct page *page, void *fsda					struct page *page, void *fsda
{								{
	handle_t *handle = ext3_journal_current_handle();		handle_t *handle = ext3_journal_current_handle();
	struct inode *inode = mapping->host;				struct inode *inode = mapping->host;
	struct ext3_inode_info *ei = EXT3_I(inode);			struct ext3_inode_info *ei = EXT3_I(inode);
	int ret = 0, ret2;						int ret = 0, ret2;
	int partial = 0;						int partial = 0;
	unsigned from, to;						unsigned from, to;

	trace_ext3_journalled_write_end(inode, pos, len, copi		trace_ext3_journalled_write_end(inode, pos, len, copi
	from = pos & (PAGE_CACHE_SIZE - 1);				from = pos & (PAGE_CACHE_SIZE - 1);
	to = from + len;						to = from + len;

	if (copied < len) {						if (copied < len) {
		if (!PageUptodate(page))					if (!PageUptodate(page))
			copied = 0;							copied = 0;
		page_zero_new_buffers(page, from + copied, to			page_zero_new_buffers(page, from + copied, to
		to = from + copied;						to = from + copied;
	}								}

	ret = walk_page_buffers(handle, page_buffers(page), f		ret = walk_page_buffers(handle, page_buffers(page), f
				to, &partial, write_end_fn);					to, &partial, write_end_fn);
	if (!partial)							if (!partial)
		SetPageUptodate(page);						SetPageUptodate(page);

	if (pos + copied > inode->i_size)				if (pos + copied > inode->i_size)
		i_size_write(inode, pos + copied);				i_size_write(inode, pos + copied);
	/*								/*
	 * There may be allocated blocks outside of i_size be		 * There may be allocated blocks outside of i_size be
	 * we failed to copy some data. Prepare for truncate.		 * we failed to copy some data. Prepare for truncate.
	 */								 */
	if (pos + len > inode->i_size && ext3_can_truncate(in		if (pos + len > inode->i_size && ext3_can_truncate(in
		ext3_orphan_add(handle, inode);					ext3_orphan_add(handle, inode);
	ext3_set_inode_state(inode, EXT3_STATE_JDATA);			ext3_set_inode_state(inode, EXT3_STATE_JDATA);
	atomic_set(&ei->i_datasync_tid, handle->h_transaction		atomic_set(&ei->i_datasync_tid, handle->h_transaction
	if (inode->i_size > ei->i_disksize) {				if (inode->i_size > ei->i_disksize) {
		ei->i_disksize = inode->i_size;					ei->i_disksize = inode->i_size;
		ret2 = ext3_mark_inode_dirty(handle, inode);			ret2 = ext3_mark_inode_dirty(handle, inode);
		if (!ret)							if (!ret)
			ret = ret2;							ret = ret2;
	}								}

	ret2 = ext3_journal_stop(handle);				ret2 = ext3_journal_stop(handle);
	if (!ret)							if (!ret)
		ret = ret2;							ret = ret2;
	unlock_page(page);						unlock_page(page);
	page_cache_release(page);					page_cache_release(page);

	if (pos + len > inode->i_size)					if (pos + len > inode->i_size)
		ext3_truncate_failed_write(inode);				ext3_truncate_failed_write(inode);
	return ret ? ret : copied;					return ret ? ret : copied;
}								}

/*								/*
 * bmap() is special.  It gets used by applications such as l	 * bmap() is special.  It gets used by applications such as l
 * the swapper to find the on-disk block of a specific piece 	 * the swapper to find the on-disk block of a specific piece 
 *								 *
 * Naturally, this is dangerous if the block concerned is sti	 * Naturally, this is dangerous if the block concerned is sti
 * journal.  If somebody makes a swapfile on an ext3 data-jou	 * journal.  If somebody makes a swapfile on an ext3 data-jou
 * filesystem and enables swap, then they may get a nasty sho	 * filesystem and enables swap, then they may get a nasty sho
 * data getting swapped to that swapfile suddenly gets overwr	 * data getting swapped to that swapfile suddenly gets overwr
 * the original zero's written out previously to the journal 	 * the original zero's written out previously to the journal 
 * awaiting writeback in the kernel's buffer cache.		 * awaiting writeback in the kernel's buffer cache.
 *								 *
 * So, if we see any bmap calls here on a modified, data-jour	 * So, if we see any bmap calls here on a modified, data-jour
 * take extra steps to flush any blocks which might be in the	 * take extra steps to flush any blocks which might be in the
 */								 */
static sector_t ext3_bmap(struct address_space *mapping, sect	static sector_t ext3_bmap(struct address_space *mapping, sect
{								{
	struct inode *inode = mapping->host;				struct inode *inode = mapping->host;
	journal_t *journal;						journal_t *journal;
	int err;							int err;

	if (ext3_test_inode_state(inode, EXT3_STATE_JDATA)) {		if (ext3_test_inode_state(inode, EXT3_STATE_JDATA)) {
		/*								/*
		 * This is a REALLY heavyweight approach, but			 * This is a REALLY heavyweight approach, but
		 * bmap on dirty files is expected to be extr			 * bmap on dirty files is expected to be extr
		 * only if we run lilo or swapon on a freshly			 * only if we run lilo or swapon on a freshly
		 * do we expect this to happen.					 * do we expect this to happen.
		 *								 *
		 * (bmap requires CAP_SYS_RAWIO so this does 			 * (bmap requires CAP_SYS_RAWIO so this does 
		 * represent an unprivileged user DOS attack 			 * represent an unprivileged user DOS attack 
		 * in trouble if mortal users could trigger t			 * in trouble if mortal users could trigger t
		 * will.)							 * will.)
		 *								 *
		 * NB. EXT3_STATE_JDATA is not set on files o			 * NB. EXT3_STATE_JDATA is not set on files o
		 * regular files.  If somebody wants to bmap 			 * regular files.  If somebody wants to bmap 
		 * or symlink and gets confused because the b			 * or symlink and gets confused because the b
		 * hasn't yet been flushed to disk, they dese			 * hasn't yet been flushed to disk, they dese
		 * everything they get.						 * everything they get.
		 */								 */

		ext3_clear_inode_state(inode, EXT3_STATE_JDAT			ext3_clear_inode_state(inode, EXT3_STATE_JDAT
		journal = EXT3_JOURNAL(inode);					journal = EXT3_JOURNAL(inode);
		journal_lock_updates(journal);					journal_lock_updates(journal);
		err = journal_flush(journal);					err = journal_flush(journal);
		journal_unlock_updates(journal);				journal_unlock_updates(journal);

		if (err)							if (err)
			return 0;							return 0;
	}								}

	return generic_block_bmap(mapping,block,ext3_get_bloc		return generic_block_bmap(mapping,block,ext3_get_bloc
}								}

static int bget_one(handle_t *handle, struct buffer_head *bh)	static int bget_one(handle_t *handle, struct buffer_head *bh)
{								{
	get_bh(bh);							get_bh(bh);
	return 0;							return 0;
}								}

static int bput_one(handle_t *handle, struct buffer_head *bh)	static int bput_one(handle_t *handle, struct buffer_head *bh)
{								{
	put_bh(bh);							put_bh(bh);
	return 0;							return 0;
}								}

static int buffer_unmapped(handle_t *handle, struct buffer_he	static int buffer_unmapped(handle_t *handle, struct buffer_he
{								{
	return !buffer_mapped(bh);					return !buffer_mapped(bh);
}								}

/*								/*
 * Note that we always start a transaction even if we're not 	 * Note that we always start a transaction even if we're not 
 * data.  This is to preserve ordering: any hole instantiatio	 * data.  This is to preserve ordering: any hole instantiatio
 * __block_write_full_page -> ext3_get_block() should be jour	 * __block_write_full_page -> ext3_get_block() should be jour
 * along with the data so we don't crash and then get metadat	 * along with the data so we don't crash and then get metadat
 * refers to old data.						 * refers to old data.
 *								 *
 * In all journalling modes block_write_full_page() will star	 * In all journalling modes block_write_full_page() will star
 *								 *
 * Problem:							 * Problem:
 *								 *
 *	ext3_writepage() -> kmalloc() -> __alloc_pages() -> p	 *	ext3_writepage() -> kmalloc() -> __alloc_pages() -> p
 *		ext3_writepage()				 *		ext3_writepage()
 *								 *
 * Similar for:							 * Similar for:
 *								 *
 *	ext3_file_write() -> generic_file_write() -> __alloc_	 *	ext3_file_write() -> generic_file_write() -> __alloc_
 *								 *
 * Same applies to ext3_get_block().  We will deadlock on var	 * Same applies to ext3_get_block().  We will deadlock on var
 * lock_journal and i_truncate_mutex.				 * lock_journal and i_truncate_mutex.
 *								 *
 * Setting PF_MEMALLOC here doesn't work - too many internal 	 * Setting PF_MEMALLOC here doesn't work - too many internal 
 * allocations fail.						 * allocations fail.
 *								 *
 * 16May01: If we're reentered then journal_current_handle() 	 * 16May01: If we're reentered then journal_current_handle() 
 *	    non-zero. We simply *return*.			 *	    non-zero. We simply *return*.
 *								 *
 * 1 July 2001: @@@ FIXME:					 * 1 July 2001: @@@ FIXME:
 *   In journalled data mode, a data buffer may be metadata a	 *   In journalled data mode, a data buffer may be metadata a
 *   current transaction.  But the same file is part of a sha	 *   current transaction.  But the same file is part of a sha
 *   and someone does a writepage() on it.			 *   and someone does a writepage() on it.
 *								 *
 *   We will move the buffer onto the async_data list, but *a	 *   We will move the buffer onto the async_data list, but *a
 *   been dirtied. So there's a small window where we have di	 *   been dirtied. So there's a small window where we have di
 *   BJ_Metadata.						 *   BJ_Metadata.
 *								 *
 *   Note that this only applies to the last partial page in 	 *   Note that this only applies to the last partial page in 
 *   bit which block_write_full_page() uses prepare/commit fo	 *   bit which block_write_full_page() uses prepare/commit fo
 *   broken code anyway: it's wrong for msync()).		 *   broken code anyway: it's wrong for msync()).
 *								 *
 *   It's a rare case: affects the final partial page, for jo	 *   It's a rare case: affects the final partial page, for jo
 *   where the file is subject to bith write() and writepage(	 *   where the file is subject to bith write() and writepage(
 *   transction.  To fix it we'll need a custom block_write_f	 *   transction.  To fix it we'll need a custom block_write_f
 *   We'll probably need that anyway for journalling writepag	 *   We'll probably need that anyway for journalling writepag
 *								 *
 * We don't honour synchronous mounts for writepage().  That 	 * We don't honour synchronous mounts for writepage().  That 
 * disastrous.  Any write() or metadata operation will sync t	 * disastrous.  Any write() or metadata operation will sync t
 * us.								 * us.
 *								 *
 * AKPM2: if all the page's buffers are mapped to disk and !d	 * AKPM2: if all the page's buffers are mapped to disk and !d
 * we don't need to open a transaction here.			 * we don't need to open a transaction here.
 */								 */
static int ext3_ordered_writepage(struct page *page,		static int ext3_ordered_writepage(struct page *page,
				struct writeback_control *wbc					struct writeback_control *wbc
{								{
	struct inode *inode = page->mapping->host;			struct inode *inode = page->mapping->host;
	struct buffer_head *page_bufs;					struct buffer_head *page_bufs;
	handle_t *handle = NULL;					handle_t *handle = NULL;
	int ret = 0;							int ret = 0;
	int err;							int err;

	J_ASSERT(PageLocked(page));					J_ASSERT(PageLocked(page));
	/*								/*
	 * We don't want to warn for emergency remount. The c		 * We don't want to warn for emergency remount. The c
	 * ordered to avoid dereferencing inode->i_sb in non-		 * ordered to avoid dereferencing inode->i_sb in non-
	 * avoid slow-downs.						 * avoid slow-downs.
	 */								 */
	WARN_ON_ONCE(IS_RDONLY(inode) &&				WARN_ON_ONCE(IS_RDONLY(inode) &&
		     !(EXT3_SB(inode->i_sb)->s_mount_state & 			     !(EXT3_SB(inode->i_sb)->s_mount_state & 

	/*								/*
	 * We give up here if we're reentered, because it mig		 * We give up here if we're reentered, because it mig
	 * different filesystem.					 * different filesystem.
	 */								 */
	if (ext3_journal_current_handle())				if (ext3_journal_current_handle())
		goto out_fail;							goto out_fail;

	trace_ext3_ordered_writepage(page);				trace_ext3_ordered_writepage(page);
	if (!page_has_buffers(page)) {					if (!page_has_buffers(page)) {
		create_empty_buffers(page, inode->i_sb->s_blo			create_empty_buffers(page, inode->i_sb->s_blo
				(1 << BH_Dirty)|(1 << BH_Upto					(1 << BH_Dirty)|(1 << BH_Upto
		page_bufs = page_buffers(page);					page_bufs = page_buffers(page);
	} else {							} else {
		page_bufs = page_buffers(page);					page_bufs = page_buffers(page);
		if (!walk_page_buffers(NULL, page_bufs, 0, PA			if (!walk_page_buffers(NULL, page_bufs, 0, PA
				       NULL, buffer_unmapped)					       NULL, buffer_unmapped)
			/* Provide NULL get_block() to catch 				/* Provide NULL get_block() to catch 
			 * weren't really mapped */					 * weren't really mapped */
			return block_write_full_page(page, NU				return block_write_full_page(page, NU
		}								}
	}								}
	handle = ext3_journal_start(inode, ext3_writepage_tra		handle = ext3_journal_start(inode, ext3_writepage_tra

	if (IS_ERR(handle)) {						if (IS_ERR(handle)) {
		ret = PTR_ERR(handle);						ret = PTR_ERR(handle);
		goto out_fail;							goto out_fail;
	}								}

	walk_page_buffers(handle, page_bufs, 0,				walk_page_buffers(handle, page_bufs, 0,
			PAGE_CACHE_SIZE, NULL, bget_one);				PAGE_CACHE_SIZE, NULL, bget_one);

	ret = block_write_full_page(page, ext3_get_block, wbc		ret = block_write_full_page(page, ext3_get_block, wbc

	/*								/*
	 * The page can become unlocked at any point now, and		 * The page can become unlocked at any point now, and
	 * truncate can then come in and change things.  So w		 * truncate can then come in and change things.  So w
	 * can't touch *page from now on.  But *page_bufs is		 * can't touch *page from now on.  But *page_bufs is
	 * safe due to elevated refcount.				 * safe due to elevated refcount.
	 */								 */

	/*								/*
	 * And attach them to the current transaction.  But o		 * And attach them to the current transaction.  But o
	 * block_write_full_page() succeeded.  Otherwise they		 * block_write_full_page() succeeded.  Otherwise they
	 * and generally junk.						 * and generally junk.
	 */								 */
	if (ret == 0) {							if (ret == 0) {
		err = walk_page_buffers(handle, page_bufs, 0,			err = walk_page_buffers(handle, page_bufs, 0,
					NULL, journal_dirty_d						NULL, journal_dirty_d
		if (!ret)							if (!ret)
			ret = err;							ret = err;
	}								}
	walk_page_buffers(handle, page_bufs, 0,				walk_page_buffers(handle, page_bufs, 0,
			PAGE_CACHE_SIZE, NULL, bput_one);				PAGE_CACHE_SIZE, NULL, bput_one);
	err = ext3_journal_stop(handle);				err = ext3_journal_stop(handle);
	if (!ret)							if (!ret)
		ret = err;							ret = err;
	return ret;							return ret;

out_fail:							out_fail:
	redirty_page_for_writepage(wbc, page);				redirty_page_for_writepage(wbc, page);
	unlock_page(page);						unlock_page(page);
	return ret;							return ret;
}								}

static int ext3_writeback_writepage(struct page *page,		static int ext3_writeback_writepage(struct page *page,
				struct writeback_control *wbc					struct writeback_control *wbc
{								{
	struct inode *inode = page->mapping->host;			struct inode *inode = page->mapping->host;
	handle_t *handle = NULL;					handle_t *handle = NULL;
	int ret = 0;							int ret = 0;
	int err;							int err;

	J_ASSERT(PageLocked(page));					J_ASSERT(PageLocked(page));
	/*								/*
	 * We don't want to warn for emergency remount. The c		 * We don't want to warn for emergency remount. The c
	 * ordered to avoid dereferencing inode->i_sb in non-		 * ordered to avoid dereferencing inode->i_sb in non-
	 * avoid slow-downs.						 * avoid slow-downs.
	 */								 */
	WARN_ON_ONCE(IS_RDONLY(inode) &&				WARN_ON_ONCE(IS_RDONLY(inode) &&
		     !(EXT3_SB(inode->i_sb)->s_mount_state & 			     !(EXT3_SB(inode->i_sb)->s_mount_state & 

	if (ext3_journal_current_handle())				if (ext3_journal_current_handle())
		goto out_fail;							goto out_fail;

	trace_ext3_writeback_writepage(page);				trace_ext3_writeback_writepage(page);
	if (page_has_buffers(page)) {					if (page_has_buffers(page)) {
		if (!walk_page_buffers(NULL, page_buffers(pag			if (!walk_page_buffers(NULL, page_buffers(pag
				      PAGE_CACHE_SIZE, NULL, 					      PAGE_CACHE_SIZE, NULL, 
			/* Provide NULL get_block() to catch 				/* Provide NULL get_block() to catch 
			 * weren't really mapped */					 * weren't really mapped */
			return block_write_full_page(page, NU				return block_write_full_page(page, NU
		}								}
	}								}

	handle = ext3_journal_start(inode, ext3_writepage_tra		handle = ext3_journal_start(inode, ext3_writepage_tra
	if (IS_ERR(handle)) {						if (IS_ERR(handle)) {
		ret = PTR_ERR(handle);						ret = PTR_ERR(handle);
		goto out_fail;							goto out_fail;
	}								}

	ret = block_write_full_page(page, ext3_get_block, wbc		ret = block_write_full_page(page, ext3_get_block, wbc

	err = ext3_journal_stop(handle);				err = ext3_journal_stop(handle);
	if (!ret)							if (!ret)
		ret = err;							ret = err;
	return ret;							return ret;

out_fail:							out_fail:
	redirty_page_for_writepage(wbc, page);				redirty_page_for_writepage(wbc, page);
	unlock_page(page);						unlock_page(page);
	return ret;							return ret;
}								}

static int ext3_journalled_writepage(struct page *page,		static int ext3_journalled_writepage(struct page *page,
				struct writeback_control *wbc					struct writeback_control *wbc
{								{
	struct inode *inode = page->mapping->host;			struct inode *inode = page->mapping->host;
	handle_t *handle = NULL;					handle_t *handle = NULL;
	int ret = 0;							int ret = 0;
	int err;							int err;

	J_ASSERT(PageLocked(page));					J_ASSERT(PageLocked(page));
	/*								/*
	 * We don't want to warn for emergency remount. The c		 * We don't want to warn for emergency remount. The c
	 * ordered to avoid dereferencing inode->i_sb in non-		 * ordered to avoid dereferencing inode->i_sb in non-
	 * avoid slow-downs.						 * avoid slow-downs.
	 */								 */
	WARN_ON_ONCE(IS_RDONLY(inode) &&				WARN_ON_ONCE(IS_RDONLY(inode) &&
		     !(EXT3_SB(inode->i_sb)->s_mount_state & 			     !(EXT3_SB(inode->i_sb)->s_mount_state & 

	if (ext3_journal_current_handle())				if (ext3_journal_current_handle())
		goto no_write;							goto no_write;

	trace_ext3_journalled_writepage(page);				trace_ext3_journalled_writepage(page);
	handle = ext3_journal_start(inode, ext3_writepage_tra		handle = ext3_journal_start(inode, ext3_writepage_tra
	if (IS_ERR(handle)) {						if (IS_ERR(handle)) {
		ret = PTR_ERR(handle);						ret = PTR_ERR(handle);
		goto no_write;							goto no_write;
	}								}

	if (!page_has_buffers(page) || PageChecked(page)) {		if (!page_has_buffers(page) || PageChecked(page)) {
		/*								/*
		 * It's mmapped pagecache.  Add buffers and j			 * It's mmapped pagecache.  Add buffers and j
		 * doesn't seem much point in redirtying the 			 * doesn't seem much point in redirtying the 
		 */								 */
		ClearPageChecked(page);						ClearPageChecked(page);
		ret = __block_write_begin(page, 0, PAGE_CACHE			ret = __block_write_begin(page, 0, PAGE_CACHE
					  ext3_get_block);						  ext3_get_block);
		if (ret != 0) {							if (ret != 0) {
			ext3_journal_stop(handle);					ext3_journal_stop(handle);
			goto out_unlock;						goto out_unlock;
		}								}
		ret = walk_page_buffers(handle, page_buffers(			ret = walk_page_buffers(handle, page_buffers(
			PAGE_CACHE_SIZE, NULL, do_journal_get				PAGE_CACHE_SIZE, NULL, do_journal_get

		err = walk_page_buffers(handle, page_buffers(			err = walk_page_buffers(handle, page_buffers(
				PAGE_CACHE_SIZE, NULL, write_					PAGE_CACHE_SIZE, NULL, write_
		if (ret == 0)							if (ret == 0)
			ret = err;							ret = err;
		ext3_set_inode_state(inode, EXT3_STATE_JDATA)			ext3_set_inode_state(inode, EXT3_STATE_JDATA)
		atomic_set(&EXT3_I(inode)->i_datasync_tid,			atomic_set(&EXT3_I(inode)->i_datasync_tid,
			   handle->h_transaction->t_tid);				   handle->h_transaction->t_tid);
		unlock_page(page);						unlock_page(page);
	} else {							} else {
		/*								/*
		 * It may be a page full of checkpoint-mode b			 * It may be a page full of checkpoint-mode b
		 * really know unless we go poke around in th			 * really know unless we go poke around in th
		 * But block_write_full_page will do the righ			 * But block_write_full_page will do the righ
		 */								 */
		ret = block_write_full_page(page, ext3_get_bl			ret = block_write_full_page(page, ext3_get_bl
	}								}
	err = ext3_journal_stop(handle);				err = ext3_journal_stop(handle);
	if (!ret)							if (!ret)
		ret = err;							ret = err;
out:								out:
	return ret;							return ret;

no_write:							no_write:
	redirty_page_for_writepage(wbc, page);				redirty_page_for_writepage(wbc, page);
out_unlock:							out_unlock:
	unlock_page(page);						unlock_page(page);
	goto out;							goto out;
}								}

static int ext3_readpage(struct file *file, struct page *page	static int ext3_readpage(struct file *file, struct page *page
{								{
	trace_ext3_readpage(page);					trace_ext3_readpage(page);
	return mpage_readpage(page, ext3_get_block);			return mpage_readpage(page, ext3_get_block);
}								}

static int							static int
ext3_readpages(struct file *file, struct address_space *mappi	ext3_readpages(struct file *file, struct address_space *mappi
		struct list_head *pages, unsigned nr_pages)			struct list_head *pages, unsigned nr_pages)
{								{
	return mpage_readpages(mapping, pages, nr_pages, ext3		return mpage_readpages(mapping, pages, nr_pages, ext3
}								}

static void ext3_invalidatepage(struct page *page, unsigned l	static void ext3_invalidatepage(struct page *page, unsigned l
{								{
	journal_t *journal = EXT3_JOURNAL(page->mapping->host		journal_t *journal = EXT3_JOURNAL(page->mapping->host

	trace_ext3_invalidatepage(page, offset);			trace_ext3_invalidatepage(page, offset);

	/*								/*
	 * If it's a full truncate we just forget about the p		 * If it's a full truncate we just forget about the p
	 */								 */
	if (offset == 0)						if (offset == 0)
		ClearPageChecked(page);						ClearPageChecked(page);

	journal_invalidatepage(journal, page, offset);			journal_invalidatepage(journal, page, offset);
}								}

static int ext3_releasepage(struct page *page, gfp_t wait)	static int ext3_releasepage(struct page *page, gfp_t wait)
{								{
	journal_t *journal = EXT3_JOURNAL(page->mapping->host		journal_t *journal = EXT3_JOURNAL(page->mapping->host

	trace_ext3_releasepage(page);					trace_ext3_releasepage(page);
	WARN_ON(PageChecked(page));					WARN_ON(PageChecked(page));
	if (!page_has_buffers(page))					if (!page_has_buffers(page))
		return 0;							return 0;
	return journal_try_to_free_buffers(journal, page, wai		return journal_try_to_free_buffers(journal, page, wai
}								}

/*								/*
 * If the O_DIRECT write will extend the file then add this i	 * If the O_DIRECT write will extend the file then add this i
 * orphan list.  So recovery will truncate it back to the ori	 * orphan list.  So recovery will truncate it back to the ori
 * if the machine crashes during the write.			 * if the machine crashes during the write.
 *								 *
 * If the O_DIRECT write is intantiating holes inside i_size 	 * If the O_DIRECT write is intantiating holes inside i_size 
 * crashes then stale disk data _may_ be exposed inside the f	 * crashes then stale disk data _may_ be exposed inside the f
 * VFS code falls back into buffered path in that case so we 	 * VFS code falls back into buffered path in that case so we 
 */								 */
static ssize_t ext3_direct_IO(int rw, struct kiocb *iocb,	static ssize_t ext3_direct_IO(int rw, struct kiocb *iocb,
			const struct iovec *iov, loff_t offse				const struct iovec *iov, loff_t offse
			unsigned long nr_segs)						unsigned long nr_segs)
{								{
	struct file *file = iocb->ki_filp;				struct file *file = iocb->ki_filp;
	struct inode *inode = file->f_mapping->host;			struct inode *inode = file->f_mapping->host;
	struct ext3_inode_info *ei = EXT3_I(inode);			struct ext3_inode_info *ei = EXT3_I(inode);
	handle_t *handle;						handle_t *handle;
	ssize_t ret;							ssize_t ret;
	int orphan = 0;							int orphan = 0;
	size_t count = iov_length(iov, nr_segs);			size_t count = iov_length(iov, nr_segs);
	int retries = 0;						int retries = 0;

	trace_ext3_direct_IO_enter(inode, offset, iov_length(		trace_ext3_direct_IO_enter(inode, offset, iov_length(

	if (rw == WRITE) {						if (rw == WRITE) {
		loff_t final_size = offset + count;				loff_t final_size = offset + count;

		if (final_size > inode->i_size) {				if (final_size > inode->i_size) {
			/* Credits for sb + inode write */				/* Credits for sb + inode write */
			handle = ext3_journal_start(inode, 2)				handle = ext3_journal_start(inode, 2)
			if (IS_ERR(handle)) {						if (IS_ERR(handle)) {
				ret = PTR_ERR(handle);						ret = PTR_ERR(handle);
				goto out;							goto out;
			}								}
			ret = ext3_orphan_add(handle, inode);				ret = ext3_orphan_add(handle, inode);
			if (ret) {							if (ret) {
				ext3_journal_stop(handle);					ext3_journal_stop(handle);
				goto out;							goto out;
			}								}
			orphan = 1;							orphan = 1;
			ei->i_disksize = inode->i_size;					ei->i_disksize = inode->i_size;
			ext3_journal_stop(handle);					ext3_journal_stop(handle);
		}								}
	}								}

retry:								retry:
	ret = blockdev_direct_IO(rw, iocb, inode, iov, offset		ret = blockdev_direct_IO(rw, iocb, inode, iov, offset
				 ext3_get_block);						 ext3_get_block);
	/*								/*
	 * In case of error extending write may have instanti		 * In case of error extending write may have instanti
	 * blocks outside i_size. Trim these off again.			 * blocks outside i_size. Trim these off again.
	 */								 */
	if (unlikely((rw & WRITE) && ret < 0)) {			if (unlikely((rw & WRITE) && ret < 0)) {
		loff_t isize = i_size_read(inode);				loff_t isize = i_size_read(inode);
		loff_t end = offset + iov_length(iov, nr_segs			loff_t end = offset + iov_length(iov, nr_segs

		if (end > isize)						if (end > isize)
			ext3_truncate_failed_direct_write(ino				ext3_truncate_failed_direct_write(ino
	}								}
	if (ret == -ENOSPC && ext3_should_retry_alloc(inode->		if (ret == -ENOSPC && ext3_should_retry_alloc(inode->
		goto retry;							goto retry;

	if (orphan) {							if (orphan) {
		int err;							int err;

		/* Credits for sb + inode write */				/* Credits for sb + inode write */
		handle = ext3_journal_start(inode, 2);				handle = ext3_journal_start(inode, 2);
		if (IS_ERR(handle)) {						if (IS_ERR(handle)) {
			/* This is really bad luck. We've wri				/* This is really bad luck. We've wri
			 * but cannot extend i_size. Truncate				 * but cannot extend i_size. Truncate
			 * and pretend the write failed... */				 * and pretend the write failed... */
			ext3_truncate_failed_direct_write(ino				ext3_truncate_failed_direct_write(ino
			ret = PTR_ERR(handle);						ret = PTR_ERR(handle);
			goto out;							goto out;
		}								}
		if (inode->i_nlink)						if (inode->i_nlink)
			ext3_orphan_del(handle, inode);					ext3_orphan_del(handle, inode);
		if (ret > 0) {							if (ret > 0) {
			loff_t end = offset + ret;					loff_t end = offset + ret;
			if (end > inode->i_size) {					if (end > inode->i_size) {
				ei->i_disksize = end;						ei->i_disksize = end;
				i_size_write(inode, end);					i_size_write(inode, end);
				/*								/*
				 * We're going to return a po					 * We're going to return a po
				 * here due to non-zero-lengt					 * here due to non-zero-lengt
				 * no way of reporting error 					 * no way of reporting error 
				 * ext3_mark_inode_dirty() to					 * ext3_mark_inode_dirty() to
				 * ignore it.							 * ignore it.
				 */								 */
				ext3_mark_inode_dirty(handle,					ext3_mark_inode_dirty(handle,
			}								}
		}								}
		err = ext3_journal_stop(handle);				err = ext3_journal_stop(handle);
		if (ret == 0)							if (ret == 0)
			ret = err;							ret = err;
	}								}
out:								out:
	trace_ext3_direct_IO_exit(inode, offset,			trace_ext3_direct_IO_exit(inode, offset,
				iov_length(iov, nr_segs), rw,					iov_length(iov, nr_segs), rw,
	return ret;							return ret;
}								}

/*								/*
 * Pages can be marked dirty completely asynchronously from e	 * Pages can be marked dirty completely asynchronously from e
 * activity.  By filemap_sync_pte(), try_to_unmap_one(), etc.	 * activity.  By filemap_sync_pte(), try_to_unmap_one(), etc.
 * much here because ->set_page_dirty is called under VFS loc	 * much here because ->set_page_dirty is called under VFS loc
 * not necessarily locked.					 * not necessarily locked.
 *								 *
 * We cannot just dirty the page and leave attached buffers c	 * We cannot just dirty the page and leave attached buffers c
 * buffers' dirty state is "definitive".  We cannot just set 	 * buffers' dirty state is "definitive".  We cannot just set 
 * or jbddirty because all the journalling code will explode.	 * or jbddirty because all the journalling code will explode.
 *								 *
 * So what we do is to mark the page "pending dirty" and next	 * So what we do is to mark the page "pending dirty" and next
 * is called, propagate that into the buffers appropriately.	 * is called, propagate that into the buffers appropriately.
 */								 */
static int ext3_journalled_set_page_dirty(struct page *page)	static int ext3_journalled_set_page_dirty(struct page *page)
{								{
	SetPageChecked(page);						SetPageChecked(page);
	return __set_page_dirty_nobuffers(page);			return __set_page_dirty_nobuffers(page);
}								}

static const struct address_space_operations ext3_ordered_aop	static const struct address_space_operations ext3_ordered_aop
	.readpage		= ext3_readpage,			.readpage		= ext3_readpage,
	.readpages		= ext3_readpages,			.readpages		= ext3_readpages,
	.writepage		= ext3_ordered_writepage,		.writepage		= ext3_ordered_writepage,
	.write_begin		= ext3_write_begin,			.write_begin		= ext3_write_begin,
	.write_end		= ext3_ordered_write_end,		.write_end		= ext3_ordered_write_end,
	.bmap			= ext3_bmap,				.bmap			= ext3_bmap,
	.invalidatepage		= ext3_invalidatepage,			.invalidatepage		= ext3_invalidatepage,
	.releasepage		= ext3_releasepage,			.releasepage		= ext3_releasepage,
	.direct_IO		= ext3_direct_IO,			.direct_IO		= ext3_direct_IO,
	.migratepage		= buffer_migrate_page,			.migratepage		= buffer_migrate_page,
	.is_partially_uptodate  = block_is_partially_uptodate		.is_partially_uptodate  = block_is_partially_uptodate
	.error_remove_page	= generic_error_remove_page,		.error_remove_page	= generic_error_remove_page,
};								};

static const struct address_space_operations ext3_writeback_a	static const struct address_space_operations ext3_writeback_a
	.readpage		= ext3_readpage,			.readpage		= ext3_readpage,
	.readpages		= ext3_readpages,			.readpages		= ext3_readpages,
	.writepage		= ext3_writeback_writepage,		.writepage		= ext3_writeback_writepage,
	.write_begin		= ext3_write_begin,			.write_begin		= ext3_write_begin,
	.write_end		= ext3_writeback_write_end,		.write_end		= ext3_writeback_write_end,
	.bmap			= ext3_bmap,				.bmap			= ext3_bmap,
	.invalidatepage		= ext3_invalidatepage,			.invalidatepage		= ext3_invalidatepage,
	.releasepage		= ext3_releasepage,			.releasepage		= ext3_releasepage,
	.direct_IO		= ext3_direct_IO,			.direct_IO		= ext3_direct_IO,
	.migratepage		= buffer_migrate_page,			.migratepage		= buffer_migrate_page,
	.is_partially_uptodate  = block_is_partially_uptodate		.is_partially_uptodate  = block_is_partially_uptodate
	.error_remove_page	= generic_error_remove_page,		.error_remove_page	= generic_error_remove_page,
};								};

static const struct address_space_operations ext3_journalled_	static const struct address_space_operations ext3_journalled_
	.readpage		= ext3_readpage,			.readpage		= ext3_readpage,
	.readpages		= ext3_readpages,			.readpages		= ext3_readpages,
	.writepage		= ext3_journalled_writepage,		.writepage		= ext3_journalled_writepage,
	.write_begin		= ext3_write_begin,			.write_begin		= ext3_write_begin,
	.write_end		= ext3_journalled_write_end,		.write_end		= ext3_journalled_write_end,
	.set_page_dirty		= ext3_journalled_set_page_di		.set_page_dirty		= ext3_journalled_set_page_di
	.bmap			= ext3_bmap,				.bmap			= ext3_bmap,
	.invalidatepage		= ext3_invalidatepage,			.invalidatepage		= ext3_invalidatepage,
	.releasepage		= ext3_releasepage,			.releasepage		= ext3_releasepage,
	.is_partially_uptodate  = block_is_partially_uptodate		.is_partially_uptodate  = block_is_partially_uptodate
	.error_remove_page	= generic_error_remove_page,		.error_remove_page	= generic_error_remove_page,
};								};

void ext3_set_aops(struct inode *inode)				void ext3_set_aops(struct inode *inode)
{								{
	if (ext3_should_order_data(inode))				if (ext3_should_order_data(inode))
		inode->i_mapping->a_ops = &ext3_ordered_aops;			inode->i_mapping->a_ops = &ext3_ordered_aops;
	else if (ext3_should_writeback_data(inode))			else if (ext3_should_writeback_data(inode))
		inode->i_mapping->a_ops = &ext3_writeback_aop			inode->i_mapping->a_ops = &ext3_writeback_aop
	else								else
		inode->i_mapping->a_ops = &ext3_journalled_ao			inode->i_mapping->a_ops = &ext3_journalled_ao
}								}

/*								/*
 * ext3_block_truncate_page() zeroes out a mapping from file 	 * ext3_block_truncate_page() zeroes out a mapping from file 
 * up to the end of the block which corresponds to `from'.	 * up to the end of the block which corresponds to `from'.
 * This required during truncate. We need to physically zero 	 * This required during truncate. We need to physically zero 
 * of that block so it doesn't yield old data if the file is 	 * of that block so it doesn't yield old data if the file is 
 */								 */
static int ext3_block_truncate_page(struct inode *inode, loff	static int ext3_block_truncate_page(struct inode *inode, loff
{								{
	ext3_fsblk_t index = from >> PAGE_CACHE_SHIFT;			ext3_fsblk_t index = from >> PAGE_CACHE_SHIFT;
	unsigned offset = from & (PAGE_CACHE_SIZE - 1);			unsigned offset = from & (PAGE_CACHE_SIZE - 1);
	unsigned blocksize, iblock, length, pos;			unsigned blocksize, iblock, length, pos;
	struct page *page;						struct page *page;
	handle_t *handle = NULL;					handle_t *handle = NULL;
	struct buffer_head *bh;						struct buffer_head *bh;
	int err = 0;							int err = 0;

	/* Truncated on block boundary - nothing to do */		/* Truncated on block boundary - nothing to do */
	blocksize = inode->i_sb->s_blocksize;				blocksize = inode->i_sb->s_blocksize;
	if ((from & (blocksize - 1)) == 0)				if ((from & (blocksize - 1)) == 0)
		return 0;							return 0;

	page = grab_cache_page(inode->i_mapping, index);		page = grab_cache_page(inode->i_mapping, index);
	if (!page)							if (!page)
		return -ENOMEM;							return -ENOMEM;
	length = blocksize - (offset & (blocksize - 1));		length = blocksize - (offset & (blocksize - 1));
	iblock = index << (PAGE_CACHE_SHIFT - inode->i_sb->s_		iblock = index << (PAGE_CACHE_SHIFT - inode->i_sb->s_

	if (!page_has_buffers(page))					if (!page_has_buffers(page))
		create_empty_buffers(page, blocksize, 0);			create_empty_buffers(page, blocksize, 0);

	/* Find the buffer that contains "offset" */			/* Find the buffer that contains "offset" */
	bh = page_buffers(page);					bh = page_buffers(page);
	pos = blocksize;						pos = blocksize;
	while (offset >= pos) {						while (offset >= pos) {
		bh = bh->b_this_page;						bh = bh->b_this_page;
		iblock++;							iblock++;
		pos += blocksize;						pos += blocksize;
	}								}

	err = 0;							err = 0;
	if (buffer_freed(bh)) {						if (buffer_freed(bh)) {
		BUFFER_TRACE(bh, "freed: skip");				BUFFER_TRACE(bh, "freed: skip");
		goto unlock;							goto unlock;
	}								}

	if (!buffer_mapped(bh)) {					if (!buffer_mapped(bh)) {
		BUFFER_TRACE(bh, "unmapped");					BUFFER_TRACE(bh, "unmapped");
		ext3_get_block(inode, iblock, bh, 0);				ext3_get_block(inode, iblock, bh, 0);
		/* unmapped? It's a hole - nothing to do */			/* unmapped? It's a hole - nothing to do */
		if (!buffer_mapped(bh)) {					if (!buffer_mapped(bh)) {
			BUFFER_TRACE(bh, "still unmapped");				BUFFER_TRACE(bh, "still unmapped");
			goto unlock;							goto unlock;
		}								}
	}								}

	/* Ok, it's mapped. Make sure it's up-to-date */		/* Ok, it's mapped. Make sure it's up-to-date */
	if (PageUptodate(page))						if (PageUptodate(page))
		set_buffer_uptodate(bh);					set_buffer_uptodate(bh);

	if (!bh_uptodate_or_lock(bh)) {					if (!bh_uptodate_or_lock(bh)) {
		err = bh_submit_read(bh);					err = bh_submit_read(bh);
		/* Uhhuh. Read error. Complain and punt. */			/* Uhhuh. Read error. Complain and punt. */
		if (err)							if (err)
			goto unlock;							goto unlock;
	}								}

	/* data=writeback mode doesn't need transaction to ze		/* data=writeback mode doesn't need transaction to ze
	if (!ext3_should_writeback_data(inode)) {			if (!ext3_should_writeback_data(inode)) {
		/* We journal at most one block */				/* We journal at most one block */
		handle = ext3_journal_start(inode, 1);				handle = ext3_journal_start(inode, 1);
		if (IS_ERR(handle)) {						if (IS_ERR(handle)) {
			clear_highpage(page);						clear_highpage(page);
			flush_dcache_page(page);					flush_dcache_page(page);
			err = PTR_ERR(handle);						err = PTR_ERR(handle);
			goto unlock;							goto unlock;
		}								}
	}								}

	if (ext3_should_journal_data(inode)) {				if (ext3_should_journal_data(inode)) {
		BUFFER_TRACE(bh, "get write access");				BUFFER_TRACE(bh, "get write access");
		err = ext3_journal_get_write_access(handle, b			err = ext3_journal_get_write_access(handle, b
		if (err)							if (err)
			goto stop;							goto stop;
	}								}

	zero_user(page, offset, length);				zero_user(page, offset, length);
	BUFFER_TRACE(bh, "zeroed end of block");			BUFFER_TRACE(bh, "zeroed end of block");

	err = 0;							err = 0;
	if (ext3_should_journal_data(inode)) {				if (ext3_should_journal_data(inode)) {
		err = ext3_journal_dirty_metadata(handle, bh)			err = ext3_journal_dirty_metadata(handle, bh)
	} else {							} else {
		if (ext3_should_order_data(inode))				if (ext3_should_order_data(inode))
			err = ext3_journal_dirty_data(handle,				err = ext3_journal_dirty_data(handle,
		mark_buffer_dirty(bh);						mark_buffer_dirty(bh);
	}								}
stop:								stop:
	if (handle)							if (handle)
		ext3_journal_stop(handle);					ext3_journal_stop(handle);

unlock:								unlock:
	unlock_page(page);						unlock_page(page);
	page_cache_release(page);					page_cache_release(page);
	return err;							return err;
}								}

/*								/*
 * Probably it should be a library function... search for fir	 * Probably it should be a library function... search for fir
 * or memcmp with zero_page, whatever is better for particula	 * or memcmp with zero_page, whatever is better for particula
 * Linus?							 * Linus?
 */								 */
static inline int all_zeroes(__le32 *p, __le32 *q)		static inline int all_zeroes(__le32 *p, __le32 *q)
{								{
	while (p < q)							while (p < q)
		if (*p++)							if (*p++)
			return 0;							return 0;
	return 1;							return 1;
}								}

/**								/**
 *	ext3_find_shared - find the indirect blocks for parti	 *	ext3_find_shared - find the indirect blocks for parti
 *	@inode:	  inode in question				 *	@inode:	  inode in question
 *	@depth:	  depth of the affected branch			 *	@depth:	  depth of the affected branch
 *	@offsets: offsets of pointers in that branch (see ext	 *	@offsets: offsets of pointers in that branch (see ext
 *	@chain:	  place to store the pointers to partial indi	 *	@chain:	  place to store the pointers to partial indi
 *	@top:	  place to the (detached) top of branch		 *	@top:	  place to the (detached) top of branch
 *								 *
 *	This is a helper function used by ext3_truncate().	 *	This is a helper function used by ext3_truncate().
 *								 *
 *	When we do truncate() we may have to clean the ends o	 *	When we do truncate() we may have to clean the ends o
 *	indirect blocks but leave the blocks themselves alive	 *	indirect blocks but leave the blocks themselves alive
 *	partially truncated if some data below the new i_size	 *	partially truncated if some data below the new i_size
 *	from it (and it is on the path to the first completel	 *	from it (and it is on the path to the first completel
 *	data block, indeed).  We have to free the top of that	 *	data block, indeed).  We have to free the top of that
 *	with everything to the right of the path. Since no al	 *	with everything to the right of the path. Since no al
 *	past the truncation point is possible until ext3_trun	 *	past the truncation point is possible until ext3_trun
 *	finishes, we may safely do the latter, but top of bra	 *	finishes, we may safely do the latter, but top of bra
 *	require special attention - pageout below the truncat	 *	require special attention - pageout below the truncat
 *	might try to populate it.				 *	might try to populate it.
 *								 *
 *	We atomically detach the top of branch from the tree,	 *	We atomically detach the top of branch from the tree,
 *	block number of its root in *@top, pointers to buffer	 *	block number of its root in *@top, pointers to buffer
 *	partially truncated blocks - in @chain[].bh and point	 *	partially truncated blocks - in @chain[].bh and point
 *	their last elements that should not be removed - in	 *	their last elements that should not be removed - in
 *	@chain[].p. Return value is the pointer to last fille	 *	@chain[].p. Return value is the pointer to last fille
 *	of @chain.						 *	of @chain.
 *								 *
 *	The work left to caller to do the actual freeing of s	 *	The work left to caller to do the actual freeing of s
 *		a) free the subtree starting from *@top		 *		a) free the subtree starting from *@top
 *		b) free the subtrees whose roots are stored i	 *		b) free the subtrees whose roots are stored i
 *			(@chain[i].p+1 .. end of @chain[i].bh	 *			(@chain[i].p+1 .. end of @chain[i].bh
 *		c) free the subtrees growing from the inode p	 *		c) free the subtrees growing from the inode p
 *			(no partially truncated stuff there).	 *			(no partially truncated stuff there).

static Indirect *ext3_find_shared(struct inode *inode, int de	static Indirect *ext3_find_shared(struct inode *inode, int de
			int offsets[4], Indirect chain[4], __				int offsets[4], Indirect chain[4], __
{								{
	Indirect *partial, *p;						Indirect *partial, *p;
	int k, err;							int k, err;

	*top = 0;							*top = 0;
	/* Make k index the deepest non-null offset + 1 */		/* Make k index the deepest non-null offset + 1 */
	for (k = depth; k > 1 && !offsets[k-1]; k--)			for (k = depth; k > 1 && !offsets[k-1]; k--)
		;								;
	partial = ext3_get_branch(inode, k, offsets, chain, &		partial = ext3_get_branch(inode, k, offsets, chain, &
	/* Writer: pointers */						/* Writer: pointers */
	if (!partial)							if (!partial)
		partial = chain + k-1;						partial = chain + k-1;
	/*								/*
	 * If the branch acquired continuation since we've lo		 * If the branch acquired continuation since we've lo
	 * fine, it should all survive and (new) top doesn't 		 * fine, it should all survive and (new) top doesn't 
	 */								 */
	if (!partial->key && *partial->p)				if (!partial->key && *partial->p)
		/* Writer: end */						/* Writer: end */
		goto no_top;							goto no_top;
	for (p=partial; p>chain && all_zeroes((__le32*)p->bh-		for (p=partial; p>chain && all_zeroes((__le32*)p->bh-
		;								;
	/*								/*
	 * OK, we've found the last block that must survive. 		 * OK, we've found the last block that must survive. 
	 * branch should be detached before unlocking. Howeve		 * branch should be detached before unlocking. Howeve
	 * of branch is all ours and does not grow immediatel		 * of branch is all ours and does not grow immediatel
	 * it's easier to cheat and just decrement partial->p		 * it's easier to cheat and just decrement partial->p
	 */								 */
	if (p == chain + k - 1 && p > chain) {				if (p == chain + k - 1 && p > chain) {
		p->p--;								p->p--;
	} else {							} else {
		*top = *p->p;							*top = *p->p;
		/* Nope, don't do this in ext3.  Must leave t			/* Nope, don't do this in ext3.  Must leave t
#if 0								#if 0
		*p->p = 0;							*p->p = 0;
#endif								#endif
	}								}
	/* Writer: end */						/* Writer: end */

	while(partial > p) {						while(partial > p) {
		brelse(partial->bh);						brelse(partial->bh);
		partial--;							partial--;
	}								}
no_top:								no_top:
	return partial;							return partial;
}								}

/*								/*
 * Zero a number of block pointers in either an inode or an i	 * Zero a number of block pointers in either an inode or an i
 * If we restart the transaction we must again get write acce	 * If we restart the transaction we must again get write acce
 * indirect block for further modification.			 * indirect block for further modification.
 *								 *
 * We release `count' blocks on disk, but (last - first) may 	 * We release `count' blocks on disk, but (last - first) may 
 * than `count' because there can be holes in there.		 * than `count' because there can be holes in there.
 */								 */
static void ext3_clear_blocks(handle_t *handle, struct inode 	static void ext3_clear_blocks(handle_t *handle, struct inode 
		struct buffer_head *bh, ext3_fsblk_t block_to			struct buffer_head *bh, ext3_fsblk_t block_to
		unsigned long count, __le32 *first, __le32 *l			unsigned long count, __le32 *first, __le32 *l
{								{
	__le32 *p;							__le32 *p;
	if (try_to_extend_transaction(handle, inode)) {			if (try_to_extend_transaction(handle, inode)) {
		if (bh) {							if (bh) {
			BUFFER_TRACE(bh, "call ext3_journal_d				BUFFER_TRACE(bh, "call ext3_journal_d
			if (ext3_journal_dirty_metadata(handl				if (ext3_journal_dirty_metadata(handl
				return;								return;
		}								}
		ext3_mark_inode_dirty(handle, inode);				ext3_mark_inode_dirty(handle, inode);
		truncate_restart_transaction(handle, inode);			truncate_restart_transaction(handle, inode);
		if (bh) {							if (bh) {
			BUFFER_TRACE(bh, "retaking write acce				BUFFER_TRACE(bh, "retaking write acce
			if (ext3_journal_get_write_access(han				if (ext3_journal_get_write_access(han
				return;								return;
		}								}
	}								}

	/*								/*
	 * Any buffers which are on the journal will be in me		 * Any buffers which are on the journal will be in me
	 * them on the hash table so journal_revoke() will ru		 * them on the hash table so journal_revoke() will ru
	 * on them.  We've already detached each block from t		 * on them.  We've already detached each block from t
	 * bforget() in journal_forget() should be safe.		 * bforget() in journal_forget() should be safe.
	 *								 *
	 * AKPM: turn on bforget in journal_forget()!!!			 * AKPM: turn on bforget in journal_forget()!!!
	 */								 */
	for (p = first; p < last; p++) {				for (p = first; p < last; p++) {
		u32 nr = le32_to_cpu(*p);					u32 nr = le32_to_cpu(*p);
		if (nr) {							if (nr) {
			struct buffer_head *bh;						struct buffer_head *bh;

			*p = 0;								*p = 0;
			bh = sb_find_get_block(inode->i_sb, n				bh = sb_find_get_block(inode->i_sb, n
			ext3_forget(handle, 0, inode, bh, nr)				ext3_forget(handle, 0, inode, bh, nr)
		}								}
	}								}

	ext3_free_blocks(handle, inode, block_to_free, count)		ext3_free_blocks(handle, inode, block_to_free, count)
}								}

/**								/**
 * ext3_free_data - free a list of data blocks			 * ext3_free_data - free a list of data blocks
 * @handle:	handle for this transaction			 * @handle:	handle for this transaction
 * @inode:	inode we are dealing with			 * @inode:	inode we are dealing with
 * @this_bh:	indirect buffer_head which contains *@first a	 * @this_bh:	indirect buffer_head which contains *@first a
 * @first:	array of block numbers				 * @first:	array of block numbers
 * @last:	points immediately past the end of array	 * @last:	points immediately past the end of array
 *								 *
 * We are freeing all blocks referred from that array (number	 * We are freeing all blocks referred from that array (number
 * little-endian 32-bit) and updating @inode->i_blocks approp	 * little-endian 32-bit) and updating @inode->i_blocks approp
 *								 *
 * We accumulate contiguous runs of blocks to free.  Convenie	 * We accumulate contiguous runs of blocks to free.  Convenie
 * blocks are contiguous then releasing them at one time will	 * blocks are contiguous then releasing them at one time will
 * or two bitmap blocks (+ group descriptor(s) and superblock	 * or two bitmap blocks (+ group descriptor(s) and superblock
 * actually use a lot of journal space.				 * actually use a lot of journal space.
 *								 *
 * @this_bh will be %NULL if @first and @last point into the 	 * @this_bh will be %NULL if @first and @last point into the 
 * block pointers.						 * block pointers.
 */								 */
static void ext3_free_data(handle_t *handle, struct inode *in	static void ext3_free_data(handle_t *handle, struct inode *in
			   struct buffer_head *this_bh,					   struct buffer_head *this_bh,
			   __le32 *first, __le32 *last)					   __le32 *first, __le32 *last)
{								{
	ext3_fsblk_t block_to_free = 0;    /* Starting block 		ext3_fsblk_t block_to_free = 0;    /* Starting block 
	unsigned long count = 0;	    /* Number of bloc		unsigned long count = 0;	    /* Number of bloc
	__le32 *block_to_free_p = NULL;	    /* Pointer into i		__le32 *block_to_free_p = NULL;	    /* Pointer into i
					       corresponding 						       corresponding 
					       block_to_free 						       block_to_free 
	ext3_fsblk_t nr;		    /* Current block 		ext3_fsblk_t nr;		    /* Current block 
	__le32 *p;			    /* Pointer into i		__le32 *p;			    /* Pointer into i
					       for current bl						       for current bl
	int err;							int err;

	if (this_bh) {				/* For indire		if (this_bh) {				/* For indire
		BUFFER_TRACE(this_bh, "get_write_access");			BUFFER_TRACE(this_bh, "get_write_access");
		err = ext3_journal_get_write_access(handle, t			err = ext3_journal_get_write_access(handle, t
		/* Important: if we can't update the indirect			/* Important: if we can't update the indirect
		 * to the blocks, we can't free them. */			 * to the blocks, we can't free them. */
		if (err)							if (err)
			return;								return;
	}								}

	for (p = first; p < last; p++) {				for (p = first; p < last; p++) {
		nr = le32_to_cpu(*p);						nr = le32_to_cpu(*p);
		if (nr) {							if (nr) {
			/* accumulate blocks to free if they'				/* accumulate blocks to free if they'
			if (count == 0) {						if (count == 0) {
				block_to_free = nr;						block_to_free = nr;
				block_to_free_p = p;						block_to_free_p = p;
				count = 1;							count = 1;
			} else if (nr == block_to_free + coun				} else if (nr == block_to_free + coun
				count++;							count++;
			} else {							} else {
				ext3_clear_blocks(handle, ino					ext3_clear_blocks(handle, ino
						  block_to_fr							  block_to_fr
						  count, bloc							  count, bloc
				block_to_free = nr;						block_to_free = nr;
				block_to_free_p = p;						block_to_free_p = p;
				count = 1;							count = 1;
			}								}
		}								}
	}								}

	if (count > 0)							if (count > 0)
		ext3_clear_blocks(handle, inode, this_bh, blo			ext3_clear_blocks(handle, inode, this_bh, blo
				  count, block_to_free_p, p);					  count, block_to_free_p, p);

	if (this_bh) {							if (this_bh) {
		BUFFER_TRACE(this_bh, "call ext3_journal_dirt			BUFFER_TRACE(this_bh, "call ext3_journal_dirt

		/*								/*
		 * The buffer head should have an attached jo			 * The buffer head should have an attached jo
		 * point. However, if the data is corrupted a			 * point. However, if the data is corrupted a
		 * block pointed to itself, it would have bee			 * block pointed to itself, it would have bee
		 * the block was cleared. Check for this inst			 * the block was cleared. Check for this inst
		 */								 */
		if (bh2jh(this_bh))						if (bh2jh(this_bh))
			ext3_journal_dirty_metadata(handle, t				ext3_journal_dirty_metadata(handle, t
		else								else
			ext3_error(inode->i_sb, "ext3_free_da				ext3_error(inode->i_sb, "ext3_free_da
				   "circular indirect block d					   "circular indirect block d
				   "inode=%lu, block=%llu",					   "inode=%lu, block=%llu",
				   inode->i_ino,						   inode->i_ino,
				   (unsigned long long)this_b					   (unsigned long long)this_b
	}								}
}								}

/**								/**
 *	ext3_free_branches - free an array of branches		 *	ext3_free_branches - free an array of branches
 *	@handle: JBD handle for this transaction		 *	@handle: JBD handle for this transaction
 *	@inode:	inode we are dealing with			 *	@inode:	inode we are dealing with
 *	@parent_bh: the buffer_head which contains *@first an	 *	@parent_bh: the buffer_head which contains *@first an
 *	@first:	array of block numbers				 *	@first:	array of block numbers
 *	@last:	pointer immediately past the end of array	 *	@last:	pointer immediately past the end of array
 *	@depth:	depth of the branches to free			 *	@depth:	depth of the branches to free
 *								 *
 *	We are freeing all blocks referred from these branche	 *	We are freeing all blocks referred from these branche
 *	stored as little-endian 32-bit) and updating @inode->	 *	stored as little-endian 32-bit) and updating @inode->
 *	appropriately.						 *	appropriately.
 */								 */
static void ext3_free_branches(handle_t *handle, struct inode	static void ext3_free_branches(handle_t *handle, struct inode
			       struct buffer_head *parent_bh,				       struct buffer_head *parent_bh,
			       __le32 *first, __le32 *last, i				       __le32 *first, __le32 *last, i
{								{
	ext3_fsblk_t nr;						ext3_fsblk_t nr;
	__le32 *p;							__le32 *p;

	if (is_handle_aborted(handle))					if (is_handle_aborted(handle))
		return;								return;

	if (depth--) {							if (depth--) {
		struct buffer_head *bh;						struct buffer_head *bh;
		int addr_per_block = EXT3_ADDR_PER_BLOCK(inod			int addr_per_block = EXT3_ADDR_PER_BLOCK(inod
		p = last;							p = last;
		while (--p >= first) {						while (--p >= first) {
			nr = le32_to_cpu(*p);						nr = le32_to_cpu(*p);
			if (!nr)							if (!nr)
				continue;		/* A 					continue;		/* A 

			/* Go read the buffer for the next le				/* Go read the buffer for the next le
			bh = sb_bread(inode->i_sb, nr);					bh = sb_bread(inode->i_sb, nr);

			/*								/*
			 * A read failure? Report error and c				 * A read failure? Report error and c
			 * (should be rare).						 * (should be rare).
			 */								 */
			if (!bh) {							if (!bh) {
				ext3_error(inode->i_sb, "ext3					ext3_error(inode->i_sb, "ext3
					   "Read failure, ino						   "Read failure, ino
					   inode->i_ino, nr);						   inode->i_ino, nr);
				continue;							continue;
			}								}

			/* This zaps the entire block.  Botto				/* This zaps the entire block.  Botto
			BUFFER_TRACE(bh, "free child branches				BUFFER_TRACE(bh, "free child branches
			ext3_free_branches(handle, inode, bh,				ext3_free_branches(handle, inode, bh,
					   (__le32*)bh->b_dat						   (__le32*)bh->b_dat
					   (__le32*)bh->b_dat						   (__le32*)bh->b_dat
					   depth);							   depth);

			/*								/*
			 * Everything below this this pointer				 * Everything below this this pointer
			 * released.  Now let this top-of-sub				 * released.  Now let this top-of-sub
			 *								 *
			 * We want the freeing of this indire				 * We want the freeing of this indire
			 * atomic in the journal with the upd				 * atomic in the journal with the upd
			 * bitmap block which owns it.  So ma				 * bitmap block which owns it.  So ma
			 * the journal.							 * the journal.
			 *								 *
			 * We zero the parent pointer *after*				 * We zero the parent pointer *after*
			 * pointee in the bitmaps, so if exte				 * pointee in the bitmaps, so if exte
			 * for some reason fails to put the b				 * for some reason fails to put the b
			 * the release into the same transact				 * the release into the same transact
			 * will merely complain about releasi				 * will merely complain about releasi
			 * rather than leaking blocks.					 * rather than leaking blocks.
			 */								 */
			if (is_handle_aborted(handle))					if (is_handle_aborted(handle))
				return;								return;
			if (try_to_extend_transaction(handle,				if (try_to_extend_transaction(handle,
				ext3_mark_inode_dirty(handle,					ext3_mark_inode_dirty(handle,
				truncate_restart_transaction(					truncate_restart_transaction(
			}								}

			/*								/*
			 * We've probably journalled the indi				 * We've probably journalled the indi
			 * times during the truncate.  But it				 * times during the truncate.  But it
			 * needed and we now drop it from the				 * needed and we now drop it from the
			 * journal_revoke().						 * journal_revoke().
			 *								 *
			 * That's easy if it's exclusively pa				 * That's easy if it's exclusively pa
			 * transaction.  But if it's part of 				 * transaction.  But if it's part of 
			 * transaction then journal_forget() 				 * transaction then journal_forget() 
			 * brelse() it.  That means that if t				 * brelse() it.  That means that if t
			 * block is reallocated in ext3_get_b				 * block is reallocated in ext3_get_b
			 * unmap_underlying_metadata() will f				 * unmap_underlying_metadata() will f
			 * and will try to get rid of it.  da				 * and will try to get rid of it.  da
			 * we don't allow a block to be reall				 * we don't allow a block to be reall
			 * a transaction freeing it has fully				 * a transaction freeing it has fully
			 *								 *
			 * We also have to make sure journal 				 * We also have to make sure journal 
			 * crash does not overwrite non-journ				 * crash does not overwrite non-journ
			 * with old metadata when the block g				 * with old metadata when the block g
			 * data.  Thus we have to store a rev				 * data.  Thus we have to store a rev
			 * block in the same transaction in w				 * block in the same transaction in w
			 * block.							 * block.
			 */								 */
			ext3_forget(handle, 1, inode, bh, bh-				ext3_forget(handle, 1, inode, bh, bh-

			ext3_free_blocks(handle, inode, nr, 1				ext3_free_blocks(handle, inode, nr, 1

			if (parent_bh) {						if (parent_bh) {
				/*								/*
				 * The block which we have ju					 * The block which we have ju
				 * pointed to by an indirect 					 * pointed to by an indirect 
				 */								 */
				BUFFER_TRACE(parent_bh, "get_					BUFFER_TRACE(parent_bh, "get_
				if (!ext3_journal_get_write_a					if (!ext3_journal_get_write_a
															
					*p = 0;								*p = 0;
					BUFFER_TRACE(parent_b						BUFFER_TRACE(parent_b
					"call ext3_journal_di						"call ext3_journal_di
					ext3_journal_dirty_me						ext3_journal_dirty_me
															
				}								}
			}								}
		}								}
	} else {							} else {
		/* We have reached the bottom of the tree. */			/* We have reached the bottom of the tree. */
		BUFFER_TRACE(parent_bh, "free data blocks");			BUFFER_TRACE(parent_bh, "free data blocks");
		ext3_free_data(handle, inode, parent_bh, firs			ext3_free_data(handle, inode, parent_bh, firs
	}								}
}								}

int ext3_can_truncate(struct inode *inode)			int ext3_can_truncate(struct inode *inode)
{								{
	if (S_ISREG(inode->i_mode))					if (S_ISREG(inode->i_mode))
		return 1;							return 1;
	if (S_ISDIR(inode->i_mode))					if (S_ISDIR(inode->i_mode))
		return 1;							return 1;
	if (S_ISLNK(inode->i_mode))					if (S_ISLNK(inode->i_mode))
		return !ext3_inode_is_fast_symlink(inode);			return !ext3_inode_is_fast_symlink(inode);
	return 0;							return 0;
}								}

/*								/*
 * ext3_truncate()						 * ext3_truncate()
 *								 *
 * We block out ext3_get_block() block instantiations across 	 * We block out ext3_get_block() block instantiations across 
 * transaction, and VFS/VM ensures that ext3_truncate() canno	 * transaction, and VFS/VM ensures that ext3_truncate() canno
 * simultaneously on behalf of the same inode.			 * simultaneously on behalf of the same inode.
 *								 *
 * As we work through the truncate and commit bits of it to t	 * As we work through the truncate and commit bits of it to t
 * is one core, guiding principle: the file's tree must alway	 * is one core, guiding principle: the file's tree must alway
 * disk.  We must be able to restart the truncate after a cra	 * disk.  We must be able to restart the truncate after a cra
 *								 *
 * The file's tree may be transiently inconsistent in memory 	 * The file's tree may be transiently inconsistent in memory 
 * probably isn't), but whenever we close off and commit a jo	 * probably isn't), but whenever we close off and commit a jo
 * the contents of (the filesystem + the journal) must be con	 * the contents of (the filesystem + the journal) must be con
 * restartable.  It's pretty simple, really: bottom up, right	 * restartable.  It's pretty simple, really: bottom up, right
 * left-to-right works OK too).					 * left-to-right works OK too).
 *								 *
 * Note that at recovery time, journal replay occurs *before*	 * Note that at recovery time, journal replay occurs *before*
 * truncate against the orphan inode list.			 * truncate against the orphan inode list.
 *								 *
 * The committed inode has the new, desired i_size (which is 	 * The committed inode has the new, desired i_size (which is 
 * i_disksize in this case).  After a crash, ext3_orphan_clea	 * i_disksize in this case).  After a crash, ext3_orphan_clea
 * that this inode's truncate did not complete and it will ag	 * that this inode's truncate did not complete and it will ag
 * ext3_truncate() to have another go.  So there will be inst	 * ext3_truncate() to have another go.  So there will be inst
 * to the right of the truncation point in a crashed ext3 fil	 * to the right of the truncation point in a crashed ext3 fil
 * that's fine - as long as they are linked from the inode, t	 * that's fine - as long as they are linked from the inode, t
 * ext3_truncate() run will find them and release them.		 * ext3_truncate() run will find them and release them.
 */								 */
void ext3_truncate(struct inode *inode)				void ext3_truncate(struct inode *inode)
{								{
	handle_t *handle;						handle_t *handle;
	struct ext3_inode_info *ei = EXT3_I(inode);			struct ext3_inode_info *ei = EXT3_I(inode);
	__le32 *i_data = ei->i_data;					__le32 *i_data = ei->i_data;
	int addr_per_block = EXT3_ADDR_PER_BLOCK(inode->i_sb)		int addr_per_block = EXT3_ADDR_PER_BLOCK(inode->i_sb)
	int offsets[4];							int offsets[4];
	Indirect chain[4];						Indirect chain[4];
	Indirect *partial;						Indirect *partial;
	__le32 nr = 0;							__le32 nr = 0;
	int n;								int n;
	long last_block;						long last_block;
	unsigned blocksize = inode->i_sb->s_blocksize;			unsigned blocksize = inode->i_sb->s_blocksize;

	trace_ext3_truncate_enter(inode);				trace_ext3_truncate_enter(inode);

	if (!ext3_can_truncate(inode))					if (!ext3_can_truncate(inode))
		goto out_notrans;						goto out_notrans;

	if (inode->i_size == 0 && ext3_should_writeback_data(		if (inode->i_size == 0 && ext3_should_writeback_data(
		ext3_set_inode_state(inode, EXT3_STATE_FLUSH_			ext3_set_inode_state(inode, EXT3_STATE_FLUSH_

	handle = start_transaction(inode);				handle = start_transaction(inode);
	if (IS_ERR(handle))						if (IS_ERR(handle))
		goto out_notrans;						goto out_notrans;

	last_block = (inode->i_size + blocksize-1)			last_block = (inode->i_size + blocksize-1)
					>> EXT3_BLOCK_SIZE_BI						>> EXT3_BLOCK_SIZE_BI
	n = ext3_block_to_path(inode, last_block, offsets, NU		n = ext3_block_to_path(inode, last_block, offsets, NU
	if (n == 0)							if (n == 0)
		goto out_stop;	/* error */					goto out_stop;	/* error */

	/*								/*
	 * OK.  This truncate is going to happen.  We add the		 * OK.  This truncate is going to happen.  We add the
	 * orphan list, so that if this truncate spans multip		 * orphan list, so that if this truncate spans multip
	 * and we crash, we will resume the truncate when the		 * and we crash, we will resume the truncate when the
	 * recovers.  It also marks the inode dirty, to catch		 * recovers.  It also marks the inode dirty, to catch
	 *								 *
	 * Implication: the file must always be in a sane, co		 * Implication: the file must always be in a sane, co
	 * truncatable state while each transaction commits.		 * truncatable state while each transaction commits.
	 */								 */
	if (ext3_orphan_add(handle, inode))				if (ext3_orphan_add(handle, inode))
		goto out_stop;							goto out_stop;

	/*								/*
	 * The orphan list entry will now protect us from any		 * The orphan list entry will now protect us from any
	 * occurs before the truncate completes, so it is now		 * occurs before the truncate completes, so it is now
	 * the new, shorter inode size (held for now in i_siz		 * the new, shorter inode size (held for now in i_siz
	 * on-disk inode. We do this via i_disksize, which is		 * on-disk inode. We do this via i_disksize, which is
	 * ext3 *really* writes onto the disk inode.			 * ext3 *really* writes onto the disk inode.
	 */								 */
	ei->i_disksize = inode->i_size;					ei->i_disksize = inode->i_size;

	/*								/*
	 * From here we block out all ext3_get_block() caller		 * From here we block out all ext3_get_block() caller
	 * modify the block allocation tree.				 * modify the block allocation tree.
	 */								 */
	mutex_lock(&ei->truncate_mutex);				mutex_lock(&ei->truncate_mutex);

	if (n == 1) {		/* direct blocks */			if (n == 1) {		/* direct blocks */
		ext3_free_data(handle, inode, NULL, i_data+of			ext3_free_data(handle, inode, NULL, i_data+of
			       i_data + EXT3_NDIR_BLOCKS);				       i_data + EXT3_NDIR_BLOCKS);
		goto do_indirects;						goto do_indirects;
	}								}

	partial = ext3_find_shared(inode, n, offsets, chain, 		partial = ext3_find_shared(inode, n, offsets, chain, 
	/* Kill the top of shared branch (not detached) */		/* Kill the top of shared branch (not detached) */
	if (nr) {							if (nr) {
		if (partial == chain) {						if (partial == chain) {
			/* Shared branch grows from the inode				/* Shared branch grows from the inode
			ext3_free_branches(handle, inode, NUL				ext3_free_branches(handle, inode, NUL
					   &nr, &nr+1, (chain						   &nr, &nr+1, (chain
			*partial->p = 0;						*partial->p = 0;
			/*								/*
			 * We mark the inode dirty prior to r				 * We mark the inode dirty prior to r
			 * and prior to stop.  No need for it				 * and prior to stop.  No need for it
			 */								 */
		} else {							} else {
			/* Shared branch grows from an indire				/* Shared branch grows from an indire
			ext3_free_branches(handle, inode, par				ext3_free_branches(handle, inode, par
					partial->p,							partial->p,
					partial->p+1, (chain+						partial->p+1, (chain+
		}								}
	}								}
	/* Clear the ends of indirect blocks on the shared br		/* Clear the ends of indirect blocks on the shared br
	while (partial > chain) {					while (partial > chain) {
		ext3_free_branches(handle, inode, partial->bh			ext3_free_branches(handle, inode, partial->bh
				   (__le32*)partial->bh->b_da					   (__le32*)partial->bh->b_da
				   (chain+n-1) - partial);					   (chain+n-1) - partial);
		BUFFER_TRACE(partial->bh, "call brelse");			BUFFER_TRACE(partial->bh, "call brelse");
		brelse (partial->bh);						brelse (partial->bh);
		partial--;							partial--;
	}								}
do_indirects:							do_indirects:
	/* Kill the remaining (whole) subtrees */			/* Kill the remaining (whole) subtrees */
	switch (offsets[0]) {						switch (offsets[0]) {
	default:							default:
		nr = i_data[EXT3_IND_BLOCK];					nr = i_data[EXT3_IND_BLOCK];
		if (nr) {							if (nr) {
			ext3_free_branches(handle, inode, NUL				ext3_free_branches(handle, inode, NUL
			i_data[EXT3_IND_BLOCK] = 0;					i_data[EXT3_IND_BLOCK] = 0;
		}								}
	case EXT3_IND_BLOCK:						case EXT3_IND_BLOCK:
		nr = i_data[EXT3_DIND_BLOCK];					nr = i_data[EXT3_DIND_BLOCK];
		if (nr) {							if (nr) {
			ext3_free_branches(handle, inode, NUL				ext3_free_branches(handle, inode, NUL
			i_data[EXT3_DIND_BLOCK] = 0;					i_data[EXT3_DIND_BLOCK] = 0;
		}								}
	case EXT3_DIND_BLOCK:						case EXT3_DIND_BLOCK:
		nr = i_data[EXT3_TIND_BLOCK];					nr = i_data[EXT3_TIND_BLOCK];
		if (nr) {							if (nr) {
			ext3_free_branches(handle, inode, NUL				ext3_free_branches(handle, inode, NUL
			i_data[EXT3_TIND_BLOCK] = 0;					i_data[EXT3_TIND_BLOCK] = 0;
		}								}
	case EXT3_TIND_BLOCK:						case EXT3_TIND_BLOCK:
		;								;
	}								}

	ext3_discard_reservation(inode);				ext3_discard_reservation(inode);

	mutex_unlock(&ei->truncate_mutex);				mutex_unlock(&ei->truncate_mutex);
	inode->i_mtime = inode->i_ctime = CURRENT_TIME_SEC;		inode->i_mtime = inode->i_ctime = CURRENT_TIME_SEC;
	ext3_mark_inode_dirty(handle, inode);				ext3_mark_inode_dirty(handle, inode);

	/*								/*
	 * In a multi-transaction truncate, we only make the 		 * In a multi-transaction truncate, we only make the 
	 * synchronous							 * synchronous
	 */								 */
	if (IS_SYNC(inode))						if (IS_SYNC(inode))
		handle->h_sync = 1;						handle->h_sync = 1;
out_stop:							out_stop:
	/*								/*
	 * If this was a simple ftruncate(), and the file wil		 * If this was a simple ftruncate(), and the file wil
	 * then we need to clear up the orphan record which w		 * then we need to clear up the orphan record which w
	 * However, if this was a real unlink then we were ca		 * However, if this was a real unlink then we were ca
	 * ext3_evict_inode(), and we allow that function to 		 * ext3_evict_inode(), and we allow that function to 
	 * orphan info for us.						 * orphan info for us.
	 */								 */
	if (inode->i_nlink)						if (inode->i_nlink)
		ext3_orphan_del(handle, inode);					ext3_orphan_del(handle, inode);

	ext3_journal_stop(handle);					ext3_journal_stop(handle);
	trace_ext3_truncate_exit(inode);				trace_ext3_truncate_exit(inode);
	return;								return;
out_notrans:							out_notrans:
	/*								/*
	 * Delete the inode from orphan list so that it doesn		 * Delete the inode from orphan list so that it doesn
	 * forever and trigger assertion on umount.			 * forever and trigger assertion on umount.
	 */								 */
	if (inode->i_nlink)						if (inode->i_nlink)
		ext3_orphan_del(NULL, inode);					ext3_orphan_del(NULL, inode);
	trace_ext3_truncate_exit(inode);				trace_ext3_truncate_exit(inode);
}								}

static ext3_fsblk_t ext3_get_inode_block(struct super_block *	static ext3_fsblk_t ext3_get_inode_block(struct super_block *
		unsigned long ino, struct ext3_iloc *iloc)			unsigned long ino, struct ext3_iloc *iloc)
{								{
	unsigned long block_group;					unsigned long block_group;
	unsigned long offset;						unsigned long offset;
	ext3_fsblk_t block;						ext3_fsblk_t block;
	struct ext3_group_desc *gdp;					struct ext3_group_desc *gdp;

	if (!ext3_valid_inum(sb, ino)) {				if (!ext3_valid_inum(sb, ino)) {
		/*								/*
		 * This error is already checked for in namei			 * This error is already checked for in namei
		 * looking at an NFS filehandle, in which cas			 * looking at an NFS filehandle, in which cas
		 * report is needed						 * report is needed
		 */								 */
		return 0;							return 0;
	}								}

	block_group = (ino - 1) / EXT3_INODES_PER_GROUP(sb);		block_group = (ino - 1) / EXT3_INODES_PER_GROUP(sb);
	gdp = ext3_get_group_desc(sb, block_group, NULL);		gdp = ext3_get_group_desc(sb, block_group, NULL);
	if (!gdp)							if (!gdp)
		return 0;							return 0;
	/*								/*
	 * Figure out the offset within the block group inode		 * Figure out the offset within the block group inode
	 */								 */
	offset = ((ino - 1) % EXT3_INODES_PER_GROUP(sb)) *		offset = ((ino - 1) % EXT3_INODES_PER_GROUP(sb)) *
		EXT3_INODE_SIZE(sb);						EXT3_INODE_SIZE(sb);
	block = le32_to_cpu(gdp->bg_inode_table) +			block = le32_to_cpu(gdp->bg_inode_table) +
		(offset >> EXT3_BLOCK_SIZE_BITS(sb));				(offset >> EXT3_BLOCK_SIZE_BITS(sb));

	iloc->block_group = block_group;				iloc->block_group = block_group;
	iloc->offset = offset & (EXT3_BLOCK_SIZE(sb) - 1);		iloc->offset = offset & (EXT3_BLOCK_SIZE(sb) - 1);
	return block;							return block;
}								}

/*								/*
 * ext3_get_inode_loc returns with an extra refcount against 	 * ext3_get_inode_loc returns with an extra refcount against 
 * underlying buffer_head on success. If 'in_mem' is true, we	 * underlying buffer_head on success. If 'in_mem' is true, we
 * data in memory that is needed to recreate the on-disk vers	 * data in memory that is needed to recreate the on-disk vers
 * inode.							 * inode.
 */								 */
static int __ext3_get_inode_loc(struct inode *inode,		static int __ext3_get_inode_loc(struct inode *inode,
				struct ext3_iloc *iloc, int i					struct ext3_iloc *iloc, int i
{								{
	ext3_fsblk_t block;						ext3_fsblk_t block;
	struct buffer_head *bh;						struct buffer_head *bh;

	block = ext3_get_inode_block(inode->i_sb, inode->i_in		block = ext3_get_inode_block(inode->i_sb, inode->i_in
	if (!block)							if (!block)
		return -EIO;							return -EIO;

	bh = sb_getblk(inode->i_sb, block);				bh = sb_getblk(inode->i_sb, block);
	if (!bh) {							if (!bh) {
		ext3_error (inode->i_sb, "ext3_get_inode_loc"			ext3_error (inode->i_sb, "ext3_get_inode_loc"
				"unable to read inode block -					"unable to read inode block -
				"inode=%lu, block="E3FSBLK,					"inode=%lu, block="E3FSBLK,
				 inode->i_ino, block);						 inode->i_ino, block);
		return -EIO;							return -EIO;
	}								}
	if (!buffer_uptodate(bh)) {					if (!buffer_uptodate(bh)) {
		lock_buffer(bh);						lock_buffer(bh);

		/*								/*
		 * If the buffer has the write error flag, we			 * If the buffer has the write error flag, we
		 * to write out another inode in the same blo			 * to write out another inode in the same blo
		 * case, we don't have to read the block beca			 * case, we don't have to read the block beca
		 * read the old inode data successfully.			 * read the old inode data successfully.
		 */								 */
		if (buffer_write_io_error(bh) && !buffer_upto			if (buffer_write_io_error(bh) && !buffer_upto
			set_buffer_uptodate(bh);					set_buffer_uptodate(bh);

		if (buffer_uptodate(bh)) {					if (buffer_uptodate(bh)) {
			/* someone brought it uptodate while 				/* someone brought it uptodate while 
			unlock_buffer(bh);						unlock_buffer(bh);
			goto has_buffer;						goto has_buffer;
		}								}

		/*								/*
		 * If we have all information of the inode in			 * If we have all information of the inode in
		 * is the only valid inode in the block, we n			 * is the only valid inode in the block, we n
		 * block.							 * block.
		 */								 */
		if (in_mem) {							if (in_mem) {
			struct buffer_head *bitmap_bh;					struct buffer_head *bitmap_bh;
			struct ext3_group_desc *desc;					struct ext3_group_desc *desc;
			int inodes_per_buffer;						int inodes_per_buffer;
			int inode_offset, i;						int inode_offset, i;
			int block_group;						int block_group;
			int start;							int start;

			block_group = (inode->i_ino - 1) /				block_group = (inode->i_ino - 1) /
					EXT3_INODES_PER_GROUP						EXT3_INODES_PER_GROUP
			inodes_per_buffer = bh->b_size /				inodes_per_buffer = bh->b_size /
				EXT3_INODE_SIZE(inode->i_sb);					EXT3_INODE_SIZE(inode->i_sb);
			inode_offset = ((inode->i_ino - 1) %				inode_offset = ((inode->i_ino - 1) %
					EXT3_INODES_PER_GROUP						EXT3_INODES_PER_GROUP
			start = inode_offset & ~(inodes_per_b				start = inode_offset & ~(inodes_per_b

			/* Is the inode bitmap in cache? */				/* Is the inode bitmap in cache? */
			desc = ext3_get_group_desc(inode->i_s				desc = ext3_get_group_desc(inode->i_s
						block_group, 							block_group, 
			if (!desc)							if (!desc)
				goto make_io;							goto make_io;

			bitmap_bh = sb_getblk(inode->i_sb,				bitmap_bh = sb_getblk(inode->i_sb,
					le32_to_cpu(desc->bg_						le32_to_cpu(desc->bg_
			if (!bitmap_bh)							if (!bitmap_bh)
				goto make_io;							goto make_io;

			/*								/*
			 * If the inode bitmap isn't in cache				 * If the inode bitmap isn't in cache
			 * optimisation may end up performing				 * optimisation may end up performing
			 * of one, so skip it.						 * of one, so skip it.
			 */								 */
			if (!buffer_uptodate(bitmap_bh)) {				if (!buffer_uptodate(bitmap_bh)) {
				brelse(bitmap_bh);						brelse(bitmap_bh);
				goto make_io;							goto make_io;
			}								}
			for (i = start; i < start + inodes_pe				for (i = start; i < start + inodes_pe
				if (i == inode_offset)						if (i == inode_offset)
					continue;							continue;
				if (ext3_test_bit(i, bitmap_b					if (ext3_test_bit(i, bitmap_b
					break;								break;
			}								}
			brelse(bitmap_bh);						brelse(bitmap_bh);
			if (i == start + inodes_per_buffer) {				if (i == start + inodes_per_buffer) {
				/* all other inodes are free,					/* all other inodes are free,
				memset(bh->b_data, 0, bh->b_s					memset(bh->b_data, 0, bh->b_s
				set_buffer_uptodate(bh);					set_buffer_uptodate(bh);
				unlock_buffer(bh);						unlock_buffer(bh);
				goto has_buffer;						goto has_buffer;
			}								}
		}								}

make_io:							make_io:
		/*								/*
		 * There are other valid inodes in the buffer			 * There are other valid inodes in the buffer
		 * has in-inode xattrs, or we don't have this			 * has in-inode xattrs, or we don't have this
		 * Read the block from disk.					 * Read the block from disk.
		 */								 */
		trace_ext3_load_inode(inode);					trace_ext3_load_inode(inode);
		get_bh(bh);							get_bh(bh);
		bh->b_end_io = end_buffer_read_sync;				bh->b_end_io = end_buffer_read_sync;
		submit_bh(READ | REQ_META | REQ_PRIO, bh);			submit_bh(READ | REQ_META | REQ_PRIO, bh);
		wait_on_buffer(bh);						wait_on_buffer(bh);
		if (!buffer_uptodate(bh)) {					if (!buffer_uptodate(bh)) {
			ext3_error(inode->i_sb, "ext3_get_ino				ext3_error(inode->i_sb, "ext3_get_ino
					"unable to read inode						"unable to read inode
					"inode=%lu, block="E3						"inode=%lu, block="E3
					inode->i_ino, block);						inode->i_ino, block);
			brelse(bh);							brelse(bh);
			return -EIO;							return -EIO;
		}								}
	}								}
has_buffer:							has_buffer:
	iloc->bh = bh;							iloc->bh = bh;
	return 0;							return 0;
}								}

int ext3_get_inode_loc(struct inode *inode, struct ext3_iloc 	int ext3_get_inode_loc(struct inode *inode, struct ext3_iloc 
{								{
	/* We have all inode data except xattrs in memory her		/* We have all inode data except xattrs in memory her
	return __ext3_get_inode_loc(inode, iloc,			return __ext3_get_inode_loc(inode, iloc,
		!ext3_test_inode_state(inode, EXT3_STATE_XATT			!ext3_test_inode_state(inode, EXT3_STATE_XATT
}								}

void ext3_set_inode_flags(struct inode *inode)			void ext3_set_inode_flags(struct inode *inode)
{								{
	unsigned int flags = EXT3_I(inode)->i_flags;			unsigned int flags = EXT3_I(inode)->i_flags;

	inode->i_flags &= ~(S_SYNC|S_APPEND|S_IMMUTABLE|S_NOA		inode->i_flags &= ~(S_SYNC|S_APPEND|S_IMMUTABLE|S_NOA
	if (flags & EXT3_SYNC_FL)					if (flags & EXT3_SYNC_FL)
		inode->i_flags |= S_SYNC;					inode->i_flags |= S_SYNC;
	if (flags & EXT3_APPEND_FL)					if (flags & EXT3_APPEND_FL)
		inode->i_flags |= S_APPEND;					inode->i_flags |= S_APPEND;
	if (flags & EXT3_IMMUTABLE_FL)					if (flags & EXT3_IMMUTABLE_FL)
		inode->i_flags |= S_IMMUTABLE;					inode->i_flags |= S_IMMUTABLE;
	if (flags & EXT3_NOATIME_FL)					if (flags & EXT3_NOATIME_FL)
		inode->i_flags |= S_NOATIME;					inode->i_flags |= S_NOATIME;
	if (flags & EXT3_DIRSYNC_FL)					if (flags & EXT3_DIRSYNC_FL)
		inode->i_flags |= S_DIRSYNC;					inode->i_flags |= S_DIRSYNC;
}								}

/* Propagate flags from i_flags to EXT3_I(inode)->i_flags */	/* Propagate flags from i_flags to EXT3_I(inode)->i_flags */
void ext3_get_inode_flags(struct ext3_inode_info *ei)		void ext3_get_inode_flags(struct ext3_inode_info *ei)
{								{
	unsigned int flags = ei->vfs_inode.i_flags;			unsigned int flags = ei->vfs_inode.i_flags;

	ei->i_flags &= ~(EXT3_SYNC_FL|EXT3_APPEND_FL|			ei->i_flags &= ~(EXT3_SYNC_FL|EXT3_APPEND_FL|
			EXT3_IMMUTABLE_FL|EXT3_NOATIME_FL|EXT				EXT3_IMMUTABLE_FL|EXT3_NOATIME_FL|EXT
	if (flags & S_SYNC)						if (flags & S_SYNC)
		ei->i_flags |= EXT3_SYNC_FL;					ei->i_flags |= EXT3_SYNC_FL;
	if (flags & S_APPEND)						if (flags & S_APPEND)
		ei->i_flags |= EXT3_APPEND_FL;					ei->i_flags |= EXT3_APPEND_FL;
	if (flags & S_IMMUTABLE)					if (flags & S_IMMUTABLE)
		ei->i_flags |= EXT3_IMMUTABLE_FL;				ei->i_flags |= EXT3_IMMUTABLE_FL;
	if (flags & S_NOATIME)						if (flags & S_NOATIME)
		ei->i_flags |= EXT3_NOATIME_FL;					ei->i_flags |= EXT3_NOATIME_FL;
	if (flags & S_DIRSYNC)						if (flags & S_DIRSYNC)
		ei->i_flags |= EXT3_DIRSYNC_FL;					ei->i_flags |= EXT3_DIRSYNC_FL;
}								}

struct inode *ext3_iget(struct super_block *sb, unsigned long	struct inode *ext3_iget(struct super_block *sb, unsigned long
{								{
	struct ext3_iloc iloc;						struct ext3_iloc iloc;
	struct ext3_inode *raw_inode;					struct ext3_inode *raw_inode;
	struct ext3_inode_info *ei;					struct ext3_inode_info *ei;
	struct buffer_head *bh;						struct buffer_head *bh;
	struct inode *inode;						struct inode *inode;
	journal_t *journal = EXT3_SB(sb)->s_journal;			journal_t *journal = EXT3_SB(sb)->s_journal;
	transaction_t *transaction;					transaction_t *transaction;
	long ret;							long ret;
	int block;							int block;
	uid_t i_uid;							uid_t i_uid;
	gid_t i_gid;							gid_t i_gid;

	inode = iget_locked(sb, ino);					inode = iget_locked(sb, ino);
	if (!inode)							if (!inode)
		return ERR_PTR(-ENOMEM);					return ERR_PTR(-ENOMEM);
	if (!(inode->i_state & I_NEW))					if (!(inode->i_state & I_NEW))
		return inode;							return inode;

	ei = EXT3_I(inode);						ei = EXT3_I(inode);
	ei->i_block_alloc_info = NULL;					ei->i_block_alloc_info = NULL;

	ret = __ext3_get_inode_loc(inode, &iloc, 0);			ret = __ext3_get_inode_loc(inode, &iloc, 0);
	if (ret < 0)							if (ret < 0)
		goto bad_inode;							goto bad_inode;
	bh = iloc.bh;							bh = iloc.bh;
	raw_inode = ext3_raw_inode(&iloc);				raw_inode = ext3_raw_inode(&iloc);
	inode->i_mode = le16_to_cpu(raw_inode->i_mode);			inode->i_mode = le16_to_cpu(raw_inode->i_mode);
	i_uid = (uid_t)le16_to_cpu(raw_inode->i_uid_low);		i_uid = (uid_t)le16_to_cpu(raw_inode->i_uid_low);
	i_gid = (gid_t)le16_to_cpu(raw_inode->i_gid_low);		i_gid = (gid_t)le16_to_cpu(raw_inode->i_gid_low);
	if(!(test_opt (inode->i_sb, NO_UID32))) {			if(!(test_opt (inode->i_sb, NO_UID32))) {
		i_uid |= le16_to_cpu(raw_inode->i_uid_high) <			i_uid |= le16_to_cpu(raw_inode->i_uid_high) <
		i_gid |= le16_to_cpu(raw_inode->i_gid_high) <			i_gid |= le16_to_cpu(raw_inode->i_gid_high) <
	}								}
	i_uid_write(inode, i_uid);					i_uid_write(inode, i_uid);
	i_gid_write(inode, i_gid);					i_gid_write(inode, i_gid);
	set_nlink(inode, le16_to_cpu(raw_inode->i_links_count		set_nlink(inode, le16_to_cpu(raw_inode->i_links_count
	inode->i_size = le32_to_cpu(raw_inode->i_size);			inode->i_size = le32_to_cpu(raw_inode->i_size);
	inode->i_atime.tv_sec = (signed)le32_to_cpu(raw_inode		inode->i_atime.tv_sec = (signed)le32_to_cpu(raw_inode
	inode->i_ctime.tv_sec = (signed)le32_to_cpu(raw_inode		inode->i_ctime.tv_sec = (signed)le32_to_cpu(raw_inode
	inode->i_mtime.tv_sec = (signed)le32_to_cpu(raw_inode		inode->i_mtime.tv_sec = (signed)le32_to_cpu(raw_inode
	inode->i_atime.tv_nsec = inode->i_ctime.tv_nsec = ino		inode->i_atime.tv_nsec = inode->i_ctime.tv_nsec = ino

	ei->i_state_flags = 0;						ei->i_state_flags = 0;
	ei->i_dir_start_lookup = 0;					ei->i_dir_start_lookup = 0;
	ei->i_dtime = le32_to_cpu(raw_inode->i_dtime);			ei->i_dtime = le32_to_cpu(raw_inode->i_dtime);
	/* We now have enough fields to check if the inode wa		/* We now have enough fields to check if the inode wa
	 * This is needed because nfsd might try to access de		 * This is needed because nfsd might try to access de
	 * the test is that same one that e2fsck uses			 * the test is that same one that e2fsck uses
	 * NeilBrown 1999oct15						 * NeilBrown 1999oct15
	 */								 */
	if (inode->i_nlink == 0) {					if (inode->i_nlink == 0) {
		if (inode->i_mode == 0 ||					if (inode->i_mode == 0 ||
		    !(EXT3_SB(inode->i_sb)->s_mount_state & E			    !(EXT3_SB(inode->i_sb)->s_mount_state & E
			/* this inode is deleted */					/* this inode is deleted */
			brelse (bh);							brelse (bh);
			ret = -ESTALE;							ret = -ESTALE;
			goto bad_inode;							goto bad_inode;
		}								}
		/* The only unlinked inodes we let through he			/* The only unlinked inodes we let through he
		 * valid i_mode and are being read by the orp			 * valid i_mode and are being read by the orp
		 * recovery code: that's fine, we're about to			 * recovery code: that's fine, we're about to
		 * the process of deleting those. */				 * the process of deleting those. */
	}								}
	inode->i_blocks = le32_to_cpu(raw_inode->i_blocks);		inode->i_blocks = le32_to_cpu(raw_inode->i_blocks);
	ei->i_flags = le32_to_cpu(raw_inode->i_flags);			ei->i_flags = le32_to_cpu(raw_inode->i_flags);
#ifdef EXT3_FRAGMENTS						#ifdef EXT3_FRAGMENTS
	ei->i_faddr = le32_to_cpu(raw_inode->i_faddr);			ei->i_faddr = le32_to_cpu(raw_inode->i_faddr);
	ei->i_frag_no = raw_inode->i_frag;				ei->i_frag_no = raw_inode->i_frag;
	ei->i_frag_size = raw_inode->i_fsize;				ei->i_frag_size = raw_inode->i_fsize;
#endif								#endif
	ei->i_file_acl = le32_to_cpu(raw_inode->i_file_acl);		ei->i_file_acl = le32_to_cpu(raw_inode->i_file_acl);
	if (!S_ISREG(inode->i_mode)) {					if (!S_ISREG(inode->i_mode)) {
		ei->i_dir_acl = le32_to_cpu(raw_inode->i_dir_			ei->i_dir_acl = le32_to_cpu(raw_inode->i_dir_
	} else {							} else {
		inode->i_size |=						inode->i_size |=
			((__u64)le32_to_cpu(raw_inode->i_size				((__u64)le32_to_cpu(raw_inode->i_size
	}								}
	ei->i_disksize = inode->i_size;					ei->i_disksize = inode->i_size;
	inode->i_generation = le32_to_cpu(raw_inode->i_genera		inode->i_generation = le32_to_cpu(raw_inode->i_genera
	ei->i_block_group = iloc.block_group;				ei->i_block_group = iloc.block_group;
	/*								/*
	 * NOTE! The in-memory inode i_data array is in littl		 * NOTE! The in-memory inode i_data array is in littl
	 * even on big-endian machines: we do NOT byteswap th		 * even on big-endian machines: we do NOT byteswap th
	 */								 */
	for (block = 0; block < EXT3_N_BLOCKS; block++)			for (block = 0; block < EXT3_N_BLOCKS; block++)
		ei->i_data[block] = raw_inode->i_block[block]			ei->i_data[block] = raw_inode->i_block[block]
	INIT_LIST_HEAD(&ei->i_orphan);					INIT_LIST_HEAD(&ei->i_orphan);

	/*								/*
	 * Set transaction id's of transactions that have to 		 * Set transaction id's of transactions that have to 
	 * to finish f[data]sync. We set them to currently ru		 * to finish f[data]sync. We set them to currently ru
	 * as we cannot be sure that the inode or some of its		 * as we cannot be sure that the inode or some of its
	 * part of the transaction - the inode could have bee		 * part of the transaction - the inode could have bee
	 * now it is reread from disk.					 * now it is reread from disk.
	 */								 */
	if (journal) {							if (journal) {
		tid_t tid;							tid_t tid;

		spin_lock(&journal->j_state_lock);				spin_lock(&journal->j_state_lock);
		if (journal->j_running_transaction)				if (journal->j_running_transaction)
			transaction = journal->j_running_tran				transaction = journal->j_running_tran
		else								else
			transaction = journal->j_committing_t				transaction = journal->j_committing_t
		if (transaction)						if (transaction)
			tid = transaction->t_tid;					tid = transaction->t_tid;
		else								else
			tid = journal->j_commit_sequence;				tid = journal->j_commit_sequence;
		spin_unlock(&journal->j_state_lock);				spin_unlock(&journal->j_state_lock);
		atomic_set(&ei->i_sync_tid, tid);				atomic_set(&ei->i_sync_tid, tid);
		atomic_set(&ei->i_datasync_tid, tid);				atomic_set(&ei->i_datasync_tid, tid);
	}								}

	if (inode->i_ino >= EXT3_FIRST_INO(inode->i_sb) + 1 &		if (inode->i_ino >= EXT3_FIRST_INO(inode->i_sb) + 1 &
	    EXT3_INODE_SIZE(inode->i_sb) > EXT3_GOOD_OLD_INOD		    EXT3_INODE_SIZE(inode->i_sb) > EXT3_GOOD_OLD_INOD
		/*								/*
		 * When mke2fs creates big inodes it does not			 * When mke2fs creates big inodes it does not
		 * the unused bytes above EXT3_GOOD_OLD_INODE			 * the unused bytes above EXT3_GOOD_OLD_INODE
		 * so ignore those first few inodes.				 * so ignore those first few inodes.
		 */								 */
		ei->i_extra_isize = le16_to_cpu(raw_inode->i_			ei->i_extra_isize = le16_to_cpu(raw_inode->i_
		if (EXT3_GOOD_OLD_INODE_SIZE + ei->i_extra_is			if (EXT3_GOOD_OLD_INODE_SIZE + ei->i_extra_is
		    EXT3_INODE_SIZE(inode->i_sb)) {				    EXT3_INODE_SIZE(inode->i_sb)) {
			brelse (bh);							brelse (bh);
			ret = -EIO;							ret = -EIO;
			goto bad_inode;							goto bad_inode;
		}								}
		if (ei->i_extra_isize == 0) {					if (ei->i_extra_isize == 0) {
			/* The extra space is currently unuse				/* The extra space is currently unuse
			ei->i_extra_isize = sizeof(struct ext				ei->i_extra_isize = sizeof(struct ext
					    EXT3_GOOD_OLD_INO						    EXT3_GOOD_OLD_INO
		} else {							} else {
			__le32 *magic = (void *)raw_inode +				__le32 *magic = (void *)raw_inode +
					EXT3_GOOD_OLD_INODE_S						EXT3_GOOD_OLD_INODE_S
					ei->i_extra_isize;						ei->i_extra_isize;
			if (*magic == cpu_to_le32(EXT3_XATTR_				if (*magic == cpu_to_le32(EXT3_XATTR_
				 ext3_set_inode_state(inode, 					 ext3_set_inode_state(inode, 
		}								}
	} else								} else
		ei->i_extra_isize = 0;						ei->i_extra_isize = 0;

	if (S_ISREG(inode->i_mode)) {					if (S_ISREG(inode->i_mode)) {
		inode->i_op = &ext3_file_inode_operations;			inode->i_op = &ext3_file_inode_operations;
		inode->i_fop = &ext3_file_operations;				inode->i_fop = &ext3_file_operations;
		ext3_set_aops(inode);						ext3_set_aops(inode);
	} else if (S_ISDIR(inode->i_mode)) {				} else if (S_ISDIR(inode->i_mode)) {
		inode->i_op = &ext3_dir_inode_operations;			inode->i_op = &ext3_dir_inode_operations;
		inode->i_fop = &ext3_dir_operations;				inode->i_fop = &ext3_dir_operations;
	} else if (S_ISLNK(inode->i_mode)) {				} else if (S_ISLNK(inode->i_mode)) {
		if (ext3_inode_is_fast_symlink(inode)) {			if (ext3_inode_is_fast_symlink(inode)) {
			inode->i_op = &ext3_fast_symlink_inod				inode->i_op = &ext3_fast_symlink_inod
			nd_terminate_link(ei->i_data, inode->				nd_terminate_link(ei->i_data, inode->
				sizeof(ei->i_data) - 1);					sizeof(ei->i_data) - 1);
		} else {							} else {
			inode->i_op = &ext3_symlink_inode_ope				inode->i_op = &ext3_symlink_inode_ope
			ext3_set_aops(inode);						ext3_set_aops(inode);
		}								}
	} else {							} else {
		inode->i_op = &ext3_special_inode_operations;			inode->i_op = &ext3_special_inode_operations;
		if (raw_inode->i_block[0])					if (raw_inode->i_block[0])
			init_special_inode(inode, inode->i_mo				init_special_inode(inode, inode->i_mo
			   old_decode_dev(le32_to_cpu(raw_ino				   old_decode_dev(le32_to_cpu(raw_ino
		else								else
			init_special_inode(inode, inode->i_mo				init_special_inode(inode, inode->i_mo
			   new_decode_dev(le32_to_cpu(raw_ino				   new_decode_dev(le32_to_cpu(raw_ino
	}								}
	brelse (iloc.bh);						brelse (iloc.bh);
	ext3_set_inode_flags(inode);					ext3_set_inode_flags(inode);
	unlock_new_inode(inode);					unlock_new_inode(inode);
	return inode;							return inode;

bad_inode:							bad_inode:
	iget_failed(inode);						iget_failed(inode);
	return ERR_PTR(ret);						return ERR_PTR(ret);
}								}

/*								/*
 * Post the struct inode info into an on-disk inode location 	 * Post the struct inode info into an on-disk inode location 
 * buffer-cache.  This gobbles the caller's reference to the	 * buffer-cache.  This gobbles the caller's reference to the
 * buffer_head in the inode location struct.			 * buffer_head in the inode location struct.
 *								 *
 * The caller must have write access to iloc->bh.		 * The caller must have write access to iloc->bh.
 */								 */
static int ext3_do_update_inode(handle_t *handle,		static int ext3_do_update_inode(handle_t *handle,
				struct inode *inode,						struct inode *inode,
				struct ext3_iloc *iloc)						struct ext3_iloc *iloc)
{								{
	struct ext3_inode *raw_inode = ext3_raw_inode(iloc);		struct ext3_inode *raw_inode = ext3_raw_inode(iloc);
	struct ext3_inode_info *ei = EXT3_I(inode);			struct ext3_inode_info *ei = EXT3_I(inode);
	struct buffer_head *bh = iloc->bh;				struct buffer_head *bh = iloc->bh;
	int err = 0, rc, block;						int err = 0, rc, block;
	int need_datasync = 0;						int need_datasync = 0;
	__le32 disksize;						__le32 disksize;
	uid_t i_uid;							uid_t i_uid;
	gid_t i_gid;							gid_t i_gid;

again:								again:
	/* we can't allow multiple procs in here at once, its		/* we can't allow multiple procs in here at once, its
	lock_buffer(bh);						lock_buffer(bh);

	/* For fields not not tracking in the in-memory inode		/* For fields not not tracking in the in-memory inode
	 * initialise them to zero for new inodes. */			 * initialise them to zero for new inodes. */
	if (ext3_test_inode_state(inode, EXT3_STATE_NEW))		if (ext3_test_inode_state(inode, EXT3_STATE_NEW))
		memset(raw_inode, 0, EXT3_SB(inode->i_sb)->s_			memset(raw_inode, 0, EXT3_SB(inode->i_sb)->s_

	ext3_get_inode_flags(ei);					ext3_get_inode_flags(ei);
	raw_inode->i_mode = cpu_to_le16(inode->i_mode);			raw_inode->i_mode = cpu_to_le16(inode->i_mode);
	i_uid = i_uid_read(inode);					i_uid = i_uid_read(inode);
	i_gid = i_gid_read(inode);					i_gid = i_gid_read(inode);
	if(!(test_opt(inode->i_sb, NO_UID32))) {			if(!(test_opt(inode->i_sb, NO_UID32))) {
		raw_inode->i_uid_low = cpu_to_le16(low_16_bit			raw_inode->i_uid_low = cpu_to_le16(low_16_bit
		raw_inode->i_gid_low = cpu_to_le16(low_16_bit			raw_inode->i_gid_low = cpu_to_le16(low_16_bit
/*								/*
 * Fix up interoperability with old kernels. Otherwise, old i	 * Fix up interoperability with old kernels. Otherwise, old i
 * re-used with the upper 16 bits of the uid/gid intact		 * re-used with the upper 16 bits of the uid/gid intact
 */								 */
		if(!ei->i_dtime) {						if(!ei->i_dtime) {
			raw_inode->i_uid_high =						raw_inode->i_uid_high =
				cpu_to_le16(high_16_bits(i_ui					cpu_to_le16(high_16_bits(i_ui
			raw_inode->i_gid_high =						raw_inode->i_gid_high =
				cpu_to_le16(high_16_bits(i_gi					cpu_to_le16(high_16_bits(i_gi
		} else {							} else {
			raw_inode->i_uid_high = 0;					raw_inode->i_uid_high = 0;
			raw_inode->i_gid_high = 0;					raw_inode->i_gid_high = 0;
		}								}
	} else {							} else {
		raw_inode->i_uid_low =						raw_inode->i_uid_low =
			cpu_to_le16(fs_high2lowuid(i_uid));				cpu_to_le16(fs_high2lowuid(i_uid));
		raw_inode->i_gid_low =						raw_inode->i_gid_low =
			cpu_to_le16(fs_high2lowgid(i_gid));				cpu_to_le16(fs_high2lowgid(i_gid));
		raw_inode->i_uid_high = 0;					raw_inode->i_uid_high = 0;
		raw_inode->i_gid_high = 0;					raw_inode->i_gid_high = 0;
	}								}
	raw_inode->i_links_count = cpu_to_le16(inode->i_nlink		raw_inode->i_links_count = cpu_to_le16(inode->i_nlink
	disksize = cpu_to_le32(ei->i_disksize);				disksize = cpu_to_le32(ei->i_disksize);
	if (disksize != raw_inode->i_size) {				if (disksize != raw_inode->i_size) {
		need_datasync = 1;						need_datasync = 1;
		raw_inode->i_size = disksize;					raw_inode->i_size = disksize;
	}								}
	raw_inode->i_atime = cpu_to_le32(inode->i_atime.tv_se		raw_inode->i_atime = cpu_to_le32(inode->i_atime.tv_se
	raw_inode->i_ctime = cpu_to_le32(inode->i_ctime.tv_se		raw_inode->i_ctime = cpu_to_le32(inode->i_ctime.tv_se
	raw_inode->i_mtime = cpu_to_le32(inode->i_mtime.tv_se		raw_inode->i_mtime = cpu_to_le32(inode->i_mtime.tv_se
	raw_inode->i_blocks = cpu_to_le32(inode->i_blocks);		raw_inode->i_blocks = cpu_to_le32(inode->i_blocks);
	raw_inode->i_dtime = cpu_to_le32(ei->i_dtime);			raw_inode->i_dtime = cpu_to_le32(ei->i_dtime);
	raw_inode->i_flags = cpu_to_le32(ei->i_flags);			raw_inode->i_flags = cpu_to_le32(ei->i_flags);
#ifdef EXT3_FRAGMENTS						#ifdef EXT3_FRAGMENTS
	raw_inode->i_faddr = cpu_to_le32(ei->i_faddr);			raw_inode->i_faddr = cpu_to_le32(ei->i_faddr);
	raw_inode->i_frag = ei->i_frag_no;				raw_inode->i_frag = ei->i_frag_no;
	raw_inode->i_fsize = ei->i_frag_size;				raw_inode->i_fsize = ei->i_frag_size;
#endif								#endif
	raw_inode->i_file_acl = cpu_to_le32(ei->i_file_acl);		raw_inode->i_file_acl = cpu_to_le32(ei->i_file_acl);
	if (!S_ISREG(inode->i_mode)) {					if (!S_ISREG(inode->i_mode)) {
		raw_inode->i_dir_acl = cpu_to_le32(ei->i_dir_			raw_inode->i_dir_acl = cpu_to_le32(ei->i_dir_
	} else {							} else {
		disksize = cpu_to_le32(ei->i_disksize >> 32);			disksize = cpu_to_le32(ei->i_disksize >> 32);
		if (disksize != raw_inode->i_size_high) {			if (disksize != raw_inode->i_size_high) {
			raw_inode->i_size_high = disksize;				raw_inode->i_size_high = disksize;
			need_datasync = 1;						need_datasync = 1;
		}								}
		if (ei->i_disksize > 0x7fffffffULL) {				if (ei->i_disksize > 0x7fffffffULL) {
			struct super_block *sb = inode->i_sb;				struct super_block *sb = inode->i_sb;
			if (!EXT3_HAS_RO_COMPAT_FEATURE(sb,				if (!EXT3_HAS_RO_COMPAT_FEATURE(sb,
					EXT3_FEATURE_RO_COMPA						EXT3_FEATURE_RO_COMPA
			    EXT3_SB(sb)->s_es->s_rev_level ==				    EXT3_SB(sb)->s_es->s_rev_level ==
					cpu_to_le32(EXT3_GOOD						cpu_to_le32(EXT3_GOOD
			       /* If this is the first large 				       /* If this is the first large 
				* created, add a flag to the 					* created, add a flag to the 
				*/								*/
				unlock_buffer(bh);						unlock_buffer(bh);
				err = ext3_journal_get_write_					err = ext3_journal_get_write_
						EXT3_SB(sb)->							EXT3_SB(sb)->
				if (err)							if (err)
					goto out_brelse;						goto out_brelse;

				ext3_update_dynamic_rev(sb);					ext3_update_dynamic_rev(sb);
				EXT3_SET_RO_COMPAT_FEATURE(sb					EXT3_SET_RO_COMPAT_FEATURE(sb
					EXT3_FEATURE_RO_COMPA						EXT3_FEATURE_RO_COMPA
				handle->h_sync = 1;						handle->h_sync = 1;
				err = ext3_journal_dirty_meta					err = ext3_journal_dirty_meta
						EXT3_SB(sb)->							EXT3_SB(sb)->
				/* get our lock and start ove					/* get our lock and start ove
				goto again;							goto again;
			}								}
		}								}
	}								}
	raw_inode->i_generation = cpu_to_le32(inode->i_genera		raw_inode->i_generation = cpu_to_le32(inode->i_genera
	if (S_ISCHR(inode->i_mode) || S_ISBLK(inode->i_mode))		if (S_ISCHR(inode->i_mode) || S_ISBLK(inode->i_mode))
		if (old_valid_dev(inode->i_rdev)) {				if (old_valid_dev(inode->i_rdev)) {
			raw_inode->i_block[0] =						raw_inode->i_block[0] =
				cpu_to_le32(old_encode_dev(in					cpu_to_le32(old_encode_dev(in
			raw_inode->i_block[1] = 0;					raw_inode->i_block[1] = 0;
		} else {							} else {
			raw_inode->i_block[0] = 0;					raw_inode->i_block[0] = 0;
			raw_inode->i_block[1] =						raw_inode->i_block[1] =
				cpu_to_le32(new_encode_dev(in					cpu_to_le32(new_encode_dev(in
			raw_inode->i_block[2] = 0;					raw_inode->i_block[2] = 0;
		}								}
	} else for (block = 0; block < EXT3_N_BLOCKS; block++		} else for (block = 0; block < EXT3_N_BLOCKS; block++
		raw_inode->i_block[block] = ei->i_data[block]			raw_inode->i_block[block] = ei->i_data[block]
							      |
	if (ei->i_extra_isize)						if (ei->i_extra_isize)
		raw_inode->i_extra_isize = cpu_to_le16(ei->i_			raw_inode->i_extra_isize = cpu_to_le16(ei->i_

	BUFFER_TRACE(bh, "call ext3_journal_dirty_metadata");		BUFFER_TRACE(bh, "call ext3_journal_dirty_metadata");
	unlock_buffer(bh);						unlock_buffer(bh);
	rc = ext3_journal_dirty_metadata(handle, bh);			rc = ext3_journal_dirty_metadata(handle, bh);
	if (!err)							if (!err)
		err = rc;							err = rc;
	ext3_clear_inode_state(inode, EXT3_STATE_NEW);			ext3_clear_inode_state(inode, EXT3_STATE_NEW);

	atomic_set(&ei->i_sync_tid, handle->h_transaction->t_		atomic_set(&ei->i_sync_tid, handle->h_transaction->t_
	if (need_datasync)						if (need_datasync)
		atomic_set(&ei->i_datasync_tid, handle->h_tra			atomic_set(&ei->i_datasync_tid, handle->h_tra
out_brelse:							out_brelse:
	brelse (bh);							brelse (bh);
	ext3_std_error(inode->i_sb, err);				ext3_std_error(inode->i_sb, err);
	return err;							return err;
}								}

/*								/*
 * ext3_write_inode()						 * ext3_write_inode()
 *								 *
 * We are called from a few places:				 * We are called from a few places:
 *								 *
 * - Within generic_file_write() for O_SYNC files.		 * - Within generic_file_write() for O_SYNC files.
 *   Here, there will be no transaction running. We wait for 	 *   Here, there will be no transaction running. We wait for 
 *   transaction to commit.					 *   transaction to commit.
 *								 *
 * - Within sys_sync(), kupdate and such.			 * - Within sys_sync(), kupdate and such.
 *   We wait on commit, if tol to.				 *   We wait on commit, if tol to.
 *								 *
 * - Within prune_icache() (PF_MEMALLOC == true)		 * - Within prune_icache() (PF_MEMALLOC == true)
 *   Here we simply return.  We can't afford to block kswapd 	 *   Here we simply return.  We can't afford to block kswapd 
 *   journal commit.						 *   journal commit.
 *								 *
 * In all cases it is actually safe for us to return without 	 * In all cases it is actually safe for us to return without 
 * because the inode has been copied into a raw inode buffer 	 * because the inode has been copied into a raw inode buffer 
 * ext3_mark_inode_dirty().  This is a correctness thing for 	 * ext3_mark_inode_dirty().  This is a correctness thing for 
 * knfsd.							 * knfsd.
 *								 *
 * Note that we are absolutely dependent upon all inode dirti	 * Note that we are absolutely dependent upon all inode dirti
 * right thing: they *must* call mark_inode_dirty() after dir	 * right thing: they *must* call mark_inode_dirty() after dir
 * which we are interested.					 * which we are interested.
 *								 *
 * It would be a bug for them to not do this.  The code:	 * It would be a bug for them to not do this.  The code:
 *								 *
 *	mark_inode_dirty(inode)					 *	mark_inode_dirty(inode)
 *	stuff();						 *	stuff();
 *	inode->i_size = expr;					 *	inode->i_size = expr;
 *								 *
 * is in error because a kswapd-driven write_inode() could oc	 * is in error because a kswapd-driven write_inode() could oc
 * `stuff()' is running, and the new i_size will be lost.  Pl	 * `stuff()' is running, and the new i_size will be lost.  Pl
 * will no longer be on the superblock's dirty inode list.	 * will no longer be on the superblock's dirty inode list.
 */								 */
int ext3_write_inode(struct inode *inode, struct writeback_co	int ext3_write_inode(struct inode *inode, struct writeback_co
{								{
	if (current->flags & PF_MEMALLOC)				if (current->flags & PF_MEMALLOC)
		return 0;							return 0;

	if (ext3_journal_current_handle()) {				if (ext3_journal_current_handle()) {
		jbd_debug(1, "called recursively, non-PF_MEMA			jbd_debug(1, "called recursively, non-PF_MEMA
		dump_stack();							dump_stack();
		return -EIO;							return -EIO;
	}								}

	if (wbc->sync_mode != WB_SYNC_ALL)				if (wbc->sync_mode != WB_SYNC_ALL)
		return 0;							return 0;

	return ext3_force_commit(inode->i_sb);				return ext3_force_commit(inode->i_sb);
}								}

/*								/*
 * ext3_setattr()						 * ext3_setattr()
 *								 *
 * Called from notify_change.					 * Called from notify_change.
 *								 *
 * We want to trap VFS attempts to truncate the file as soon 	 * We want to trap VFS attempts to truncate the file as soon 
 * possible.  In particular, we want to make sure that when t	 * possible.  In particular, we want to make sure that when t
 * shrinks i_size, we put the inode on the orphan list and mo	 * shrinks i_size, we put the inode on the orphan list and mo
 * i_disksize immediately, so that during the subsequent flus	 * i_disksize immediately, so that during the subsequent flus
 * dirty pages and freeing of disk blocks, we can guarantee t	 * dirty pages and freeing of disk blocks, we can guarantee t
 * commit will leave the blocks being flushed in an unused st	 * commit will leave the blocks being flushed in an unused st
 * disk.  (On recovery, the inode will get truncated and the 	 * disk.  (On recovery, the inode will get truncated and the 
 * be freed, so we have a strong guarantee that no future com	 * be freed, so we have a strong guarantee that no future com
 * leave these blocks visible to the user.)			 * leave these blocks visible to the user.)
 *								 *
 * Called with inode->sem down.					 * Called with inode->sem down.
 */								 */
int ext3_setattr(struct dentry *dentry, struct iattr *attr)	int ext3_setattr(struct dentry *dentry, struct iattr *attr)
{								{
	struct inode *inode = dentry->d_inode;				struct inode *inode = dentry->d_inode;
	int error, rc = 0;						int error, rc = 0;
	const unsigned int ia_valid = attr->ia_valid;			const unsigned int ia_valid = attr->ia_valid;

	error = inode_change_ok(inode, attr);				error = inode_change_ok(inode, attr);
	if (error)							if (error)
		return error;							return error;

	if (is_quota_modification(inode, attr))				if (is_quota_modification(inode, attr))
		dquot_initialize(inode);					dquot_initialize(inode);
	if ((ia_valid & ATTR_UID && !uid_eq(attr->ia_uid, ino		if ((ia_valid & ATTR_UID && !uid_eq(attr->ia_uid, ino
	    (ia_valid & ATTR_GID && !gid_eq(attr->ia_gid, ino		    (ia_valid & ATTR_GID && !gid_eq(attr->ia_gid, ino
		handle_t *handle;						handle_t *handle;

		/* (user+group)*(old+new) structure, inode wr			/* (user+group)*(old+new) structure, inode wr
		 * inode block, ? - but truncate inode update			 * inode block, ? - but truncate inode update
		handle = ext3_journal_start(inode, EXT3_MAXQU			handle = ext3_journal_start(inode, EXT3_MAXQU
					EXT3_MAXQUOTAS_DEL_BL						EXT3_MAXQUOTAS_DEL_BL
		if (IS_ERR(handle)) {						if (IS_ERR(handle)) {
			error = PTR_ERR(handle);					error = PTR_ERR(handle);
			goto err_out;							goto err_out;
		}								}
		error = dquot_transfer(inode, attr);				error = dquot_transfer(inode, attr);
		if (error) {							if (error) {
			ext3_journal_stop(handle);					ext3_journal_stop(handle);
			return error;							return error;
		}								}
		/* Update corresponding info in inode so that			/* Update corresponding info in inode so that
		 * one transaction */						 * one transaction */
		if (attr->ia_valid & ATTR_UID)					if (attr->ia_valid & ATTR_UID)
			inode->i_uid = attr->ia_uid;					inode->i_uid = attr->ia_uid;
		if (attr->ia_valid & ATTR_GID)					if (attr->ia_valid & ATTR_GID)
			inode->i_gid = attr->ia_gid;					inode->i_gid = attr->ia_gid;
		error = ext3_mark_inode_dirty(handle, inode);			error = ext3_mark_inode_dirty(handle, inode);
		ext3_journal_stop(handle);					ext3_journal_stop(handle);
	}								}

	if (attr->ia_valid & ATTR_SIZE)					if (attr->ia_valid & ATTR_SIZE)
		inode_dio_wait(inode);						inode_dio_wait(inode);

	if (S_ISREG(inode->i_mode) &&					if (S_ISREG(inode->i_mode) &&
	    attr->ia_valid & ATTR_SIZE && attr->ia_size < ino		    attr->ia_valid & ATTR_SIZE && attr->ia_size < ino
		handle_t *handle;						handle_t *handle;

		handle = ext3_journal_start(inode, 3);				handle = ext3_journal_start(inode, 3);
		if (IS_ERR(handle)) {						if (IS_ERR(handle)) {
			error = PTR_ERR(handle);					error = PTR_ERR(handle);
			goto err_out;							goto err_out;
		}								}

		error = ext3_orphan_add(handle, inode);				error = ext3_orphan_add(handle, inode);
		if (error) {							if (error) {
			ext3_journal_stop(handle);					ext3_journal_stop(handle);
			goto err_out;							goto err_out;
		}								}
		EXT3_I(inode)->i_disksize = attr->ia_size;			EXT3_I(inode)->i_disksize = attr->ia_size;
		error = ext3_mark_inode_dirty(handle, inode);			error = ext3_mark_inode_dirty(handle, inode);
		ext3_journal_stop(handle);					ext3_journal_stop(handle);
		if (error) {							if (error) {
			/* Some hard fs error must have happe				/* Some hard fs error must have happe
			ext3_orphan_del(NULL, inode);					ext3_orphan_del(NULL, inode);
			goto err_out;							goto err_out;
		}								}
		rc = ext3_block_truncate_page(inode, attr->ia			rc = ext3_block_truncate_page(inode, attr->ia
		if (rc) {							if (rc) {
			/* Cleanup orphan list and exit */				/* Cleanup orphan list and exit */
			handle = ext3_journal_start(inode, 3)				handle = ext3_journal_start(inode, 3)
			if (IS_ERR(handle)) {						if (IS_ERR(handle)) {
				ext3_orphan_del(NULL, inode);					ext3_orphan_del(NULL, inode);
				goto err_out;							goto err_out;
			}								}
			ext3_orphan_del(handle, inode);					ext3_orphan_del(handle, inode);
			ext3_journal_stop(handle);					ext3_journal_stop(handle);
			goto err_out;							goto err_out;
		}								}
	}								}

	if ((attr->ia_valid & ATTR_SIZE) &&				if ((attr->ia_valid & ATTR_SIZE) &&
	    attr->ia_size != i_size_read(inode)) {			    attr->ia_size != i_size_read(inode)) {
		truncate_setsize(inode, attr->ia_size);				truncate_setsize(inode, attr->ia_size);
		ext3_truncate(inode);						ext3_truncate(inode);
	}								}

	setattr_copy(inode, attr);					setattr_copy(inode, attr);
	mark_inode_dirty(inode);					mark_inode_dirty(inode);

	if (ia_valid & ATTR_MODE)					if (ia_valid & ATTR_MODE)
		rc = ext3_acl_chmod(inode);					rc = ext3_acl_chmod(inode);

err_out:							err_out:
	ext3_std_error(inode->i_sb, error);				ext3_std_error(inode->i_sb, error);
	if (!error)							if (!error)
		error = rc;							error = rc;
	return error;							return error;
}								}


/*								/*
 * How many blocks doth make a writepage()?			 * How many blocks doth make a writepage()?
 *								 *
 * With N blocks per page, it may be:				 * With N blocks per page, it may be:
 * N data blocks						 * N data blocks
 * 2 indirect block						 * 2 indirect block
 * 2 dindirect							 * 2 dindirect
 * 1 tindirect							 * 1 tindirect
 * N+5 bitmap blocks (from the above)				 * N+5 bitmap blocks (from the above)
 * N+5 group descriptor summary blocks				 * N+5 group descriptor summary blocks
 * 1 inode block						 * 1 inode block
 * 1 superblock.						 * 1 superblock.
 * 2 * EXT3_SINGLEDATA_TRANS_BLOCKS for the quote files		 * 2 * EXT3_SINGLEDATA_TRANS_BLOCKS for the quote files
 *								 *
 * 3 * (N + 5) + 2 + 2 * EXT3_SINGLEDATA_TRANS_BLOCKS		 * 3 * (N + 5) + 2 + 2 * EXT3_SINGLEDATA_TRANS_BLOCKS
 *								 *
 * With ordered or writeback data it's the same, less the N d	 * With ordered or writeback data it's the same, less the N d
 *								 *
 * If the inode's direct blocks can hold an integral number o	 * If the inode's direct blocks can hold an integral number o
 * page cannot straddle two indirect blocks, and we can only 	 * page cannot straddle two indirect blocks, and we can only 
 * and dindirect block, and the "5" above becomes "3".		 * and dindirect block, and the "5" above becomes "3".
 *								 *
 * This still overestimates under most circumstances.  If we 	 * This still overestimates under most circumstances.  If we 
 * start and end offsets in here as well we could do block_to	 * start and end offsets in here as well we could do block_to
 * block and work out the exact number of indirects which are	 * block and work out the exact number of indirects which are
 */								 */

static int ext3_writepage_trans_blocks(struct inode *inode)	static int ext3_writepage_trans_blocks(struct inode *inode)
{								{
	int bpp = ext3_journal_blocks_per_page(inode);			int bpp = ext3_journal_blocks_per_page(inode);
	int indirects = (EXT3_NDIR_BLOCKS % bpp) ? 5 : 3;		int indirects = (EXT3_NDIR_BLOCKS % bpp) ? 5 : 3;
	int ret;							int ret;

	if (ext3_should_journal_data(inode))				if (ext3_should_journal_data(inode))
		ret = 3 * (bpp + indirects) + 2;				ret = 3 * (bpp + indirects) + 2;
	else								else
		ret = 2 * (bpp + indirects) + indirects + 2;			ret = 2 * (bpp + indirects) + indirects + 2;

#ifdef CONFIG_QUOTA						#ifdef CONFIG_QUOTA
	/* We know that structure was already allocated durin		/* We know that structure was already allocated durin
	 * we will be updating only the data blocks + inodes 		 * we will be updating only the data blocks + inodes 
	ret += EXT3_MAXQUOTAS_TRANS_BLOCKS(inode->i_sb);		ret += EXT3_MAXQUOTAS_TRANS_BLOCKS(inode->i_sb);
#endif								#endif

	return ret;							return ret;
}								}

/*								/*
 * The caller must have previously called ext3_reserve_inode_	 * The caller must have previously called ext3_reserve_inode_
 * Give this, we know that the caller already has write acces	 * Give this, we know that the caller already has write acces
 */								 */
int ext3_mark_iloc_dirty(handle_t *handle,			int ext3_mark_iloc_dirty(handle_t *handle,
		struct inode *inode, struct ext3_iloc *iloc)			struct inode *inode, struct ext3_iloc *iloc)
{								{
	int err = 0;							int err = 0;

	/* the do_update_inode consumes one bh->b_count */		/* the do_update_inode consumes one bh->b_count */
	get_bh(iloc->bh);						get_bh(iloc->bh);

	/* ext3_do_update_inode() does journal_dirty_metadata		/* ext3_do_update_inode() does journal_dirty_metadata
	err = ext3_do_update_inode(handle, inode, iloc);		err = ext3_do_update_inode(handle, inode, iloc);
	put_bh(iloc->bh);						put_bh(iloc->bh);
	return err;							return err;
}								}

/*								/*
 * On success, We end up with an outstanding reference count 	 * On success, We end up with an outstanding reference count 
 * iloc->bh.  This _must_ be cleaned up later.			 * iloc->bh.  This _must_ be cleaned up later.
 */								 */

int								int
ext3_reserve_inode_write(handle_t *handle, struct inode *inod	ext3_reserve_inode_write(handle_t *handle, struct inode *inod
			 struct ext3_iloc *iloc)					 struct ext3_iloc *iloc)
{								{
	int err = 0;							int err = 0;
	if (handle) {							if (handle) {
		err = ext3_get_inode_loc(inode, iloc);				err = ext3_get_inode_loc(inode, iloc);
		if (!err) {							if (!err) {
			BUFFER_TRACE(iloc->bh, "get_write_acc				BUFFER_TRACE(iloc->bh, "get_write_acc
			err = ext3_journal_get_write_access(h				err = ext3_journal_get_write_access(h
			if (err) {							if (err) {
				brelse(iloc->bh);						brelse(iloc->bh);
				iloc->bh = NULL;						iloc->bh = NULL;
			}								}
		}								}
	}								}
	ext3_std_error(inode->i_sb, err);				ext3_std_error(inode->i_sb, err);
	return err;							return err;
}								}

/*								/*
 * What we do here is to mark the in-core inode as clean with	 * What we do here is to mark the in-core inode as clean with
 * dirtiness (it may still be data-dirty).			 * dirtiness (it may still be data-dirty).
 * This means that the in-core inode may be reaped by prune_i	 * This means that the in-core inode may be reaped by prune_i
 * without having to perform any I/O.  This is a very good th	 * without having to perform any I/O.  This is a very good th
 * because *any* task may call prune_icache - even ones which	 * because *any* task may call prune_icache - even ones which
 * have a transaction open against a different journal.		 * have a transaction open against a different journal.
 *								 *
 * Is this cheating?  Not really.  Sure, we haven't written t	 * Is this cheating?  Not really.  Sure, we haven't written t
 * inode out, but prune_icache isn't a user-visible syncing f	 * inode out, but prune_icache isn't a user-visible syncing f
 * Whenever the user wants stuff synced (sys_sync, sys_msync,	 * Whenever the user wants stuff synced (sys_sync, sys_msync,
 * we start and wait on commits.				 * we start and wait on commits.
 */								 */
int ext3_mark_inode_dirty(handle_t *handle, struct inode *ino	int ext3_mark_inode_dirty(handle_t *handle, struct inode *ino
{								{
	struct ext3_iloc iloc;						struct ext3_iloc iloc;
	int err;							int err;

	might_sleep();							might_sleep();
	trace_ext3_mark_inode_dirty(inode, _RET_IP_);			trace_ext3_mark_inode_dirty(inode, _RET_IP_);
	err = ext3_reserve_inode_write(handle, inode, &iloc);		err = ext3_reserve_inode_write(handle, inode, &iloc);
	if (!err)							if (!err)
		err = ext3_mark_iloc_dirty(handle, inode, &il			err = ext3_mark_iloc_dirty(handle, inode, &il
	return err;							return err;
}								}

/*								/*
 * ext3_dirty_inode() is called from __mark_inode_dirty()	 * ext3_dirty_inode() is called from __mark_inode_dirty()
 *								 *
 * We're really interested in the case where a file is being 	 * We're really interested in the case where a file is being 
 * i_size has been changed by generic_commit_write() and we t	 * i_size has been changed by generic_commit_write() and we t
 * to include the updated inode in the current transaction.	 * to include the updated inode in the current transaction.
 *								 *
 * Also, dquot_alloc_space() will always dirty the inode when	 * Also, dquot_alloc_space() will always dirty the inode when
 * are allocated to the file.					 * are allocated to the file.
 *								 *
 * If the inode is marked synchronous, we don't honour that h	 * If the inode is marked synchronous, we don't honour that h
 * so would cause a commit on atime updates, which we don't b	 * so would cause a commit on atime updates, which we don't b
 * We handle synchronous inodes at the highest possible level	 * We handle synchronous inodes at the highest possible level
 */								 */
void ext3_dirty_inode(struct inode *inode, int flags)		void ext3_dirty_inode(struct inode *inode, int flags)
{								{
	handle_t *current_handle = ext3_journal_current_handl		handle_t *current_handle = ext3_journal_current_handl
	handle_t *handle;						handle_t *handle;

	handle = ext3_journal_start(inode, 2);				handle = ext3_journal_start(inode, 2);
	if (IS_ERR(handle))						if (IS_ERR(handle))
		goto out;							goto out;
	if (current_handle &&						if (current_handle &&
		current_handle->h_transaction != handle->h_tr			current_handle->h_transaction != handle->h_tr
		/* This task has a transaction open against a			/* This task has a transaction open against a
		printk(KERN_EMERG "%s: transactions do not ma			printk(KERN_EMERG "%s: transactions do not ma
		       __func__);						       __func__);
	} else {							} else {
		jbd_debug(5, "marking dirty.  outer handle=%p			jbd_debug(5, "marking dirty.  outer handle=%p
				current_handle);						current_handle);
		ext3_mark_inode_dirty(handle, inode);				ext3_mark_inode_dirty(handle, inode);
	}								}
	ext3_journal_stop(handle);					ext3_journal_stop(handle);
out:								out:
	return;								return;
}								}

#if 0								#if 0
/*								/*
 * Bind an inode's backing buffer_head into this transaction,	 * Bind an inode's backing buffer_head into this transaction,
 * it from being flushed to disk early.  Unlike			 * it from being flushed to disk early.  Unlike
 * ext3_reserve_inode_write, this leaves behind no bh referen	 * ext3_reserve_inode_write, this leaves behind no bh referen
 * returns no iloc structure, so the caller needs to repeat t	 * returns no iloc structure, so the caller needs to repeat t
 * lookup to mark the inode dirty later.			 * lookup to mark the inode dirty later.
 */								 */
static int ext3_pin_inode(handle_t *handle, struct inode *ino	static int ext3_pin_inode(handle_t *handle, struct inode *ino
{								{
	struct ext3_iloc iloc;						struct ext3_iloc iloc;

	int err = 0;							int err = 0;
	if (handle) {							if (handle) {
		err = ext3_get_inode_loc(inode, &iloc);				err = ext3_get_inode_loc(inode, &iloc);
		if (!err) {							if (!err) {
			BUFFER_TRACE(iloc.bh, "get_write_acce				BUFFER_TRACE(iloc.bh, "get_write_acce
			err = journal_get_write_access(handle				err = journal_get_write_access(handle
			if (!err)							if (!err)
				err = ext3_journal_dirty_meta					err = ext3_journal_dirty_meta
															
			brelse(iloc.bh);						brelse(iloc.bh);
		}								}
	}								}
	ext3_std_error(inode->i_sb, err);				ext3_std_error(inode->i_sb, err);
	return err;							return err;
}								}
#endif								#endif

int ext3_change_inode_journal_flag(struct inode *inode, int v	int ext3_change_inode_journal_flag(struct inode *inode, int v
{								{
	journal_t *journal;						journal_t *journal;
	handle_t *handle;						handle_t *handle;
	int err;							int err;

	/*								/*
	 * We have to be very careful here: changing a data b		 * We have to be very careful here: changing a data b
	 * journaling status dynamically is dangerous.  If we		 * journaling status dynamically is dangerous.  If we
	 * data block to the journal, change the status and t		 * data block to the journal, change the status and t
	 * that block, we risk forgetting to revoke the old l		 * that block, we risk forgetting to revoke the old l
	 * from the journal and so a subsequent replay can co		 * from the journal and so a subsequent replay can co
	 * So, first we make sure that the journal is empty a		 * So, first we make sure that the journal is empty a
	 * nobody is changing anything.					 * nobody is changing anything.
	 */								 */

	journal = EXT3_JOURNAL(inode);					journal = EXT3_JOURNAL(inode);
	if (is_journal_aborted(journal))				if (is_journal_aborted(journal))
		return -EROFS;							return -EROFS;

	journal_lock_updates(journal);					journal_lock_updates(journal);
	journal_flush(journal);						journal_flush(journal);

	/*								/*
	 * OK, there are no updates running now, and all cach		 * OK, there are no updates running now, and all cach
	 * synced to disk.  We are now in a completely consis		 * synced to disk.  We are now in a completely consis
	 * which doesn't have anything in the journal, and we		 * which doesn't have anything in the journal, and we
	 * no filesystem updates are running, so it is safe t		 * no filesystem updates are running, so it is safe t
	 * the inode's in-core data-journaling state flag now		 * the inode's in-core data-journaling state flag now
	 */								 */

	if (val)							if (val)
		EXT3_I(inode)->i_flags |= EXT3_JOURNAL_DATA_F			EXT3_I(inode)->i_flags |= EXT3_JOURNAL_DATA_F
	else								else
		EXT3_I(inode)->i_flags &= ~EXT3_JOURNAL_DATA_			EXT3_I(inode)->i_flags &= ~EXT3_JOURNAL_DATA_
	ext3_set_aops(inode);						ext3_set_aops(inode);

	journal_unlock_updates(journal);				journal_unlock_updates(journal);

	/* Finally we can mark the inode as dirty. */			/* Finally we can mark the inode as dirty. */

	handle = ext3_journal_start(inode, 1);				handle = ext3_journal_start(inode, 1);
	if (IS_ERR(handle))						if (IS_ERR(handle))
		return PTR_ERR(handle);						return PTR_ERR(handle);

	err = ext3_mark_inode_dirty(handle, inode);			err = ext3_mark_inode_dirty(handle, inode);
	handle->h_sync = 1;						handle->h_sync = 1;
	ext3_journal_stop(handle);					ext3_journal_stop(handle);
	ext3_std_error(inode->i_sb, err);				ext3_std_error(inode->i_sb, err);

	return err;							return err;
}								}
